unit u_objet_TStringListList;

interface


uses
     Classes,
     graphics;



const

     MaxListSize = Maxint div 16;
     SListIndexError = 'Index hors borne !';
     SListCapacityError = 'Erreur de capacite !';
     SListCountError = 'Erreur de Count';


type

     PStringListArray = ^TStringListArray;
     TStringListArray = array[0..MaxListSize - 1] of TStringList;


     PPointerList = ^TPointerList;
     TPointerList = array[0..MaxListSize - 1] of Pointer;

     TStringListList = class(TObject)

     private
          FList: PStringListArray;
          FCount: Integer;
          FCapacity: Integer;
     protected
          function Get(Index: Integer): TStringList;
          procedure Grow; virtual;
          procedure Put(Index: Integer; Item: TStringList);
          procedure SetCapacity(NewCapacity: Integer);
          procedure SetCount(NewCount: Integer);
     public
          destructor Destroy; override;
          function Add(Item: TStringList): Integer;
          procedure Clear;
          procedure Delete(Index: Integer);
          class procedure Error(const Msg: string; Data: Integer); virtual;
          procedure Exchange(Index1, Index2: Integer);
          function Expand: TStringListList;
          function First: TStringList;
          function IndexOf(Item: TStringList): Integer;
          procedure Insert(Index: Integer; Item: TStringList);
          function Last: TStringList;
          procedure Move(CurIndex, NewIndex: Integer);
          function Remove(Item: TStringList): Integer;
          property Capacity: Integer read FCapacity write SetCapacity;
          property Count: Integer read FCount write SetCount;
          property Items[Index: Integer]: TStringList read Get write Put; default;
          property List: PStringListArray read FList;
     end;


implementation

uses
     SysUtils,
     inifiles,
     u_objet_TRectangleElement,
     u_objet_TCircleElement,
     u_objet_TLineElement,
     u_objet_TTextElement;



destructor TStringListList.Destroy;
begin
     Clear;

     inherited Destroy;
end;

function TStringListList.Add(Item: TStringList): Integer;
begin
     Result := FCount;
     if Result = FCapacity then Grow;
     FList^[Result] := Item;
     Inc(FCount);
end;

procedure TStringListList.Clear;
var
     n: integer;
begin
     for n := Count downto 1 do
     begin
          try
               FList^[n - 1].Free;
          except
          end;
     end;

     SetCount(0);
     SetCapacity(0);
end;

procedure TStringListList.Delete(Index: Integer);
begin
     if (Index < 0) or (Index >= FCount) then Error(SListIndexError, Index);
     Dec(FCount);
     if Index < FCount then
     begin
          FList^[index].Free;
          System.Move(FList^[Index + 1], FList^[Index], (FCount - Index) * SizeOf(Pointer));
     end;
end;

class procedure TStringListList.Error(const Msg: string; Data: Integer);

     function ReturnAddr: Pointer;
     asm
          MOV     EAX,[EBP+4]
     end;

begin
     raise EListError.CreateFmt(Msg, [Data])at ReturnAddr;
end;

procedure TStringListList.Exchange(Index1, Index2: Integer);
var
     Item: TStringList;
begin
     if (Index1 < 0) or (Index1 >= FCount) then Error(SListIndexError, Index1);
     if (Index2 < 0) or (Index2 >= FCount) then Error(SListIndexError, Index2);
     Item := FList^[Index1];
     FList^[Index1] := FList^[Index2];
     FList^[Index2] := Item;
end;

function TStringListList.Expand: TStringListList;
begin
     if FCount = FCapacity then Grow;
     Result := Self;
end;

function TStringListList.First: TStringList;
begin
     Result := Get(0);
end;

function TStringListList.Get(Index: Integer): TStringList;
begin
     if (Index < 0) or (Index >= FCount) then Error(SListIndexError, Index);
     Result := FList^[Index];
end;

procedure TStringListList.Grow;
var
     Delta: Integer;
begin
     if FCapacity > 64 then Delta := FCapacity div 4 else
          if FCapacity > 8 then Delta := 16 else
          Delta := 4;
     SetCapacity(FCapacity + Delta);
end;

function TStringListList.IndexOf(Item: TStringList): Integer;
begin
     Result := 0;
     while (Result < FCount) and (FList^[Result] <> Item) do Inc(Result);
     if Result = FCount then Result := -1;
end;

procedure TStringListList.Insert(Index: Integer; Item: TStringList);
begin
     if (Index < 0) or (Index > FCount) then Error(SListIndexError, Index);
     if FCount = FCapacity then Grow;
     if Index < FCount then
          System.Move(FList^[Index], FList^[Index + 1],
               (FCount - Index) * SizeOf(Pointer));
     FList^[Index] := Item;
     Inc(FCount);
end;

function TStringListList.Last: TStringList;
begin
     Result := Get(FCount - 1);
end;

procedure TStringListList.Move(CurIndex, NewIndex: Integer);
var
     Item: TStringList;
begin
     if CurIndex <> NewIndex then
     begin
          if (NewIndex < 0) or (NewIndex >= FCount) then Error(SListIndexError, NewIndex);
          Item := Get(CurIndex);
          Delete(CurIndex);
          Insert(NewIndex, Item);
     end;
end;

procedure TStringListList.Put(Index: Integer; Item: TStringList);
begin
     if (Index < 0) or (Index >= FCount) then Error(SListIndexError, Index);
     FList^[Index] := Item;
end;

function TStringListList.Remove(Item: TStringList): Integer;
begin
     Result := IndexOf(Item);
     if Result <> -1 then Delete(Result);
end;


procedure TStringListList.SetCapacity(NewCapacity: Integer);
begin
     if (NewCapacity < FCount) or (NewCapacity > MaxListSize) then
          Error(SListCapacityError, NewCapacity);
     if NewCapacity <> FCapacity then
     begin
          ReallocMem(FList, NewCapacity * SizeOf(Pointer));
          FCapacity := NewCapacity;
     end;
end;

procedure TStringListList.SetCount(NewCount: Integer);
begin
     if (NewCount < 0) or (NewCount > MaxListSize) then
          Error(SListCountError, NewCount);
     if NewCount > FCapacity then SetCapacity(NewCount);
     if NewCount > FCount then
          FillChar(FList^[FCount], (NewCount - FCount) * SizeOf(Pointer), 0);
     FCount := NewCount;
end;




end.

