unit u_comp_TSuperDBImage;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, dbctrls,
  ExtCtrls, DB, GraphicEx, Clipbrd, extdlgs, JPEG, menus;

type

  TSuperDBImage = class(TCustomPanel)
  private
    //a: TPanel;

    MS: TMemoryStream;
    fImage: TImage;
    J1: TJPEGImage;
    BClipBoard: Boolean;
    bStretch: boolean;
    FDatalink: TFieldDatalink;
    fPathFileNameImage: string;
    fTauxCompression: integer;
    fThePopup: TPopupMenu;
    fStretch: Boolean;


    {Mthodes pour les donnes --------------------------------------------------------------------}
    function GetDataField: string;
    function GetDataSource: TDataSource;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure DataChange(Sender: TObject);
    procedure UpdateData(Sender: TObject);
    function up_Error: boolean;
    procedure SetThePopup(const Value: TPopupMenu);
    procedure SetStretch(const Value: Boolean);

  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure Loaded; override;

  public
    property Image: TImage read fImage;

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure DoLoadImage(aPathFileName: string);
    procedure DoDeleteImage;
    procedure DoCopier;
    procedure DoColler;

  published
    property Align;
    property Anchors;


    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderWidth;
    property BorderStyle;
    property Caption;
    property Color;
    property Ctl3D;
    property Enabled;
    property TabStop;
    property Visible;

    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property TauxCompression: integer read fTauxCompression write fTauxCompression;
    property ThePopup: TPopupMenu read fThePopup write SetThePopup;
    property Stretch: Boolean read fStretch write SetStretch;
  end;


procedure Register;

implementation


constructor TSuperDBImage.Create(AOwner: TComponent);
begin
  inherited create(AOwner);

  Caption := '';

  // --- Cration et init du DataLink
  FDatalink := TFieldDataLink.Create;
  FDatalink.OnDataChange := DataChange;
  FDatalink.OnUpdateData := UpdateData;

  // --- Cration et init de l'image
  fImage := TImage.Create(Self);
  with fImage do begin
      Parent := Self;
      Align := alClient;
      Stretch := False;
      Center := True;
    end;

  J1 := TJPEGImage.Create;
  with J1 do begin
      PixelFormat := jf24Bit;
      Scale := jsFullSize;
      Grayscale := False;
      Performance := jpBestQuality;
      ProgressiveDisplay := False;
      ProgressiveEncoding := True;
    end;
  Stretch := false;
end;

destructor TSuperDBImage.Destroy;
begin
  FDataLink.free;
  fImage.Picture := nil;
  fImage.Free;
  J1.Free;
  inherited destroy;
end;

function TSuperDBImage.GetDataField: string;
begin
  result := FDataLink.FieldName;
end;

function TSuperDBImage.GetDataSource: TDataSource;
begin
  result := FDataLink.DataSource;
end;

procedure TSuperDBImage.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

procedure TSuperDBImage.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
end;

procedure TSuperDBImage.DataChange(Sender: TObject);
begin
  if FDataLink.Field = nil then
    fImage.Picture := nil
  else
    begin
      MS := TMemoryStream.Create;

      try
        TBlobField(FDataLink.Field).SaveToStream(MS);
        MS.Seek(soFromBeginning, 0);

        J1.LoadFromStream(MS);

        if MS.Size > 0 then
          begin
            if bStretch then
              if (J1.Height > fImage.Height) or (J1.Width > fImage.Width) then
                fImage.Stretch := True
              else
                fImage.Stretch := False;

            fImage.Picture.Assign(J1);
          end
        else
          fImage.Picture := nil;

      finally
        MS.Free;
      end;
    end;
end;

procedure TSuperDBImage.UpdateData(Sender: TObject);
var
  t_Image: TPicture;
  ms_ImageTempo: TMemoryStream;
  s: string;
begin
  inherited;

  if up_Error then Exit;

  {Creation de l'image temporaire ---------------------------------------------------------}
  ms_ImageTempo := TMemoryStream.Create;
  t_Image := TPicture.Create;

  if not BClipBoard then
    begin
      t_Image.LoadFromFile(fPathFileNameImage);
      ClipBoard.Assign(t_Image);
      fImage.Picture.LoadFromClipboardFormat(CF_BITMAP, Clipboard.GetAsHandle(CF_BITMAP), 0);

      with TJPEGImage.Create do
        begin
          try
            Assign(t_Image.Graphic);

            s := UpperCase(ExtractFileExt(ExtractFileName(fPathFileNameImage)));
            if (s <> '.JPG') and (s <> '.JPEG') then
              begin
                CompressionQuality := fTauxCompression;
                Compress;
                SaveToStream(ms_ImageTempo);

                {On stocke l'image convertie en JPEG dans le champ de la table---------------}
                TBlobField(FDataLink.Field as TBlobField).LoadFromStream(ms_ImageTempo);
              end
            else
              {On stocke l'image JPEG dans le champ de la table-------------------------------}
              TBlobField(FDataLink.Field as TBlobField).LoadFromFile(fPathFileNameImage);

          finally
            Free;
          end
        end;

    end
  else
    begin
      t_Image.LoadFromClipboardFormat(CF_BITMAP, Clipboard.GetAsHandle(CF_BITMAP), 0);
      fImage.Picture.LoadFromClipboardFormat(CF_BITMAP, Clipboard.GetAsHandle(CF_BITMAP), 0);

      with TJPEGImage.Create do
        begin
          Assign(t_Image.Graphic);
          CompressionQuality := fTauxCompression;
          Compress;
          SaveToStream(ms_ImageTempo);

          {On stocke l'image convertie en JPEG dans le champ de la table---------------}
          TBlobField(FDataLink.Field as TBlobField).LoadFromStream(ms_ImageTempo);
        end;

    end;

  {On dtruit l'image tempo--------------------------------------------------------------}
  t_Image.free;
  ms_ImageTempo.free;

end;





function TSuperDBImage.up_Error: Boolean;
var b_Retour: boolean;
begin
  b_Retour := False;

  if FDataLink.DataSource = nil then begin
      ShowMessage('C''est un composant orient donne. Vous n''avez pas renseign' +
        ' la propit DataSource');
      b_Retour := True;
    end;

  if b_retour = False then
    if FDataLink.Field = nil then begin
        ShowMessage('C''est un composant orient donne. Vous n''avez pas renseign' +
          ' la propit DataField');
        b_Retour := True;
      end;

  Result := b_Retour;
end;







procedure TSuperDBImage.DoLoadImage(aPathFileName: string);
begin
  fPathFileNameImage := aPathFileName;
  BClipBoard := False;
  UpdateData(self);
end;

procedure TSuperDBImage.DoDeleteImage;
begin
  //	FDataLink.DataSource.DataSet.Delete;
  TBlobField(FDataLink.Field as TBlobField).Clear;
  FDataLink.Modified;
  DataChange(Self);
end;


procedure Register;
begin
  RegisterComponents('EvaluatePak', [TSuperDBImage]);
end;


procedure TSuperDBImage.DoColler;
begin
  BClipBoard := True;
  fPathFileNameImage := '';
  UpdateData(Self);
end;

procedure TSuperDBImage.DoCopier;
begin
  if fImage.Picture <> nil then Clipboard.Assign(fImage.Picture)
end;

procedure TSuperDBImage.SetThePopup(const Value: TPopupMenu);
begin
  fThePopup := Value;
  fImage.PopupMenu := Value;
end;


procedure TSuperDBImage.Loaded;
begin
  inherited;

  if fImage <> nil then
    begin
      fImage.PopupMenu := fThePopup;
      fImage.Cursor := Cursor;
    end;


end;

procedure TSuperDBImage.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);

  if (Operation = opRemove) then
    begin
      if (AComponent = fThePopup) then ThePopup := nil;
    end;
end;

procedure TSuperDBImage.SetStretch(const Value: Boolean);
begin
  fStretch := Value;
  fImage.Stretch := Value;
end;

end.

