unit u_comp_TFlatDBGrid;

interface

uses
     Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
     Grids, DBGrids, db;

type
     Tcellstyle = (csRaised, csLowered, csNone);

     TDirectionOrderBy = (obDown, obUp);

     TOnMyMouseEvent = procedure(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer) of object;

     TOnUserClickOnTitleEvent = procedure(Sender: TObject; FieldName: string; Direction: TDirectionOrderBy) of object;

     TFlatDBGrid = class(TDBGrid)

     private
          fWeAreOrderingColumns: boolean;

          fcellstyle: tcellstyle;
          fOrderBy: string;
          fCanOrderByClickOnTitle: boolean;
          fOnOrderByChange: TNotifyEvent;
          fDirectionOrderBy: TDirectionOrderBy;
          fOnMyMouseDown: TOnMyMouseEvent;
          fDesignColumnCanSave: boolean;
          fDesignColumnCanLoad: boolean;
          fDesignColumnPathFromApplication: boolean;
          fDesignColumnFileName: string;
          fDesignColumnSection: string;
          fOnUserClickOnTitle: TOnUserClickOnTitleEvent;

          procedure setcellstyle(const Value: tcellstyle);


     protected
          procedure DrawCell(ACol, ARow: Longint; ARect: TRect; AState: TGridDrawState); override;
          procedure TitleClick(Column: TColumn); override;
          procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;

     public
          constructor Create(AOwner: TComponent); override;
          destructor Destroy; override;

          procedure SaveColumnDesign;
          procedure LoadColumnDesign;

          function GetSqlLineOrderBy: string;

     published
          property Cellstyle: tcellstyle read fcellstyle write setcellstyle;
          property OrderBy: string read fOrderBy write fOrderBy;
          property DirectionOrderBy: TDirectionOrderBy read fDirectionOrderBy write fDirectionOrderBy;
          property CanOrderByClickOnTitle: boolean read fCanOrderByClickOnTitle write fCanOrderByClickOnTitle;
          property OnOrderByChange: TNotifyEvent read fOnOrderByChange write fOnOrderByChange;

          property OnMyMouseDown: TOnMyMouseEvent read fOnMyMouseDown write fOnMyMouseDown;

          property OnUserClickOnTitle: TOnUserClickOnTitleEvent read fOnUserClickOnTitle write fOnUserClickOnTitle;

          property DesignColumnCanLoad: boolean read fDesignColumnCanLoad write fDesignColumnCanLoad;
          property DesignColumnCanSave: boolean read fDesignColumnCanSave write fDesignColumnCanSave;
          property DesignColumnFileName: string read fDesignColumnFileName write fDesignColumnFileName;
          property DesignColumnSection: string read fDesignColumnSection write fDesignColumnSection;
          property DesignColumnPathFromApplication: boolean read fDesignColumnPathFromApplication write fDesignColumnPathFromApplication;
     end;

procedure Register;

implementation

uses
     inifiles;


procedure Register;
begin
     RegisterComponents('FlatStyle', [TFlatDBGrid]);
end;



{ TFlatDBGrid }


constructor TFlatDBGrid.Create(AOwner: TComponent);
begin
     inherited Create(AOwner);

     cellstyle := csNone;
     fCanOrderByClickOnTitle := False;
     fOrderBy := '';
     fDirectionOrderBy := obDown;

     fDesignColumnCanSave := false;
     fDesignColumnCanLoad := false;
     fDesignColumnPathFromApplication := true;
     fDesignColumnFileName := 'config\dbgrids.ini';
     fDesignColumnSection := '';
end;



destructor TFlatDBGrid.Destroy;
begin


     inherited;
end;

procedure TFlatDBGrid.DrawCell(ACol, ARow: Longint; ARect: TRect; AState: TGridDrawState);

//dessin de la cellule d'entte pour l'ordre du trie.
     procedure DrawTriangleOrdreTrie(x, y: integer; aDirection: TDirectionOrderBy);
     begin

          if aDirection = obUp then
          begin
               Canvas.Pen.Color := clBtnShadow;
               Canvas.MoveTo(x, y + 6);
               Canvas.lineto(x + 3, y);
               Canvas.Pen.Color := clWindow;
               Canvas.lineTo(x + 4, y);
               Canvas.lineto(x + 7, y + 6);
               Canvas.lineto(x + 1, y + 6);
          end
          else
          begin
               Canvas.Pen.Color := clBtnShadow;
               Canvas.MoveTo(x + 6, y);
               Canvas.lineto(x, y);
               Canvas.LineTo(x + 3, y + 6);
               Canvas.Pen.Color := clWindow;
               Canvas.lineTo(x + 4, y + 6);
               Canvas.lineto(x + 7, y);
          end;
     end;


     procedure drawrect(l, t, r, b: integer; p1, p2: Tcolor);
     begin
          with ARect, canvas do
          begin
               Pen.Color := p1;
               PolyLine([Point(l, b), Point(l, t), Point(r, t)]);
               Pen.Color := p2;
               PolyLine([Point(l, b), Point(r, b), Point(r, t)]);
          end;
     end;

begin
     inherited;



     if gdFixed in AState then
     begin
          with ARect do
          begin
               case fcellstyle of
                    csRaised: drawrect(left, top, right, bottom - 1, clWindow, clBtnShadow);
                    csLowered: drawrect(left, top, right, bottom, clBtnShadow, clWindow);
               end;
          end;

          if fCanOrderByClickOnTitle then
          begin
               if (ACol >= 0) and (ACol < Columns.count) then
               begin
                    //	if dgIndicator	in Options then inc(ACol);

                    if Columns[ACol].FieldName = fOrderBy then
                    begin
                         //Quel est la direction du trie ?
                         with ARect, canvas do
                         begin
                              if fWeAreOrderingColumns
                                   then Brush.Color := $004080FF;

                              FillRect(ARect);

                              case fcellstyle of
                                   csRaised: drawrect(left, top, right, bottom - 1, clWindow, clBtnShadow);
                                   csLowered: drawrect(left, top, right, bottom, clBtnShadow, clWindow);
                              end;

                              DrawTriangleOrdreTrie(ARect.Left + 3, ARect.Top + 5, fDirectionOrderBy);

                              ARect.Left := ARect.Left + 15;
                              ARect.Top := 2;

                              drawtext(canvas.handle, pchar(Columns[ACol].Title.Caption), -1, ARect, DT_WORDBREAK or DT_NOPREFIX);
                         end;
                    end;
               end;
          end;
     end;
end;


procedure TFlatDBGrid.MouseDown(Button: TMouseButton; Shift: TShiftState;
     X, Y: Integer);
begin
     inherited;
     if assigned(fOnMyMouseDown) then fOnMyMouseDown(self, Button, Shift, X, Y);
end;


procedure TFlatDBGrid.setcellstyle(const Value: tcellstyle);
begin
     if (fcellstyle <> value) then fcellstyle := value;
     if (csDesigning in componentstate) then invalidate;
end;

procedure TFlatDBGrid.TitleClick(Column: TColumn);
var
     fOldCursor: TCursor;
begin
     if assigned(DataSource) then
     begin
          if assigned(DataSource.DataSet) then
          begin
               if fCanOrderByClickOnTitle then
               begin
                    //Si la colonne de trie change
                    if (fOrderBy <> Column.FieldName) then
                    begin
                         fDirectionOrderBy := obDown;
                         fOrderBy := Column.FieldName;
                    end
                    else
                         //c'est la mme colonne
                    begin
                         case fDirectionOrderBy of
                              obDown: fDirectionOrderBy := obUp;
                              obUp: fDirectionOrderBy := obDown;
                         end;
                    end;

                    //On rafraichit la grille
                    fWeAreOrderingColumns := true;
                    fOldCursor := Cursor;
                    try
                         Repaint;


                         Cursor := crHourGlass;

                         if assigned(fOnUserClickOnTitle) then fOnUserClickOnTitle(self, fOrderBy, fDirectionOrderBy);

                    finally
                         Cursor := fOldCursor;
                         fWeAreOrderingColumns := false;
                    end;

                    Repaint;
               end;
          end;
     end;

     inherited;
end;

procedure TFlatDBGrid.SaveColumnDesign;
var
     fini: TIniFile;
     n: integer;
begin
     if fDesignColumnCanSave then
     begin
          if fDesignColumnSection <> '' then
          begin
               if fDesignColumnPathFromApplication
                    then fIni := TIniFile.Create(extractFilePath(application.exeName) + fDesignColumnFileName)
               else fIni := TIniFile.Create(fDesignColumnFileName);

               try
                    //Destruction de la section
                    try
                         fIni.EraseSection(fDesignColumnSection);
                    except
                    end;

                    //Ecriture des infos de Design//Longueur des colonnes
                    for n := 0 to Columns.Count - 1 do
                    begin
                         if Columns[n].Visible then
                         begin
                              fIni.WriteInteger(fDesignColumnSection, 'Size_' + Columns[n].FieldName, Columns[n].Width);
                         end;
                    end;

                    {
                    //Ecriture des infos de Design//Position des colonnes
                    for n := 0 to Columns.Count - 1 do
                    begin
                     if Columns[n].Visible then
     begin
                      fIni.WriteInteger(fDesignColumnSection, 'Pos_' + Columns[n].FieldName, Columns[n].Index);
                     end;
                    end;
                     }

    {
    //Tri selon la column
    fIni.WriteString(fDesignColumnSection, 'OrderBy', OrderBy);
    //Sens du trie
    fIni.WriteBool(fDesignColumnSection, 'Direction', fDirectionOrderBy = obDown);
     }


                    fIni.UpdateFile;
               finally
                    fIni.Free;
               end;
          end;
     end;
end;




procedure TFlatDBGrid.LoadColumnDesign;
var
     fini: TIniFile;
     n, k: integer;
     //   FromIndex, ToIndex: Longint;
     //	IDColumns: TIntegerList;
begin
     if fDesignColumnCanLoad then
     begin
          if fDesignColumnSection <> '' then
          begin
               if fDesignColumnPathFromApplication
                    then fIni := TIniFile.Create(extractFilePath(application.exeName) + fDesignColumnFileName)
               else fIni := TIniFile.Create(fDesignColumnFileName);

               try

                    //Lecture des infos de Design//Longueur des colonnes
                    for n := 0 to Columns.Count - 1 do
                    begin
                         if Columns[n].Visible then
                         begin
                              k := fIni.ReadInteger(fDesignColumnSection, 'Size_' + Columns[n].FieldName, 0);
                              if k > 0 then Columns[n].Width := k;
                         end;
                    end;

                    {
                    //Lecture des infos de Design//Position des colonnes
                    IDColumns := TIntegerList.create;
                    try
     //On fait une liste des ID des column, pour pouvoir les reprer, mme quand on les bouge
                     for n := 0 to Columns.Count - 1 do IDColumns.Add(Columns[n].ID);

                     //On replace les columns
                     for n := 0 to IDColumns.Count - 1 do
                     begin
                      try
                       //On recherche la colonne qui a l'ID[n]
                       k := -1;
       for w := 0 to Columns.Count - 1 do
                       begin
                        if Columns[w].ID = IDColumns[n] then
                        begin
                         k := w;
                         break;
                        end;
                                                   end;

                       //si on a trouv la colonne
                       if k <> -1 then
                                                   begin
                                                        w := fIni.ReadInteger(fDesignColumnSection, 'Pos_' + Columns[k].FieldName, -1);
                                                        if w <> -1 then
                                                        begin
                                                             FromIndex := RawToDataColumn(Columns[k].Index);
             ToIndex := RawToDataColumn(w);
             Columns[FromIndex].Index := ToIndex;
            end;
           end;
          except
          end;
         end;


    finally
     IDColumns.Free;
    end;
        }


      {

        //Tri selon la column
    OrderBy := fIni.ReadString(fDesignColumnSection, 'OrderBy', '');
    //Sens du trie
    if fIni.ReadBool(fDesignColumnSection, 'Direction', true) = true
     then fDirectionOrderBy := obDown
    else fDirectionOrderBy := obUp;
       }

                    fIni.UpdateFile;


               finally
                    fIni.Free;
               end;
          end;
     end;
end;



function TFlatDBGrid.GetSqlLineOrderBy: string;
begin
     result := '';
     if fCanOrderByClickOnTitle then
     begin
          if fOrderBy <> '' then result := 'order by ' + fOrderBy;
          if fDirectionOrderBy = obUp then result := result + ' DESC';
     end;
end;

end.

