unit CompSuperForm;

interface

uses
  Classes,
  Controls,
  extctrls,
  buttons,
  Forms;


type
  TFormMode = (sfmEdit, sfmConsult, sfmNew, sfmUnknown);

  TIncrustMode = (aicCenter, aicTopLeft, aicAllClient, aicAllHeight, aicCenterTop, aicAllWidth);

  TNotifyReceiveMessageEvent = procedure(sender: TObject; theMessage: string) of object;

  TSuperForm = class(TForm)

  private
    fFormMode: TFormMode;
    fOnFillFields: TNotifyEvent;
    fOnFillProperties: TNotifyEvent;
    fOnRefreshControls: TNotifyEvent;
    fAutoFillFieldsWhenShow: boolean;
    fIncrustMode: TIncrustMode;
    fAdaptParentSize: boolean;
    fOnReceiveMessage: TNotifyReceiveMessageEvent;
    fPanelParent: TPanel;
    fFixedClientHeight: integer;
    fFixedClientWidth: integer;
    //		fCloseAction: TCloseAction;
    fAutoFreeOnClose: boolean;
    fShowBevel: boolean;
    fOnFirstActivate: TNotifyEvent;
    fAlreadyPassedInFirstActivate: boolean;
    fAlreadyPassedInFirstShow: boolean;
    fOnShowFirstTime: TNotifyEvent;
    fOnPrepare: TNotifyEvent;
    fDescription: string;

    procedure Incrust(ParentControl: TPanel);
    procedure SetShowBevel(value: boolean);
    procedure SetcsDisplayDragImage;

  protected
    procedure CallEventFillFields; dynamic;
    procedure CallEventFillProperties; dynamic;
    procedure CallEventRefreshControls; dynamic;
    procedure CallEventFirstActivate; dynamic;
    procedure CallEventReceiveMessage(theMessage: string); dynamic;
    procedure CallEventShowFirstTime; dynamic;
    procedure CallEventPrepare; dynamic;
    procedure DoShow; override;
    procedure DoClose(var Action: TCloseAction); override;

    procedure Paint; override;
    procedure Activate; override;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure DoFillFields;
    procedure DoFillProperties;
    procedure ShowIncrust(ParentControl: TPanel);
    //		function  ShowIncrustModal(ParentControl: TPanel): integer;
    procedure HideIncrust;
    procedure AlignFormInParent(sender: TObject);
    procedure DoRefreshControls;
    procedure DoSendMessage(theComponent: TObject; theMessage: string); virtual;
    function ShowModal: Integer; override;
    procedure DoPrepare;

  published
    property FormMode: TFormMode read fFormMode write fFormMode;
    property OnFirstActivate: TNotifyEvent read fOnFirstActivate write fOnFirstActivate;
    property OnFillFields: TNotifyEvent read fOnFillFields write fOnFillFields;
    property OnFillProperties: TNotifyEvent read fOnFillProperties write fOnFillProperties;
    property OnRefreshControls: TNotifyEvent read fOnRefreshControls write fOnRefreshControls;
    property AutoFillFieldsWhenShow: boolean read fAutoFillFieldsWhenShow write fAutoFillFieldsWhenShow;
    property IncrustMode: TIncrustMode read fIncrustMode write fIncrustMode;
    property AdaptParentSize: boolean read fAdaptParentSize write fAdaptParentSize;
    property OnReceiveMessage: TNotifyReceiveMessageEvent read fOnReceiveMessage write fOnReceiveMessage;
    property PanelParent: TPanel read fPanelParent;
    property AutoFreeOnClose: boolean read fAutoFreeOnClose write fAutoFreeOnClose;
    property ShowBevel: boolean read fShowBevel write SetShowBevel;
    property AlreadyPassedInFirstActivate: boolean read fAlreadyPassedInFirstActivate write fAlreadyPassedInFirstActivate;
    property OnShowFirstTime: TNotifyEvent read fOnShowFirstTime write fOnShowFirstTime;
    property OnPrepare: TNotifyEvent read fOnPrepare write fOnPrepare;
    property Description: string read fDescription write fDescription;

  end;


implementation

constructor TSuperForm.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

 //ControlStyle := ControlStyle - [csSetCaption, csAcceptsControls] + [csDesignInteractive];

  fFormMode := sfmEdit;
  //    fAutoFillFieldsWhenShow := true;
      //	fIncrustMode:=aicCenter;
  fAdaptParentSize := true;
  fPanelParent := nil;
  fFixedClientHeight := ClientHeight;
  fFixedClientWidth := ClientWidth;
  //	fCloseAction:=caHide;
  fAutoFreeOnClose := false;
  //	fShowBevel:=false;
  fAlreadyPassedInFirstActivate := false;
  fAlreadyPassedInFirstShow := false;
  //	fDescription:='';
end;

destructor TSuperForm.Destroy;
begin
  if assigned(fPanelParent) then
    if assigned(fPanelParent.OnResize) then
      begin
        //if fPanelParent.OnResize=OnResize then
      //		fPanelParent.onResize:=Nil;
      end;

  inherited;
end;


procedure TSuperForm.CallEventFillFields;
begin
  if assigned(fOnFillFields) then fOnFillFields(self);
end;


procedure TSuperForm.DoFillFields;
begin
  CallEventFillFields;
end;


procedure TSuperForm.CallEventFillProperties;
begin
  if assigned(fOnFillProperties) then fOnFillProperties(self);
end;

procedure TSuperForm.CallEventReceiveMessage(theMessage: string);
begin
  if assigned(fOnReceiveMessage) then fOnReceiveMessage(self, theMessage);
end;


procedure TSuperForm.DoFillProperties;
begin
  CallEventFillProperties;
end;


procedure TSuperForm.CallEventRefreshControls;
begin
  if assigned(fOnRefreshControls) then fOnRefreshControls(self);
end;


procedure TSuperForm.DoRefreshControls;
begin
  CallEventRefreshControls;
end;




procedure TSuperForm.DoShow;
begin
  SetcsDisplayDragImage;

  if (not fAlreadyPassedInFirstShow) then
    begin
      CallEventShowFirstTime; ;
      fAlreadyPassedInFirstShow := true;
    end;

  inherited;

  if fAutoFillFieldsWhenShow then CallEventFillFields;
end;


procedure TSuperForm.Incrust(ParentControl: TPanel);
begin
  {
  case fIncrustMode of

   aicTopLeft:
    begin
    end;

   aicAllClient:
    begin
     BorderStyle:=bsNone;
    end;

   aicCenter:
begin
    end;

   aicAllHeight:
    begin
 BorderStyle:=bsSizeable;
    end;
  end;
   }
  BorderStyle := bsNone;


  parent := ParentControl;
  fPanelParent := ParentControl;

  {
  case fIncrustMode of
   aicCenter,aicTopLeft: Align:=alNone;
   aicAllClient: Align:=alClient;
end;
  }

  ClientHeight := fFixedClientHeight;
  ClientWidth := fFixedClientWidth;

  AlignFormInParent(nil);

  fPanelParent.OnResize := AlignFormInParent;
end;

procedure TSuperForm.ShowIncrust(ParentControl: TPanel);
begin
  Incrust(ParentControl);

  Show;

  DoShow;
end;

{
function  TSuperForm.ShowIncrustModal(ParentControl: TPanel): integer;
begin
 Incrust(ParentControl);
//	fCloseAction:=caFree;
 Show;
end;
}

procedure TSuperForm.HideIncrust;
begin
  fPanelParent.OnResize := nil;
  Hide;
end;


procedure TSuperForm.AlignFormInParent(sender: TObject);
begin

  case fIncrustMode of

    aicTopLeft:
      begin
        left := 0;
        top := 0;
      end;

    aicAllClient:
      begin
        left := 0;
        top := 0;
        width := fPanelParent.ClientWidth;
        height := fPanelParent.ClientHeight;


      end;

    aicCenter:
      begin
        left := (fPanelParent.ClientWidth - ClientWidth) div 2;
        top := (fPanelParent.ClientHeight - ClientHeight) div 2;

      end;

    aicCenterTop:
      begin
        top := 0;
        left := (fPanelParent.ClientWidth - ClientWidth) div 2;
      end;

    aicAllHeight:
      begin
        left := (fPanelParent.ClientWidth - ClientWidth) div 2;
        top := 0;
        ClientWidth := fFixedClientWidth;
        //ClientHeight := fFixedClientHeight;
        height := fPanelParent.Height - 10;
      end;

    aicAllWidth:
      begin
        left := 0;
        top := (fPanelParent.ClientHeight - ClientHeight) div 2;
        width := fPanelParent.ClientWidth;
        ClientHeight := fFixedClientHeight;
        //height := fPanelParent.Height - 10;
      end;

  end;
end;


procedure TSuperForm.DoSendMessage(theComponent: TObject; theMessage: string);
begin
  if (theComponent is TSuperForm) then
    begin
      (theComponent as TSuperForm).CallEventReceiveMessage(theMessage);
    end;
end;


procedure TSuperForm.DoClose(var Action: TCloseAction);
begin
  if fAutoFreeOnClose then Action := caFree;
  inherited DoClose(Action);
  //	if Assigned(OnClose) then FOnClose(Self, Action);
end;


procedure TSuperForm.SetShowBevel(value: boolean);
begin
  fShowBevel := value;
  repaint;
end;

procedure TSuperForm.Paint;
begin
  inherited paint;
  if fShowBevel then DrawButtonFace(Canvas, ClientRect, 1, bsNew, True, False, False);
end;


procedure TSuperForm.Activate;
begin
  inherited Activate;

  if (not fAlreadyPassedInFirstActivate) then
    begin
      CallEventFirstActivate;
      fAlreadyPassedInFirstActivate := true;
    end;
end;

procedure TSuperForm.CallEventFirstActivate;
begin
  if assigned(fOnFirstActivate) then fOnFirstActivate(self);
end;

procedure TSuperForm.CallEventShowFirstTime;
begin
  if assigned(fOnShowFirstTime) then fOnShowFirstTime(self);
end;

function TSuperForm.ShowModal: Integer;
begin
  if (not fAlreadyPassedInFirstShow) then
    begin
      CallEventShowFirstTime; ;
      fAlreadyPassedInFirstShow := true;
    end;

  if fAutoFillFieldsWhenShow then CallEventFillFields;

  result := inherited ShowModal;
end;

procedure TSuperForm.DoPrepare;
begin
  CallEventPrepare;
end;

procedure TSuperForm.CallEventPrepare;
begin
  if assigned(fOnPrepare) then fOnPrepare(self);
end;


procedure TSuperForm.SetcsDisplayDragImage;
var
  n: integer;
begin
  ControlStyle := ControlStyle + [csDisplayDragImage];
  for n := 0 to ComponentCount - 1 do
    if (Components[n] is TControl)
      then TControl(Components[n]).ControlStyle := TControl(Components[n]).ControlStyle + [csDisplayDragImage];
end;

end.


