{-----------------------------------------------------------------------}
{                                                                       }
{           Subprogram Name:                                            }
{           Purpose:          Ancestrologie                             }
{           Source Language:  Francais                                  }
{           Auteur:                                                     }
{           Philippe Cazaux-Moutou                                      }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Description:                                                }
{           Ancestrologie est un Logiciel Libre                         }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Revision History                                            }
{           v#    ,Date       ,Author Name            ,Description      }
{                                                                       }
{-----------------------------------------------------------------------}
unit U_Form_Cartographie;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, TGClasses, Globe4, Db, TGLayerDS, TGObjects, GLobeUtils, TGPresenters,
  TGsysUtils, TGLYRMapper, Printers,
  ExtCtrls, IBQuery, dxCntner, dxExEdtr, dxEdLib, u_comp_TNakedSpeedButton,
  u_comp_TGoodBtn, DynamicSkinForm;
type
  TFCartoGraphie = class(TForm)
    Globe: TGlobe4;
    Panel5: TPanel;
    Bevel1: TBevel;
    Bevel2: TBevel;
    btnPrint: TdxCheckEdit;
    pnlHint: TPanel;
    btnFrance: TGoodBtn;
    btnMonde: TGoodBtn;
    btnEurope: TGoodBtn;
    btnNormal: TGoodBtn;
    btnTransparent: TGoodBtn;
    btnSphere: TGoodBtn;
    btnCarte: TGoodBtn;
    btnMerca: TGoodBtn;
    Bevel3: TBevel;
    btnZoomP: TGoodBtn;
    btnZoomM: TGoodBtn;
    Bevel4: TBevel;
    btnJour: TGoodBtn;
    btnRotation: TGoodBtn;
    Timer1: TTimer;
    btnVilles: TGoodBtn;
    btnTexture: TGoodBtn;
    btnAjout: TGoodBtn;
    OpenDialog: TOpenDialog;
    spDynamicSkinForm1: TspDynamicSkinForm;
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnConfigClick(Sender: TObject);
    procedure btnPrintChange(Sender: TObject);
    procedure GlobeMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure GlobePaint(Sender: TObject);
    procedure GlobeRender(Sender: TObject);
    procedure rbPDFChange(Sender: TObject);
    procedure rbRTFChange(Sender: TObject);
    procedure rbHtmlChange(Sender: TObject);
    procedure DoInit(Sender: TObject);
    procedure DoTexture(Sender: TObject);
    procedure DoProjection(Sender: TObject);
    procedure DoZoom(Sender: TObject);
    procedure btnJourClick(Sender: TObject);
    procedure btnRotationClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure btnVillesClick(Sender: TObject);
    procedure btnTextureClick(Sender: TObject);
    procedure btnAjoutClick(Sender: TObject);
  private
    { Dclarations prives }
    //procedure DoInit;
    procedure DoInitBase;
    procedure DoAfterInit;
  public
    { Dclarations publiques }
  end;

var
  FCartoGraphie: TFCartoGraphie;
  sName, sLon, sLat: string;
  idx: integer;
  aLayer: TGlobeLayer;
  iLong, iLat: integer;
  aPresenter: TPointPresenter;
  MyObject: TGeoDataObject;
  gbAutomate, gbShadow: Boolean;
  giTimerCounter: integer;

implementation

uses u_dm, u_form_main;

{$R *.DFM}

procedure TFCartoGraphie.btnPrintChange(Sender: TObject);
begin
  Printer.BeginDoc;
  Globe.RenderToCanvas(Printer.Canvas, Globe.ClientRect, False);
  Printer.EndDoc;
end;

procedure TFCartoGraphie.GlobeMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
var
  sTmp: string;
  pt: TPointLL;
  MouseObj: TGlobeObject;
begin
  with Globe do
    begin
      Projection.DeviceXYToLL(X, Y, pt);

      with pt do
        sTmp := 'Longitude : ' + GlobeUnitsToStr(iLongX, '%d.%m.%s.%t% E ')
          + '  Latitude : ' + GlobeUnitsToStr(iLatY, '%d.%m.%s.%t% N');

      MouseObj := Layers.ObjectAtXY(X, Y);
      if MouseObj <> nil then
        sTmp := sTmp + ' ---- ' + MouseObj.Title;

      pnlHint.Caption := Format('%s', [sTmp]);
    end;
end;

procedure TFCartoGraphie.GlobePaint(Sender: TObject);
var
  ptLondon: TPointLL;
  pt, pt1: TPoint;
begin
  if MyObject <> nil then
    begin
      ptLondon := PointLL(DecimalToGlobeUnits(-0.007), DecimalToGlobeUnits(51.52));

      Globe.RenderCircle(ptLondon, GlobeUnitsFrom(200, kilometer), 3);
      Globe.RenderCircle(MyObject.Centroid, GU_DEGREE * 10, 36);
      Globe.RenderGreatCircleLine(ptLondon, MyObject.Centroid, gcShortest);
      Globe.Projection.LLToDeviceXY(MyObject.Centroid.iLongX, MyObject.Centroid.iLatY, pt);

      with GreatCirclePoint(MyObject.Centroid, ptLondon, 5 * GU_DEGREE / EllipsoidDistanceLLToLL(MyObject.Centroid, ptLondon, WGS84)) do
        Globe.Projection.LLToDeviceXY(iLongX, iLatY, pt1);

      { rotate the font so that the aircraft points away from London }
      with TPointPresenter(Globe.Layers.GlobalPresenters.ByID(MyObject.PresenterID, False)) do
        PointFont.FontAngle :=
          -Sign(MyObject.Centroid.iLongX) * 900 + Round(ArcTan2(pt.x - pt1.x, pt.y - pt1.y) * (180 / LocalPI)) * 10;
    end;
end;

procedure TFCartoGraphie.GlobeRender(Sender: TObject);
var
  iLeft, iTop, iBkMode, iTA: integer;
  Units: TGlobeUnitTypes;
  iBias: integer;
{$IFDEF WIN32}
  TimeZone: TTimeZoneInformation;
{$ENDIF}
begin
  if gbShadow then
    begin
      iBias := 0;
{$IFDEF WIN32}
      case GetTimeZoneInformation(TimeZone) of
        1: iBias := TimeZone.Bias + TimeZone.StandardBias;
        2: iBias := TimeZone.Bias + TimeZone.DayLightBias;
      end;
{$ENDIF}
      Globe.RenderDibShadow(Now + iBias / 1440, clGray, clSilver, 1);
    end;

  iLeft := Globe.Width - (50 + Screen.PixelsPerInch);
  iTop := Globe.Height - 50;

  with Globe.GlobeCanvas do
    begin
      Font.Assign(Self.Font);

      iBkMode := SetBkMode(Handle, TRANSPARENT);
      iTA := SetTextAlign(Handle, TA_CENTER);

      Font.Color := clRed;
      TextOut(iLeft, iTop, '0');
      Units := KiloMeter;
      if GlobeUnitsTo(Globe.Projection.UnitsPerInch, KiloMeter) < 10 then
        if GlobeUnitsTo(Globe.Projection.UnitsPerInch, Meter) < 10 then
          Units := Centimeter
        else
          Units := Meter;

      Font.Color := clRed;
      TextOut(iLeft + Screen.PixelsPerInch, iTop,
        Format('%d %s', [Round(GlobeUnitsTo(Globe.Projection.UnitsPerInch, Units)), UnitsToStr(Units)]));

      SetBkMode(Handle, iBkMode);
      SetTextAlign(Handle, iTA);

      Pen.color := clRed;
      MoveTo(iLeft, iTop + 16);
      LineTo(iLeft, iTop + 20);
      LineTo(iLeft + Screen.PixelsPerInch, iTop + 20);
      LineTo(iLeft + Screen.PixelsPerInch, iTop + 15);
    end;
end;

procedure TFCartoGraphie.rbPDFChange(Sender: TObject);
begin
  with Globe.Projection do
    ScaleFactor := ScaleFactor * 2;
end;

procedure TFCartoGraphie.rbRTFChange(Sender: TObject);
begin
  with Globe.Projection do
    ScaleFactor := ScaleFactor / 2;
end;

procedure TFCartoGraphie.rbHtmlChange(Sender: TObject);
begin
  with Globe do
    if Layers.SelectedObject <> nil then
      begin
        ZoomToMER(Globe, Layers.SelectedObject.ObjectMER);
      end
    else
      Projection.ScaleFactor := 0.0;
end;

procedure TFCartoGraphie.FormShow(Sender: TObject);
begin
  btnTexture.Visible := FileExists(ExtractFilePath(Application.ExeName) + 'Cartographie\World.bmp');

  DoInit(btnEurope);
end;

procedure TFCartoGraphie.DoInit(Sender: TObject);
var sPays, aPays: string;
begin
  Globe.SurfaceTextureName := '';

  case (Sender as TGoodBtn).Tag of
    1: apays := 'France';
    2: aPays := 'Europe';
    3: aPays := 'Globe';
  end;

  btnFrance.Font.Style := [];
  btnEurope.Font.Style := [];
  btnMonde.Font.Style := [];

  gbShadow := False;
  gbAutomate := False;

  btnRotation.Enabled := aPays = 'Globe';
  btnJour.Enabled := aPays = 'Globe';
  btnTexture.Enabled := aPays = 'Globe';
  btnVilles.Enabled := aPays = 'Europe';
  btnMerca.Font.Style := [];
  btnSphere.Font.Style := [];

  Globe.DataDirectory := ExtractFilePath(Application.ExeName) + 'Cartographie';

  if aPays = 'Europe' then
    begin
      btnMerca.Font.Style := [fsBold];
      btnEurope.Font.Style := [fsBold];

      // Projection utilise
      Globe.Projection.ProjectionClass := 'TMercatorPrj';
      // Altitude de visualisation
      Globe.Projection.Altitude := 140000;
      aPays := 'Europe';
      sPays := 'Europe.lyr'
    end
  else if aPays = 'France' then
    begin
      btnMerca.Font.Style := [fsBold];
      btnFrance.Font.Style := [fsBold];

      // Projection utilise
      Globe.Projection.ProjectionClass := 'TMercatorPrj';
      // Altitude de visualisation
      Globe.Projection.Altitude := 140000;
      aPays := 'France';
      sPays := 'France.lyr'
    end
  else
    begin
      btnSphere.Font.Style := [fsBold];
      btnMonde.Font.Style := [fsBold];

      // Projection utilise
      Globe.Projection.ProjectionClass := 'TSphericalPrj';
      // Altitude de visualisation
      Globe.Projection.Altitude := 0;
      aPays := 'Globe';
      sPays := 'World_countries_shp.shp';
    end;

  //aLayer.Free;
  Globe.Clear;

  // On centre la carte sur une position (environ le centre de la france)
  Globe.Projection.CenterLatitude := 46.9;
  Globe.Projection.CenterLongitude := 2;

  // Chargement du fichier couche contenant les objets constituant la carte de france
  CreateFileLayer(Globe, sPays);

  if aPays = 'Europe' then CreateFileLayer(Globe, 'Eur_caps.lyr');

  if aPays = 'Globe' then
    begin
      CreateFileLayer(Globe, 'TGLakes.lyr');
      CreateFileLayer(Globe, 'TGRivers.lyr');
    end;

  DoAfterInit;
end;

procedure TFCartoGraphie.DoAfterInit;
begin
  // Cration de la couche pour afficher les points
  aLayer := TGLobeLayer.Create(Globe);
  // Cration d'un Presenter
  aPresenter := TPointPresenter.Create(Globe, 1);

  // Affectation du Presenter  la couche de dessin
  aLayer.Presenters.Add(aPresenter);

  // Parametres du Presenter
  with aPresenter do
    begin
      PointType := ppTriangleUp;
      PointSize := 25;
      PointUnit := kilometer;
      TitleAlignment := taBottomRight;
      TitleFont.FontName := 'verdana';
      //TitleFont.FontStyle := [fsbold];
      TitleFont.FontColor := clnavy;
      TitleFont.FontUnit := kilometer;
      TitleFont.FontSize := 0;
    end;

  DoInitBase;
end;

procedure TFCartoGraphie.DoInitBase;
var s_Query: TIBQuery;
begin
  s_Query := TIBQuery.Create(Application);

  with s_Query do begin
      DataBase := dm.ibd_BASE;
      SQL.ADD('SELECT * FROM PROC_POSITION_GEO_EVENT (:CLEF)');
      Prepare;
      Params[0].AsInteger := dm.FicheActive;
      Open;
      while not eof do
        begin
          iLat := DecimalToGlobeUnits(s_Query.fieldByName('CP_LATITUDE').AsFloat);
          iLong := DecimalToGlobeUnits(s_Query.fieldByName('CP_LONGITUDE').AsFloat);

          // Cration du point
          with TGeoDataObject.Create(Alayer.Objects) do
            begin
              // Centre du point
              Centroid := PointLL(iLong, iLat);
              // MyObject.Centroid := PointLL(160 * GU_DEGREE, 0);

             // Label du point
              Title := s_Query.fieldByName('EV_IND_VILLE').AsString;
              // Presenter utilis pour l'affichage du point
              PresenterID := aPresenter.PresenterID;
            end;
          next;
        end;

    end;
  s_Query.Close;
  s_Query.Free;
end;

procedure TFCartoGraphie.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  // aLayer.Free;
  Globe.Clear;
  Globe.Free;
end;

procedure TFCartoGraphie.btnConfigClick(Sender: TObject);
begin
  DoInit(Sender);
end;

procedure TFCartoGraphie.DoTexture(Sender: TObject);
begin
  btnNormal.Font.Style := [];
  btnTransparent.Font.Style := [];

  with Globe do
    case TGoodBtn(Sender).Tag of
      0: // Normal
        begin
          btnNormal.Font.Style := [fsBold];
          btnTexture.Enabled := True;
          SurfaceTextureName := '';
          GlobeOptions := GlobeOptions - [goTransparentGlobe];
        end;
      1: // Transparent
        begin
          btnTransparent.Font.Style := [fsBold];
          btnTexture.Enabled := False;
          SurfaceTextureName := '';
          GlobeOptions := GlobeOptions + [goTransparentGlobe];
        end;

    end;
end;

procedure TFCartoGraphie.DoProjection(Sender: TObject);
begin
  btnMerca.Font.Style := [];
  btnSphere.Font.Style := [];
  btnCarte.Font.Style := [];
  gbAutomate := false;

  case TGoodBtn(Sender).Tag of
    0:
      begin
        btnSphere.Font.Style := [fsBold];
        Globe.Projection.ProjectionClass := 'TSphericalPrj';
      end;
    1:
      begin
        btnCarte.Font.Style := [fsBold];
        Globe.Projection.ProjectionClass := 'TCartesianPrj';
      end;
    2:
      begin
        btnMerca.Font.Style := [fsBold];
        Globe.Projection.ProjectionClass := 'TMercatorPrj';
      end;
  end;
  Globe.Projection.ScaleFactor := 0;
end;

procedure TFCartoGraphie.DoZoom(Sender: TObject);
begin
  with TGoodBtn(Sender) do
    Globe.Projection.Altitude := Round(Globe.Projection.Altitude * (Tag / 100));
end;

procedure TFCartoGraphie.btnJourClick(Sender: TObject);
begin
  gbShadow := not gbShadow;

  if gbShadow then
    btnJour.Font.Style := [fsBold]
  else
    btnJour.Font.Style := [];

end;

procedure TFCartoGraphie.btnRotationClick(Sender: TObject);
begin
  gbAutomate := not gbAutomate;

  if gbAutomate then
    btnRotation.Font.Style := [fsBold]
  else
    btnRotation.Font.Style := [];

end;

procedure TFCartoGraphie.Timer1Timer(Sender: TObject);
var
  iX, iY: integer;
begin
  if MyObject <> nil then
    with MyObject.Centroid do
      begin
        iX := (iLongX + GU_DEGREE) mod GU_360_DEGREE;
        iY := Round(60 * GU_DEGREE * Cos(iLongX * GU_TORADIANS));
        MyObject.Centroid := PointLLH(iX, iY, GU_DEGREE * 5);
      end;

  if gbAutomate then
    Globe.Projection.YRotation := Globe.Projection.YRotation - 0.5;

  if gbShadow then
    begin
      Dec(giTimerCounter);
      if giTimerCounter <= 0 then
        begin
          giTimerCounter := 40;
          Globe.RedrawLayers; { update every 10 seconds }
        end;
    end;

end;

procedure TFCartoGraphie.btnVillesClick(Sender: TObject);
begin
  if btnVilles.Font.Style = [fsBold] then
    begin
      btnVilles.Font.Style := [];
      // Globe.Layers.Delete(Globe.Layers[2]);
    end
  else
    begin
      btnVilles.Font.Style := [fsBold];
      CreateFileLayer(Globe, 'Eucty_6k.lyr');
    end;
end;

procedure TFCartoGraphie.btnTextureClick(Sender: TObject);
begin
  Globe.Clear;

  if Length(Globe.SurfaceTextureName) > 0 then
    begin
      btnTexture.Font.Style := [];
      Globe.SurfaceTextureName := '';
      CreateFileLayer(Globe, 'World_countries_shp.shp');
      CreateFileLayer(Globe, 'TGLakes.lyr');
      CreateFileLayer(Globe, 'TGRivers.lyr');
      btnTransparent.Enabled := True;
      btnNormal.Enabled := True;
      //  DoInitBase;
    end
  else
    begin
      btnTransparent.Enabled := False;
      btnNormal.Enabled := False;
      btnTexture.Font.Style := [fsBold];
      Globe.SurfaceTextureName := ExtractFilePath(Application.ExeName) + 'Cartographie\World.bmp';
    end;


end;

procedure TFCartoGraphie.btnAjoutClick(Sender: TObject);
begin
  if OpenDialog.Execute then
    begin
      btnFrance.Font.Style := [];
      btnEurope.Font.Style := [];
      btnMonde.Font.Style := [];

      gbShadow := False;
      gbAutomate := False;

      CreateFileLayer(Globe, OpenDialog.FileName);
    end;

end;

end.
