{-----------------------------------------------------------------------}
{                                                                       }
{           Subprogram Name:                                            }
{           Purpose:          Ancestrologie                             }
{           Source Language:  Francais                                  }
{           Auteur:  f                                                   }
{           Philippe Cazaux-Moutou                                      }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Description:                                                }
{           Ancestrologie est un Logiciel Libre                         }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Revision History                                            }
{           v#    ,Date       ,Author Name            ,Description      }
{                                                                       }
{-----------------------------------------------------------------------}

unit u_gedcom_type;

interface

uses
  Classes,
  typinfo,
  contnrs,
  Dialogs;

type
  TGedComStructure = class;

  TTag = class(TPersistent)
  private
    fSimple: boolean;
    fClassSubTags: TClassList;
    fObjectSubTags: TObjectList;
    fCaptionSubTags: TStringList;
    fValue: string;
    fID: string;
    fCaption: string;
    fPtrGedComStructure: TGedComStructure;
    fPropInfoSubTags: TList;
    fLevel: integer;
    fPtrNextSameTag: TObject;

    procedure DoIntrospection;
    procedure DoAttachInList; virtual;

  public
    //est-ce un SimpleTag ? (ou Tag terminal)
    property Simple: boolean read fSimple write fSimple;

    //contenue de ce tag
    property Level: integer read fLevel write fLevel;
    property Caption: string read fCaption write fCaption;
    property ID: string read fID write fID;
    property Value: string read fValue write fValue;
    property PtrNextSameTag: TObject read fPtrNextSameTag write fPtrNextSameTag;

    //pointeurs sur les tags de ce Tag, si ils existent
    property CaptionSubTags: TStringList read fCaptionSubTags write fCaptionSubTags;
    property ClassSubTags: TClassList read fClassSubTags write fClassSubTags;
    property ObjectSubTags: TObjectList read fObjectSubTags write fObjectSubTags;
    property PropInfoSubTags: TList read fPropInfoSubTags write fPropInfoSubTags;

    procedure AfterCreate(aGedComStructure: TGedComStructure); virtual;
    destructor Destroy; override;
    function ExistSubTag(TagName: string; var index: integer): boolean;


  end;

  //Dfinition des class utiliss
  TTag_HEADER_RECORD = class;
  TTag_APPROVED_SYSTEM_ID = class;
  TTag_NAME_OF_BUSINESS = class;
  TTag_ADDRESS_STRUCTURE = class;
  TTag_INDIVIDUAL_RECORD = class;
  TTag_SOURCE_RECORD = class;
  TTag_NOTE_RECORD = class;
  TTag_REPO_RECORD = class;
  TTag_FAM_RECORD = class;
  TTag_NAME_OF_SOURCE_DATA = class;
  TTag_DATE_CHANGE_DATE = class;
  TTag_TRANSMISSION_DATE = class;
  TTag_GEDC = class;
  TTag_CHAR = class;
  TTag_PLAC = class;
  TTag_NOTE_STRUCTURE = class;
  //     TTag_NOTE = class;

  TTag_SOURCE_CITATION = class;


  TGedComStructure = class(TTag)
  private
    fIndividuList: TObjectList;
    fFamilleList: TObjectList;
    fOtherList: TObjectList;
    fAllList: TObjectList;

    f_TRLR: TTag;
    f_HEAD: TTag_Header_Record;
    f_INDI: TTag_Individual_Record;
    f_FAM: TTag_FAM_RECORD;
    f_SOUR: TTag_SOURCE_RECORD;
    fNoteList: TObjectList;
    fSourceList: TObjectList;
    f_NOTE: TTag_NOTE_RECORD;
    f_REPO: TTag_REPO_RECORD;
    fRepoList: TObjectList;
    fPtrObjetImportGedcom: TObject;

  public
    property AllList: TObjectList read fAllList write fAllList;
    property IndividuList: TObjectList read fIndividuList write fIndividuList;
    property FamilleList: TObjectList read fFamilleList write fFamilleList;
    property SourceList: TObjectList read fSourceList write fSourceList;
    property NoteList: TObjectList read fNoteList write fNoteList;
    property RepoList: TObjectList read fRepoList write fRepoList;

    //#########  pointeur vers la liste des autres structures [rfrences] : notes, sources, repo, etc...
    property PtrObjetImportGedcom: TObject read fPtrObjetImportGedcom write fPtrObjetImportGedcom;

    constructor Create;
    destructor Destroy; override;

    function GetIndividuByID(ID: string): TTag_INDIVIDUAL_RECORD;
    function GetFamilleByID(ID: string): TTag_FAM_RECORD;
    function GetSourceRecordByID(ID: string): TTag_SOURCE_RECORD;
    function GetNoteRecordByID(ID: string): TTag_NOTE_RECORD;
    function GetRepoRecordByID(ID: string): TTag_REPO_RECORD;

    procedure Clear;

  published
    property _HEAD: TTag_HEADER_RECORD read f_HEAD write f_HEAD;
    //SUBMISSION_RECORD
    property _FAM: TTag_FAM_RECORD read f_FAM write f_FAM;
    property _INDI: TTag_INDIVIDUAL_RECORD read f_INDI write f_INDI;
    property _SOUR: TTag_SOURCE_RECORD read f_SOUR write f_SOUR;
    property _NOTE: TTag_NOTE_RECORD read f_NOTE write f_NOTE;
    property _REPO: TTag_REPO_RECORD read f_REPO write f_REPO;
    //SUBMITTER_RECORD
    property _TRLR: TTag read f_TRLR write f_TRLR;
  end;

  // Header *************************************************************************************
  TTag_HEADER_RECORD = class(TTag)
  private
    f_SOUR: TTag_APPROVED_SYSTEM_ID;
    f_DEST: TTag;
    f_LANG: TTag;
    f_DATE: TTag_TRANSMISSION_DATE;
    f_SUBN: TTag;
    f_SUBM: TTag;
    f_FILE: TTag;
    f_COPR: TTag;
    f_CHAR: TTag_CHAR;
    f_GEDC: TTag_GEDC;
    f_PLAC: TTag_PLAC;
  published
    property _SOUR: TTag_APPROVED_SYSTEM_ID read f_SOUR write f_SOUR;
    property _DEST: TTag read f_DEST write f_DEST;
    property _DATE: TTag_TRANSMISSION_DATE read f_DATE write f_DATE;
    property _SUBM: TTag read f_SUBM write f_SUBM;
    property _SUBN: TTag read f_SUBN write f_SUBN;
    property _FILE: TTag read f_FILE write f_FILE;
    property _COPR: TTag read f_COPR write f_COPR;
    property _GEDC: TTag_GEDC read f_GEDC write f_GEDC;
    property _CHAR: TTag_CHAR read f_CHAR write f_CHAR;
    property _LANG: TTag read f_LANG write f_LANG;
    property _PLAC: TTag_PLAC read f_PLAC write f_PLAC;
  end;

  TTag_APPROVED_SYSTEM_ID = class(TTag)
  private
    f_NAME: TTag;
    f_VERS: TTag;
    f_CORP: TTag_NAME_OF_BUSINESS;
    f_DATA: TTag_NAME_OF_SOURCE_DATA;
  published
    property _VERS: TTag read f_VERS write f_VERS;
    property _NAME: TTag read f_NAME write f_NAME;
    property _CORP: TTag_NAME_OF_BUSINESS read f_CORP write f_CORP;
    property _DATA: TTag_NAME_OF_SOURCE_DATA read f_DATA write f_DATA;
  end;

  TTag_NAME_OF_BUSINESS = class(TTag)
  private
    f_ADDR: TTag_ADDRESS_STRUCTURE;
    f_PHON: TTag;
  published
    property _ADDR: TTag_ADDRESS_STRUCTURE read f_ADDR write f_ADDR;
    property _PHON: TTag read f_PHON write f_PHON;
  end;


  TTag_ADDRESS_STRUCTURE = class(TTag)
  private
    f_CITY: TTag;
    f_ADR2: TTag;
    f_ADR1: TTag;
    f_POST: TTag;
    f_CONT: TTag;
    f_STAE: TTag;
    f_CTRY: TTag;
  public
    function AsLine: string;

  published
    property _CONT: TTag read f_CONT write f_CONT;
    property _ADR1: TTag read f_ADR1 write f_ADR1;
    property _ADR2: TTag read f_ADR2 write f_ADR2;
    property _CITY: TTag read f_CITY write f_CITY;
    property _STAE: TTag read f_STAE write f_STAE;
    property _POST: TTag read f_POST write f_POST;
    property _CTRY: TTag read f_CTRY write f_CTRY;
  end;

  TTag_NAME_OF_SOURCE_DATA = class(TTag)
  private
    f_DATE: TTag;
    f_COPR: TTag;
  published
    property _DATE: TTag read f_DATE write f_DATE;
    property _COPR: TTag read f_COPR write f_COPR;
  end;

  TTag_GEDC = class(TTag)
  private
    f_VERS: TTag;
    f_FORM: TTag;
  published
    property _VERS: TTag read f_VERS write f_VERS;
    property _FORM: TTag read f_FORM write f_FORM;
  end;

  TTag_CHAR = class(TTag)
  private
    f_VERS: TTag;
  published
    property _VERS: TTag read f_VERS write f_VERS;
  end;

  TTag_PLAC = class(TTag)
  private
    f_FORM: TTag;
  published
    property _FORM: TTag read f_FORM write f_FORM;
  end;

  // Individu *********************************************************************************
  TTag_Personal_Name_Structure = class;
  TTag_Event_Detail = class;
  TTag_EVENT_TYPE_CITED_FROM = class;
  TTag_ASSOCIATION_STRUCTURE = class;
  TTag_CHANGE_DATE = class;
  TTag_CHILD_TO_FAMILY_LINK = class;
  TTag_SPOUSE_TO_FAMILY_LINK = class;
  TTag_USER_REFERENCE_NUMBER = class;
  TTag_PLACE_STRUCTURE = class;

  TTag_MEMO = class;

  TTag_DOMICILE = class(TTag)
  private
    f_CITY: TTag;
    f_MAIL: TTag;
    f_POST: TTag;
    f_STAT: TTag;
    f_CTRY: TTag;
    f_REG: TTag;
    f_DEPT: TTag;
    f_ADDR: TTag;
    f_ADR1: TTag_MEMO;
    f_ADR2: TTag_MEMO;
    // f_ADR: TTag_MEMO;
    f_TEL: TTag_MEMO;
    fCle: integer;
    f_DATE: TTag;
  public
    property Cle: integer read fCle write fCle;
  published
    property _ADDR: TTag read f_ADDR write f_ADDR;
    property _ADR1: TTag_MEMO read f_ADR1 write f_ADR1;
    property _ADR2: TTag_MEMO read f_ADR2 write f_ADR2;

    //  property _ADR: TTag_MEMO read f_ADR write f_ADR;
    property _POST: TTag read f_POST write f_POST;
    property _CITY: TTag read f_CITY write f_CITY;
    property _DEPT: TTag read f_DEPT write f_DEPT;
    property _REG: TTag read f_REG write f_REG;
    property _STAT: TTag read f_STAT write f_STAT;
    property _CTRY: TTag read f_CTRY write f_CTRY;
    property _MAIL: TTag read f_MAIL write f_MAIL;
    property _TEL: TTag_MEMO read f_TEL write f_TEL;
    property _DATE: TTag read f_DATE write f_DATE;
  end;

  TTag_MEDIA = class(TTag)

  private
    f_PICT: TTag;
    f_PATH: TTag;
    f_XIDEN: TTag;
    f_XTYPE: TTag;
    f_XMODE: TTag;
    f_XACTE: TTag;
    fCle: integer;
  public
    property Cle: integer read fCle write fCle;
  published
    property _PICT: TTag read f_PICT write f_PICT;
    property _PATH: TTag read f_PATH write f_PATH;
    property _XIDEN: TTag read f_XIDEN write f_XIDEN;
    property _XTYPE: TTag read f_XTYPE write f_XTYPE;
    property _XMODE: TTag read f_XMODE write f_XMODE;
    property _XACTE: TTag read f_XACTE write f_XACTE;
  end;


  TTag_INDIVIDUAL_RECORD = class(TTag)
  private
    f_SEX: TTag;
    f_RESN: TTag;
    f_EMIG: TTag_Event_Detail;
    f_DIPL: TTag_Event_Detail;
    f_PROB: TTag_Event_Detail;
    f_FCOM: TTag_Event_Detail;
    f_BURI: TTag_Event_Detail;
    f_NATU: TTag_Event_Detail;
    f_CHRA: TTag_Event_Detail;
    f_ORDN: TTag_Event_Detail;
    f_DEAT: TTag_Event_Detail;
    f_RETI: TTag_Event_Detail;
    f_WILL: TTag_Event_Detail;
    f_CENS: TTag_Event_Detail;
    f_BLES: TTag_Event_Detail;
    f_CREM: TTag_Event_Detail;
    f_BASM: TTag_Event_Detail;
    f_IMMI: TTag_Event_Detail;
    f_BAPM: TTag_Event_Detail;
    f_BARM: TTag_Event_Detail;
    f_GRAD: TTag_Event_Detail;
    f_CONF: TTag_Event_Detail;
    f_EVEN: TTag_Event_Detail;
    f_CHR: TTag_Event_Detail;
    f_BIRT: TTag_Event_Detail;
    f_ADOP: TTag_Event_Detail;
    f_NAME: TTag_Personal_Name_Structure;
    f_TITL: TTag_Event_Detail;
    f_NMR: TTag_Event_Detail;
    f_RESI: TTag_Event_Detail;
    f_CAST: TTag_Event_Detail;
    f_DSCR: TTag_Event_Detail;
    f_EDUC: TTag_Event_Detail;
    f_IDNO: TTag_Event_Detail;
    f_PROP: TTag_Event_Detail;
    f_NATI: TTag_Event_Detail;
    f_RELI: TTag_Event_Detail;
    f_SSN: TTag_Event_Detail;
    f_OCCU: TTag_Event_Detail;
    f_NCHI: TTag_Event_Detail;
    f_CONL: TTag_Event_Detail;
    f_ENDL: TTag_Event_Detail;
    f_BAPL: TTag_Event_Detail;
    f_SLGC: TTag_Event_Detail;
    f_ALIA: TTag;
    f_AFN: TTag;
    f_ANCI: TTag;
    f_RFN: TTag;
    f_SUBM: TTag;
    f_RIN: TTag;
    f_DESI: TTag;
    //          f_ASSO: TTag_ASSOCIATION_STRUCTURE;
    f_CHAN: TTag_CHANGE_DATE;
    f_FAMC: TTag_CHILD_TO_FAMILY_LINK;
    f_NOTE: TTag_NOTE_STRUCTURE; //TTag_NOTE_STRUCTURE;
    f_SOUR: TTag_SOURCE_CITATION; //SOURCE_CITATION;
    f_FAMS: TTag_SPOUSE_TO_FAMILY_LINK;
    f_REFN: TTag_USER_REFERENCE_NUMBER;

    fCle: integer;
    fCLE_PARENTS: integer;
    fCLE_PERE: integer;
    fCLE_MERE: integer;
    f_SALE: TTag_Event_Detail;
    f_PURC: TTag_Event_Detail;
    f_MILI: TTag_Event_Detail;
    f_ALIV: TTag_Event_Detail;
    f_TRIP: TTag_Event_Detail;
    f_BRIT: TTag_Event_Detail;
    f_FILA: TTag;
    f_INHU: TTag_Event_Detail;
    //f_DOMI: TTag_DOMICILE;
    f_ADDR: TTag_DOMICILE;
    f_PICT: TTag_MEDIA;

  protected
    procedure DoAttachInList; override;

  public

    property Cle: integer read fCle write fCle;
    property CLE_PARENTS: integer read fCLE_PARENTS write fCLE_PARENTS;
    property CLE_PERE: integer read fCLE_PERE write fCLE_PERE;
    property CLE_MERE: integer read fCLE_MERE write fCLE_MERE;

    procedure AfterCreate(aGedComStructure: TGedComStructure); override;

  published
    property _RESN: TTag read f_RESN write f_RESN;
    property _NAME: TTag_Personal_Name_Structure read f_NAME write f_NAME;
    property _SEX: TTag read f_SEX write f_SEX;

    //INDIVIDUAL_EVENT_STRUCTURE
    property _BIRT: TTag_Event_Detail read f_BIRT write f_BIRT;
    property _CHR: TTag_Event_Detail read f_CHR write f_CHR;
    property _DEAT: TTag_Event_Detail read f_DEAT write f_DEAT;
    property _BURI: TTag_Event_Detail read f_BURI write f_BURI;
    property _CREM: TTag_Event_Detail read f_CREM write f_CREM;
    property _ADOP: TTag_Event_Detail read f_ADOP write f_ADOP;
    property _BAPM: TTag_Event_Detail read f_BAPM write f_BAPM;
    property _BARM: TTag_Event_Detail read f_BARM write f_BARM;
    property _BASM: TTag_Event_Detail read f_BASM write f_BASM;
    property _BLES: TTag_Event_Detail read f_BLES write f_BLES;
    property _CHRA: TTag_Event_Detail read f_CHRA write f_CHRA;
    property _CONF: TTag_Event_Detail read f_CONF write f_CONF;
    property _FCOM: TTag_Event_Detail read f_FCOM write f_FCOM;
    property _ORDN: TTag_Event_Detail read f_ORDN write f_ORDN;
    property _NATU: TTag_Event_Detail read f_NATU write f_NATU;
    property _EMIG: TTag_Event_Detail read f_EMIG write f_EMIG;

    property _DIPL: TTag_Event_Detail read f_DIPL write f_DIPL;

    property _IMMI: TTag_Event_Detail read f_IMMI write f_IMMI;
    property _CENS: TTag_Event_Detail read f_CENS write f_CENS;
    property _PROB: TTag_Event_Detail read f_PROB write f_PROB;
    property _WILL: TTag_Event_Detail read f_WILL write f_WILL;
    property _GRAD: TTag_Event_Detail read f_GRAD write f_GRAD;
    property _RETI: TTag_Event_Detail read f_RETI write f_RETI;
    property _EVEN: TTag_Event_Detail read f_EVEN write f_EVEN;
    property _BRIT: TTag_Event_Detail read f_BRIT write f_BRIT;

    //INDIVIDUAL_ATTRIBUTE_STRUCTURE
    property _CAST: TTag_Event_Detail read f_CAST write f_CAST;
    property _DSCR: TTag_Event_Detail read f_DSCR write f_DSCR;
    property _EDUC: TTag_Event_Detail read f_EDUC write f_EDUC;
    property _IDNO: TTag_Event_Detail read f_IDNO write f_IDNO;
    property _NATI: TTag_Event_Detail read f_NATI write f_NATI;
    property _NCHI: TTag_Event_Detail read f_NCHI write f_NCHI;
    property _NMR: TTag_Event_Detail read f_NMR write f_NMR;
    property _OCCU: TTag_Event_Detail read f_OCCU write f_OCCU;
    property _PROP: TTag_Event_Detail read f_PROP write f_PROP;
    property _RELI: TTag_Event_Detail read f_RELI write f_RELI;
    property _RESI: TTag_Event_Detail read f_RESI write f_RESI;
    property _SSN: TTag_Event_Detail read f_SSN write f_SSN;
    property _TITL: TTag_Event_Detail read f_TITL write f_TITL;

    //LDS_INDIVIDUAL_ORDINANCE
    property _BAPL: TTag_Event_Detail read f_BAPL write f_BAPL;
    property _CONL: TTag_Event_Detail read f_CONL write f_CONL;
    property _ENDL: TTag_Event_Detail read f_ENDL write f_ENDL;
    property _SLGC: TTag_Event_Detail read f_SLGC write f_SLGC;

    //Autres Tag (de Heredis...)
    property _PURC: TTag_Event_Detail read f_PURC write f_PURC; //Acquisition
    property _ALIV: TTag_Event_Detail read f_ALIV write f_ALIV; //En vie
    property _MILI: TTag_Event_Detail read f_MILI write f_MILI; //Service militaire
    property _SALE: TTag_Event_Detail read f_SALE write f_SALE; //Vente d'un bien
    property _TRIP: TTag_Event_Detail read f_TRIP write f_TRIP; //Voyage
    property _INHU: TTag_Event_Detail read f_INHU write f_INHU; //Inhumation

    property _FAMC: TTag_CHILD_TO_FAMILY_LINK read f_FAMC write f_FAMC;
    property _FAMS: TTag_SPOUSE_TO_FAMILY_LINK read f_FAMS write f_FAMS;

    property _SUBM: TTag read f_SUBM write f_SUBM;
    property _ALIA: TTag read f_ALIA write f_ALIA;
    property _ANCI: TTag read f_ANCI write f_ANCI;
    property _DESI: TTag read f_DESI write f_DESI;

    property _SOUR: TTag_SOURCE_CITATION read f_SOUR write f_SOUR;
    property _NOTE: TTag_NOTE_STRUCTURE read f_NOTE write f_NOTE;

    property _RFN: TTag read f_RFN write f_RFN;
    property _AFN: TTag read f_AFN write f_AFN;
    property _REFN: TTag_USER_REFERENCE_NUMBER read f_REFN write f_REFN;
    property _RIN: TTag read f_RIN write f_RIN;
    property _CHAN: TTag_CHANGE_DATE read f_CHAN write f_CHAN;

    //Filliation
    property _FILA: TTag read f_FILA write f_FILA;

    //Domiciles
   // property _DOMI: TTag_DOMICILE read f_DOMI write f_DOMI;
    property _ADDR: TTag_DOMICILE read f_ADDR write f_ADDR;
    property _PICT: TTag_MEDIA read f_PICT write f_PICT;

  end;

  TTag_Personal_Name_Structure = class(TTag)
  private
    f_SURN: TTag;
    f_NSFX: TTag;
    f_NPFX: TTag;
    f_GIVN: TTag;
    f_SPFX: TTag;
    f_NICK: TTag;
    f_NOTE: TTag_NOTE_STRUCTURE; //TTag_NOTE_STRUCTURE;
    f_SOUR: TTag_SOURCE_CITATION; //TTag_SOURCE_CITATION;

  published
    property _NPFX: TTag read f_NPFX write f_NPFX;
    property _GIVN: TTag read f_GIVN write f_GIVN;
    property _NICK: TTag read f_NICK write f_NICK;
    property _SPFX: TTag read f_SPFX write f_SPFX;
    property _SURN: TTag read f_SURN write f_SURN;
    property _NSFX: TTag read f_NSFX write f_NSFX;
    property _SOUR: TTag_SOURCE_CITATION read f_SOUR write f_SOUR;
    property _NOTE: TTag_NOTE_STRUCTURE read f_NOTE write f_NOTE;
  end;

  TTag_Event_Detail = class(TTag)
  private
    f_AGE: TTag;
    f_CAUS: TTag;
    f_DATE: TTag;
    f_TIME: TTag;
    f_TYPE: TTag;
    f_AGNC: TTag;
    f_SOUR: TTag_SOURCE_CITATION;
    f_NOTE: TTag_NOTE_STRUCTURE;
    f_PLAC: TTag_PLACE_STRUCTURE;
    f_ADDR: TTag_ADDRESS_STRUCTURE;
    f_PHON: TTag;
    f_ASSO: TTag_ASSOCIATION_STRUCTURE;

    fCle: integer;
    f_LIEU: TTag;
    f_XINSEE: TTag;
    f_XORD: TTag;
    f_XACTE: TTag;

  public
    property Cle: integer read fCle write fCle;

  published
    property _TYPE: TTag read f_TYPE write f_TYPE;
    property _DATE: TTag read f_DATE write f_DATE;
    property _TIME: TTag read f_TIME write f_TIME;
    property _PLAC: TTag_PLACE_STRUCTURE read f_PLAC write f_PLAC;
    property _LIEU: TTag read f_LIEU write f_LIEU;
    property _XINSEE: TTag read f_XINSEE write f_XINSEE;
    property _XORD: TTag read f_XORD write f_XORD;
    property _XACTE: TTag read f_XACTE write f_XACTE;
    property _ADDR: TTag_ADDRESS_STRUCTURE read f_ADDR write f_ADDR;
    property _PHON: TTag read f_PHON write f_PHON;
    property _AGE: TTag read f_AGE write f_AGE;
    property _AGNC: TTag read f_AGNC write f_AGNC;
    property _CAUS: TTag read f_CAUS write f_CAUS;
    property _SOUR: TTag_SOURCE_CITATION read f_SOUR write f_SOUR;
    property _NOTE: TTag_NOTE_STRUCTURE read f_NOTE write f_NOTE;
    property _ASSO: TTag_ASSOCIATION_STRUCTURE read f_ASSO write f_ASSO;
  end;

  TTag_PLACE_STRUCTURE = class(TTag)
  private
    f_FORM: TTag;
    f_NOTE: TTag_NOTE_STRUCTURE;
    f_SOUR: TTag_SOURCE_CITATION;

  published
    property _FORM: TTag read f_FORM write f_FORM;
    property _SOUR: TTag_SOURCE_CITATION read f_SOUR write f_SOUR;
    property _NOTE: TTag_NOTE_STRUCTURE read f_NOTE write f_NOTE;
  end;

  TTag_MEMO = class(TTag)
  private

    f_CONC: TTag;
    f_CONT: TTag;

  public
    //le memo
    Memo: TStringList;

    procedure AfterCreate(aGedComStructure: TGedComStructure); override;
    destructor Destroy; override;
    function AsLine: string;

  published
    property _CONC: TTag read f_CONC write f_CONC;
    property _CONT: TTag read f_CONT write f_CONT;
  end;

  TTag_EVENT_TYPE_CITED_FROM = class(TTag)
  private
    f_ROLE: TTag;

  published
    property _ROLE: TTag read f_ROLE write f_ROLE;
  end;

  TTag_DATA1 = class(TTag)
  private
    f_DATE: TTag;
    f_TEXT: TTag_MEMO;
    f_CONC: TTag;
    f_CONT: TTag;
  public
    //le memo
    Memo: TStringList;

    procedure AfterCreate(aGedComStructure: TGedComStructure); override;
    destructor Destroy; override;
    function AsLine: string;

  published
    property _DATE: TTag read f_DATE write f_DATE;
    property _TEXT: TTag_MEMO read f_TEXT write f_TEXT;
    property _CONC: TTag read f_CONC write f_CONC;
    property _CONT: TTag read f_CONT write f_CONT;
  end;


  TTag_SOURCE_CITATION = class(TTag)
  private

    f_CONC: TTag;
    f_CONT: TTag;
    f_PAGE: TTag;
    f_EVEN: TTag_EVENT_TYPE_CITED_FROM;
    f_DATA: TTag_DATA1;
    f_QUAY: TTag;
    f_NOTE: TTag_NOTE_STRUCTURE;
    f_TEXT: TTag_MEMO;
    f_ABBR: TTag;
    f_REPO: TTag;

  public
    //le memo
    Memo: TStringList;

    procedure AfterCreate(aGedComStructure: TGedComStructure); override;
    destructor Destroy; override;
    function AsLine: string;

  published
    //soit
    property _PAGE: TTag read f_PAGE write f_PAGE;
    property _EVEN: TTag_EVENT_TYPE_CITED_FROM read f_EVEN write f_EVEN;
    property _DATA: TTag_DATA1 read f_DATA write f_DATA;
    property _QUAY: TTag read f_QUAY write f_QUAY;

    //soit
    property _CONC: TTag read f_CONC write f_CONC;
    property _CONT: TTag read f_CONT write f_CONT;
    property _TEXT: TTag_MEMO read f_TEXT write f_TEXT;
    property _ABBR: TTag read f_ABBR write f_ABBR;
    property _REPO: TTag read f_REPO write f_REPO;

    property _NOTE: TTag_NOTE_STRUCTURE read f_NOTE write f_NOTE;
  end;

  TTag_NOTE_STRUCTURE = class(TTag)
  private

    f_CONC: TTag;
    f_CONT: TTag;
    f_SOUR: TTag_SOURCE_RECORD;

  public
    //le memo
    Memo: TStringList;

    procedure AfterCreate(aGedComStructure: TGedComStructure); override;
    destructor Destroy; override;
    function AsLine: string;


  published
    property _CONC: TTag read f_CONC write f_CONC;
    property _CONT: TTag read f_CONT write f_CONT;
    property _SOUR: TTag_SOURCE_RECORD read f_SOUR write f_SOUR;
  end;


  TTag_ASSOCIATION_STRUCTURE = class(TTag)
  private
    f_RELA: TTag;
    f_NOTE: TTag_NOTE_STRUCTURE;
    f_SOUR: TTag_SOURCE_CITATION;

  published
    property _RELA: TTag read f_RELA write f_RELA;
    property _NOTE: TTag_NOTE_STRUCTURE read f_NOTE write f_NOTE;
    property _SOUR: TTag_SOURCE_CITATION read f_SOUR write f_SOUR;

  end;


  TTag_CHANGE_DATE = class(TTag)
  private
    f_DATE: TTag_DATE_CHANGE_DATE;
    f_NOTE: TTag_NOTE_STRUCTURE;

  public
    function AsLine: string;

  published
    property _DATE: TTag_DATE_CHANGE_DATE read f_DATE write f_DATE;
    property _NOTE: TTag_NOTE_STRUCTURE read f_NOTE write f_NOTE;

  end;


  TTag_DATE_CHANGE_DATE = class(TTag)
  private
    f_TIME: TTag;

  published
    property _TIME: TTag read f_TIME write f_TIME;

  end;

  TTag_TRANSMISSION_DATE = class(TTag)
  end;


  TTag_CHILD_TO_FAMILY_LINK = class(TTag)
  private
    f_PEDI: TTag;
    f_NOTE: TTag_NOTE_STRUCTURE;

  published
    property _PEDI: TTag read f_PEDI write f_PEDI;
    property _NOTE: TTag_NOTE_STRUCTURE read f_NOTE write f_NOTE;
  end;



  TTag_SPOUSE_TO_FAMILY_LINK = class(TTag)
  private
    f_NOTE: TTag_NOTE_STRUCTURE;
  published
    property _NOTE: TTag_NOTE_STRUCTURE read f_NOTE write f_NOTE;
  end;


  TTag_USER_REFERENCE_NUMBER = class(TTag)
  private
    f_TYPE: TTag;
  published
    property _TYPE: TTag read f_TYPE write f_TYPE;
  end;


  // Famille ***********************************************************************************

  TTag_LDS_SPOUSE_SEALING = class;
  TTag_AGE_AT_EVENT = class;


  TTag_FAM_RECORD = class(TTag)
  private
    f_CHIL: TTag;
    f_RIN: TTag;
    f_HUSB: TTag;
    f_SUBM: TTag;
    f_NCHI: TTag;
    f_WIFE: TTag;
    f_CHAN: TTag_CHANGE_DATE;
    f_EVEN: TTag_EVENT_DETAIL;
    f_ENGA: TTag_EVENT_DETAIL;
    f_DIV: TTag_EVENT_DETAIL;
    f_MARR: TTag_EVENT_DETAIL;
    f_MARS: TTag_EVENT_DETAIL;
    f_MARB: TTag_EVENT_DETAIL;
    f_ANUL: TTag_EVENT_DETAIL;
    f_MARC: TTag_EVENT_DETAIL;
    f_CENS: TTag_EVENT_DETAIL;
    f_DIVF: TTag_EVENT_DETAIL;
    f_MARL: TTag_EVENT_DETAIL;
    f_SLGS: TTag_LDS_SPOUSE_SEALING;
    f_NOTE: TTag_NOTE_STRUCTURE;
    f_SOUR: TTag_SOURCE_CITATION;
    f_REFN: TTag_USER_REFERENCE_NUMBER;

    fCLEF_HUSB: integer;
    fCLEF_WIFE: integer;
    fCLE: integer;
    fAlreadySaveThisUnion: boolean;
    f_TYPU: TTag;

  protected
    procedure DoAttachInList; override;

  public
    property CLE: integer read fCLE write fCLE; //cl du record dans la table
    property CLEF_HUSB: integer read fCLEF_HUSB write fCLEF_HUSB;
    property CLEF_WIFE: integer read fCLEF_WIFE write fCLEF_WIFE;

    property AlreadySaveThisUnion: boolean read fAlreadySaveThisUnion write fAlreadySaveThisUnion;

    procedure AfterCreate(aGedComStructure: TGedComStructure); override;

  published
    //FAMILY_EVENT_STRUCTURE        //revoir, peur-tre
    property _ANUL: TTag_EVENT_DETAIL read f_ANUL write f_ANUL;
    property _CENS: TTag_EVENT_DETAIL read f_CENS write f_CENS;
    property _DIV: TTag_EVENT_DETAIL read f_DIV write f_DIV;
    property _DIVF: TTag_EVENT_DETAIL read f_DIVF write f_DIVF;
    property _ENGA: TTag_EVENT_DETAIL read f_ENGA write f_ENGA;
    property _MARR: TTag_EVENT_DETAIL read f_MARR write f_MARR;
    property _MARB: TTag_EVENT_DETAIL read f_MARB write f_MARB;
    property _MARC: TTag_EVENT_DETAIL read f_MARC write f_MARC;
    property _MARL: TTag_EVENT_DETAIL read f_MARL write f_MARL;
    property _MARS: TTag_EVENT_DETAIL read f_MARS write f_MARS;
    property _EVEN: TTag_EVENT_DETAIL read f_EVEN write f_EVEN;


    property _HUSB: TTag read f_HUSB write f_HUSB;
    property _WIFE: TTag read f_WIFE write f_WIFE;
    property _CHIL: TTag read f_CHIL write f_CHIL;
    property _NCHI: TTag read f_NCHI write f_NCHI;
    property _SUBM: TTag read f_SUBM write f_SUBM;

    property _SLGS: TTag_LDS_SPOUSE_SEALING read f_SLGS write f_SLGS;
    property _SOUR: TTag_SOURCE_CITATION read f_SOUR write f_SOUR;

    property _NOTE: TTag_NOTE_STRUCTURE read f_NOTE write f_NOTE;
    property _REFN: TTag_USER_REFERENCE_NUMBER read f_REFN write f_REFN;
    property _RIN: TTag read f_RIN write f_RIN;
    property _CHAN: TTag_CHANGE_DATE read f_CHAN write f_CHAN;

    //type d'union ( nous)
    property _TYPU: TTag read f_TYPU write f_TYPU;

  end;


  TTag_AGE_AT_EVENT = class(TTag)
  private
    f_AGE: TTag;
  published
    property _AGE: TTag read f_AGE write f_AGE;
  end;


  TTag_LDS_SPOUSE_SEALING = class(TTag)
  private
    f_DATE: TTag;
    f_STAT: TTag;
    f_PLAC: TTag;
    f_TEMP: TTag;
    f_NOTE: TTag_NOTE_STRUCTURE;
    f_SOUR: TTag_SOURCE_CITATION;

  published
    property _STAT: TTag read f_STAT write f_STAT;
    property _DATE: TTag read f_DATE write f_DATE;
    property _TEMP: TTag read f_TEMP write f_TEMP;
    property _PLAC: TTag read f_PLAC write f_PLAC;
    property _SOUR: TTag_SOURCE_CITATION read f_SOUR write f_SOUR;
    property _NOTE: TTag_NOTE_STRUCTURE read f_NOTE write f_NOTE;

  end;

  TTag_EVENT_RECORD = class(TTag)
  private
    f_DATE: TTag;
    f_PLAC: TTag;

  published
    property _DATE: TTag read f_DATE write f_DATE;
    property _PLAC: TTag read f_PLAC write f_PLAC;
  end;

  TTag_DATA_SOURCE_RECORD = class(TTag)
  private
    f_EVEN: TTag_EVENT_RECORD;
    f_AGNC: TTag;
    f_NOTE: TTag_NOTE_STRUCTURE;

  public
    function AsLine: string;

  published
    property _EVEN: TTag_EVENT_RECORD read f_EVEN write f_EVEN;
    property _AGNC: TTag read f_AGNC write f_AGNC;
    property _NOTE: TTag_NOTE_STRUCTURE read f_NOTE write f_NOTE;

  end;


  TTag_SOURCE_RECORD = class(TTag)

  private
    f_REFN: TTag;
    f_ABBR: TTag;
    f_EVEN: TTag;
    f_CHAN: TTag_CHANGE_DATE;
    f_DATA: TTag_DATA_SOURCE_RECORD;
    f_PUBL: TTag_MEMO;
    f_AUTH: TTag_MEMO;
    f_TITL: TTag_MEMO;
    f_NOTE: TTag_NOTE_STRUCTURE;
    f_TEXT: TTag_MEMO;
    f_REPO: TTag_REPO_RECORD;

    procedure DoAttachInList; override;
  public
    function AsLine: string;

  published
    property _DATA: TTag_DATA_SOURCE_RECORD read f_DATA write f_DATA;
    property _AUTH: TTag_MEMO read f_AUTH write f_AUTH;
    property _TITL: TTag_MEMO read f_TITL write f_TITL;
    property _ABBR: TTag read f_ABBR write f_ABBR;
    property _EVEN: TTag read f_EVEN write f_EVEN;
    property _PUBL: TTag_MEMO read f_PUBL write f_PUBL;
    property _TEXT: TTag_MEMO read f_TEXT write f_TEXT;
    property _REPO: TTag_REPO_RECORD read f_REPO write f_REPO;
    property _NOTE: TTag_NOTE_STRUCTURE read f_NOTE write f_NOTE;
    property _REFN: TTag read f_REFN write f_REFN; //0:M
    property _CHAN: TTag_CHANGE_DATE read f_CHAN write f_CHAN;
  end;

  TTag_NOTE_RECORD = class(TTag)

  private

    f_CONC: TTag;
    f_CONT: TTag;
    f_SOUR: TTag_SOURCE_CITATION;
    f_REFN: TTag;
    f_CHAN: TTag_CHANGE_DATE;

    procedure DoAttachInList; override;
  public
    //le memo
    Memo: TStringList;
    procedure AfterCreate(aGedComStructure: TGedComStructure); override;
    destructor Destroy; override;
    function AsLine: string;

  published
    property _CONC: TTag read f_CONC write f_CONC;
    property _CONT: TTag read f_CONT write f_CONT;

    property _SOUR: TTag_SOURCE_CITATION read f_SOUR write f_SOUR;
    property _REFN: TTag read f_REFN write f_REFN; //0:M
    property _CHAN: TTag_CHANGE_DATE read f_CHAN write f_CHAN;
  end;

  TTag_REPO_RECORD = class(TTag)

  private
    f_REFN: TTag;
    f_NAME: TTag;
    f_ADDR: TTag_ADDRESS_STRUCTURE;
    f_CHAN: TTag_CHANGE_DATE;
    f_NOTE: TTag_NOTE_STRUCTURE;
    procedure DoAttachInList; override;
  public
    function AsLine: string;

  published
    property _NAME: TTag read f_NAME write f_NAME;
    property _ADDR: TTag_ADDRESS_STRUCTURE read f_ADDR write f_ADDR;
    property _NOTE: TTag_NOTE_STRUCTURE read f_NOTE write f_NOTE;
    property _REFN: TTag read f_REFN write f_REFN; //0:M
    property _CHAN: TTag_CHANGE_DATE read f_CHAN write f_CHAN;
  end;


implementation

uses
  u_objet_TImportGedCom, u_common_var;

{ TTag }

procedure TTag.AfterCreate(aGedComStructure: TGedComStructure);
begin
  //     fMultiple := false;
  fSimple := true;
  fClassSubTags := nil;
  fObjectSubTags := nil;
  fCaptionSubTags := nil;
  fPropInfoSubTags := nil;

  fPtrGedComStructure := aGedComStructure;
  fPtrNextSameTag := nil;

  //     fLinkLastInstance := nil;

  DoAttachInList;
  DoIntrospection;
end;


destructor TTag.Destroy;
begin
  if fClassSubTags <> nil then fClassSubTags.Free;
  if fObjectSubTags <> nil then fObjectSubTags.Free;
  if fCaptionSubTags <> nil then fCaptionSubTags.Free;
  if fPropInfoSubTags <> nil then fPropInfoSubTags.Free;

  inherited;
end;

procedure TTag.DoAttachInList;
begin
  //par dfaut, l'objet est attach dans la liste des 'OtherList'
  if fPtrGedComStructure <> nil then fPtrGedComStructure.fOtherList.Add(self);
  if fPtrGedComStructure <> nil then fPtrGedComStructure.fAllList.Add(self);
end;

procedure TTag.DoIntrospection;
var
  k, w: integer;
  fList: PPropList;
  FSize: Integer;
  FCount: Integer;
  PropInfo: PPropInfo;
  aClass: TClass;
  //    aObject: TObject;
  s: string;
begin
  FCount := GetPropList(ClassInfo, [tkClass], nil);

  //Ce Tag possde t'il d'autres Tags ?
  if FCount > 0 then
    begin
      fSimple := false;

      FSize := FCount * SizeOf(Pointer);
      GetMem(FList, FSize);
      try
        //Cration des listes d'accueil des tags
        fCaptionSubTags := TStringList.create;
        fClassSubTags := TClassList.create;
        fObjectSubTags := TObjectList.create(false);
        fPropInfoSubTags := TList.create;


        k := GetPropList(self.ClassInfo, [tkClass], FList);
        for w := 1 to k do
          begin
            s := FList^[w - 1].Name;
            PropInfo := GetPropInfo(self.ClassInfo, s);
            if PropInfo <> nil then
              begin
                SetObjectProp(self, PropInfo, nil);

                delete(s, 1, 1);

                fCaptionSubTags.Add(s);
                aClass := GetObjectPropClass(self, PropInfo);
                fClassSubTags.Add(aClass);
                fObjectSubTags.Add(nil);
                fPropInfoSubTags.Add(PropInfo);

              end;
          end;
      finally
        if FList <> nil then FreeMem(FList, FSize);
      end;
    end;
end;

function TTag.ExistSubTag(TagName: string; var index: integer): boolean;
begin
  if fSimple = false then
    begin
      index := fcaptionSubTags.IndexOf(TagName);
      result := index <> -1;
    end
  else
    result := false;
end;


constructor TGedComStructure.Create;
begin
  fIndividuList := TObjectList.create(true);
  fFamilleList := TObjectList.create(true);
  fOtherList := TObjectList.create(true);
  fNoteList := TObjectList.create(true);
  fSourceList := TObjectList.create(true);
  fRepoList := TObjectList.create(true);

  fAllList := TObjectList.create(false);

  fPtrObjetImportGedcom := nil;
end;

destructor TGedComStructure.Destroy;
begin
  fIndividuList.free;
  fFamilleList.free;
  fOtherList.free;
  fNoteList.free;
  fSourceList.free;
  fRepoList.free;
  fAllList.free;

  inherited;
end;


procedure TTag_INDIVIDUAL_RECORD.AfterCreate(aGedComStructure: TGedComStructure);
begin
  fCle := -1;
  fCLE_PARENTS := -1;
  fCLE_PERE := -1;
  fCLE_MERE := -1;

  inherited AfterCreate(aGedComStructure);
end;

procedure TTag_Individual_Record.DoAttachInList;
begin
  if fPtrGedComStructure <> nil then fPtrGedComStructure.fIndividuList.Add(self);
  if fPtrGedComStructure <> nil then fPtrGedComStructure.fAllList.Add(self);
end;

{ TTag_FAM_RECORD }

procedure TTag_FAM_RECORD.AfterCreate(aGedComStructure: TGedComStructure);
begin
  fCLE := -1;
  fCLEF_HUSB := -1;
  fCLEF_WIFE := -1;
  fAlreadySaveThisUnion := false;

  inherited AfterCreate(aGedComStructure);
end;

procedure TTag_FAM_RECORD.DoAttachInList;
begin
  if fPtrGedComStructure <> nil then fPtrGedComStructure.fFamilleList.Add(self);
  if fPtrGedComStructure <> nil then fPtrGedComStructure.fAllList.Add(self);
end;

function TGedComStructure.GetIndividuByID(ID: string): TTag_INDIVIDUAL_RECORD;
var
  n: integer;
begin
  result := nil;

  for n := 0 to IndividuList.count - 1 do
    begin
      if TTag_INDIVIDUAL_RECORD(IndividuList[n]).ID = ID then
        begin
          result := TTag_INDIVIDUAL_RECORD(IndividuList[n]);
          break;
        end;
    end;
end;


function TGedComStructure.GetFamilleByID(ID: string): TTag_FAM_RECORD;
var
  n: integer;
begin
  result := nil;

  for n := 0 to FamilleList.count - 1 do
    begin
      if TTag_FAM_RECORD(FamilleList[n]).ID = ID then
        begin
          result := TTag_FAM_RECORD(FamilleList[n]);
          break;
        end;
    end;
end;


{ TTag_MEMO }

procedure TTag_MEMO.AfterCreate(aGedComStructure: TGedComStructure);
begin
  inherited;
  Memo := TStringList.create;
end;

function TTag_MEMO.AsLine: string;
var
  n: integer;
begin
  result := Value;
  if Memo.Count > 0 then
    begin
      for n := 0 to Memo.Count - 1 do result := result + #13#10 + Memo[n];
    end;
end;

destructor TTag_MEMO.Destroy;
begin
  Memo.Free;
  inherited;
end;

procedure TGedComStructure.Clear;
begin
  fClassSubTags.Clear;
  fObjectSubTags.Clear;
  fCaptionSubTags.Clear;
  fPropInfoSubTags.Clear;

  f_TRLR := nil;
  f_HEAD := nil;
  f_INDI := nil;
  f_FAM := nil;

  fAllList.Clear;
  fIndividuList.Clear;
  fFamilleList.Clear;
  fSourceList.Clear;
  fNoteList.Clear;
  fRepoList.Clear;

  DoIntrospection;
end;

{ TTag_SOURCE_CITATION }

procedure TTag_SOURCE_CITATION.AfterCreate(
  aGedComStructure: TGedComStructure);
begin
  inherited;
  Memo := TStringList.create;
end;

function TTag_SOURCE_CITATION.AsLine: string;
var
  n: integer;
  aTEXT: TTag_MEMO;
  aNOTE: TTag_NOTE_STRUCTURE;
  aImportGedcom: TImportGedCom;
  aSour: TSour;
begin
  result := '';

  //Soit une source normal
  if ID = '' then
    begin
      result := Value;
      if Memo.Count > 0 then
        begin
          for n := 0 to Memo.Count - 1 do result := result + #13#10 + Memo[n];
        end;

      aTEXT := f_TEXT;
      while (aTEXT <> nil) do
        begin
          result := result + #13#10;
          result := result + aTEXT.AsLine;

          aTEXT := TTag_MEMO(aTEXT.PtrNextSameTag);
        end;

      aNOTE := f_NOTE;
      while (aNOTE <> nil) do
        begin
          result := result + #13#10;
          result := result + f_NOTE.AsLine;

          aNOTE := TTag_NOTE_STRUCTURE(aNOTE.PtrNextSameTag);
        end;
    end
  else
    //soit une source qui pointe ailleurs + d'autres infos
    begin
      //On cherche la source qui porte l'ID
      if fPtrGedComStructure <> nil then
        begin
          aImportGedcom := TImportGedCom(fPtrGedComStructure.PtrObjetImportGedcom);
          aSour := aImportGedcom.GetSourceByID(ID);
          if aSour <> nil then
            begin
              if aSour.GedcomBlock._SOUR <> nil then
                begin
                  result := result + aSour.GedcomBlock._SOUR.AsLine;
                end;
            end;
        end;

      if f_PAGE <> nil then
        begin
          result := result + #13#10;
          result := result + 'PAGE: ' + f_PAGE.Value;
        end;

      if f_EVEN <> nil then
        begin
          result := result + #13#10;
          result := result + 'EVENT:' + f_EVEN.Value;
          if f_EVEN._ROLE <> nil then
            begin
              result := result + #13#10;
              result := result + 'ROLE: ' + f_EVEN._ROLE.Value;
            end;
        end;

      if f_DATA <> nil then
        begin
          result := result + #13#10;
          result := result + _DATA.AsLine;
        end;

      if f_QUAY <> nil then
        begin
          result := result + #13#10;
          result := result + 'QUAY:' + f_QUAY.Value;
        end;

      aNOTE := f_NOTE;
      while (aNOTE <> nil) do
        begin
          result := result + #13#10;
          result := result + f_NOTE.AsLine;

          aNOTE := TTag_NOTE_STRUCTURE(aNOTE.PtrNextSameTag);
        end;
    end;
end;


destructor TTag_SOURCE_CITATION.Destroy;
begin
  Memo.Free;
  inherited;
end;

{ TTag_SOURCE_RECORD }

function TTag_SOURCE_RECORD.AsLine: string;
var
  aNOTE_STRUCTURE: TTag_NOTE_STRUCTURE;
  aREFN: TTag;
  //     aTagRepoRecord: TTag_REPO_RECORD;
  aImportGedcom: TImportGedCom;
  obRepo: TRepo;
begin
  result := '';


  if f_EVEN <> nil then
    begin
      result := result + #13#10;
      result := result + #13#10;
      result := result + 'EVEN: ' + f_EVEN.Value;
      s_EVEN := f_EVEN.Value;
    end;

  if f_DATA <> nil then
    begin
      result := result + #13#10;
      result := result + #13#10;
      result := result + 'DATA: ' + f_DATA.AsLine;
    end;

  if f_AUTH <> nil then
    begin
      result := result + #13#10;
      result := result + #13#10;
      result := result + 'AUTH: ' + f_AUTH.AsLine;
      s_AUTH := f_AUTH.AsLine;
    end;

  if f_TITL <> nil then
    begin
      result := result + #13#10;
      result := result + #13#10;
      result := result + 'TITL: ' + f_TITL.AsLine;
      s_TITL := f_TITL.AsLine;
    end;

  if f_ABBR <> nil then
    begin
      result := result + #13#10;
      result := result + #13#10;
      result := result + 'ABBR: ' + f_ABBR.Value;
      s_ABBR := f_ABBR.Value;
    end;

  if f_PUBL <> nil then
    begin
      result := result + #13#10;
      result := result + #13#10;
      result := result + 'PUBL: ' + f_PUBL.AsLine;
      s_PUBL := f_PUBL.AsLine;
    end;

  if f_TEXT <> nil then
    begin
      result := result + #13#10;
      result := result + #13#10;
      result := result + 'TEXT: ' + f_TEXT.AsLine; ;
      s_TEXT := f_TEXT.AsLine;
    end;

  if f_REPO <> nil then
    begin
      if f_REPO.ID <> '' then
        begin
          //On cherche le repo qui porte l'ID
          if fPtrGedComStructure <> nil then
            begin
              aImportGedcom := TImportGedCom(fPtrGedComStructure.PtrObjetImportGedcom);
              obRepo := aImportGedcom.GetRepoByID(f_REPO.ID);
              if obRepo <> nil then
                begin
                  if obRepo.GedcomBlock._REPO <> nil then
                    begin
                      result := result + obRepo.GedcomBlock._REPO.AsLine;

                      s_REPO := obRepo.GedcomBlock._REPO.AsLine;
                    end;
                end;
            end;
        end;
    end;

  aNOTE_STRUCTURE := _NOTE;
  while (aNOTE_STRUCTURE <> nil) do
    begin
      result := result + #13#10;
      result := result + #13#10;
      result := result + 'NOTE: ' + aNOTE_STRUCTURE.AsLine;

      aNOTE_STRUCTURE := TTag_NOTE_STRUCTURE(aNOTE_STRUCTURE.PtrNextSameTag);
    end;

  aREFN := _REFN;
  while (aREFN <> nil) do
    begin
      result := result + #13#10;
      result := result + 'REFN: ' + aREFN.Value;

      aREFN := TTag(aREFN.PtrNextSameTag);
    end;

  if f_CHAN <> nil then
    begin
      result := result + #13#10;
      result := result + #13#10;
      result := result + f_CHAN.AsLine;
    end;
end;

procedure TTag_SOURCE_RECORD.DoAttachInList;
begin
  if fPtrGedComStructure <> nil then fPtrGedComStructure.fSourceList.Add(self);
end;

{ TTag_NOTE_RECORD }

procedure TTag_NOTE_RECORD.AfterCreate(aGedComStructure: TGedComStructure);
begin
  inherited;
  Memo := TStringList.Create;
end;

function TTag_NOTE_RECORD.AsLine: string;
var
  n: integer;
  aSOUR: TTag_SOURCE_CITATION;
  aREFN: TTag;
begin
  result := Value;
  if Memo.Count > 0 then
    begin
      for n := 0 to Memo.Count - 1 do result := result + #13#10 + Memo[n];
    end;

  aSOUR := f_SOUR;
  while (aSOUR <> nil) do
    begin
      result := result + #13#10;
      result := result + #13#10;
      result := result + aSOUR.AsLine;

      aSOUR := TTag_SOURCE_CITATION(aSOUR.PtrNextSameTag);
    end;

  aREFN := f_REFN;
  while (aREFN <> nil) do
    begin
      result := result + #13#10;
      result := result + aSOUR.Value;
      aREFN := TTag(aREFN.PtrNextSameTag);
    end;


  if f_CHAN <> nil then
    begin
      result := result + #13#10;
      result := result + #13#10;
      result := result + f_CHAN.AsLine;
    end;
end;

destructor TTag_NOTE_RECORD.Destroy;
begin
  Memo.Free;
  inherited;
end;

procedure TTag_NOTE_RECORD.DoAttachInList;
begin
  if fPtrGedComStructure <> nil then fPtrGedComStructure.fNoteList.Add(self);
end;

{ TTag_REPO_RECORD }


procedure TTag_REPO_RECORD.DoAttachInList;
begin
  if fPtrGedComStructure <> nil then fPtrGedComStructure.fRepoList.Add(self);
end;

{ TTag_NOTE_STRUCTURE }

procedure TTag_NOTE_STRUCTURE.AfterCreate(
  aGedComStructure: TGedComStructure);
begin
  inherited;
  Memo := TStringList.create;
end;

function TTag_NOTE_STRUCTURE.AsLine: string;
var
  n: integer;
  aSOUR: TTag_SOURCE_RECORD;
  obSour: TSour;
  obNote: TNote;
  aTagSourceRecord: TTag_SOURCE_RECORD;
  aImportGedcom: TImportGedCom;
begin
  result := '';

  //soit une note simple + des sources qui pointe ailleurs
  if ID = '' then
    begin
      result := Value;
      if Memo.Count > 0 then
        begin
          for n := 0 to Memo.Count - 1 do result := result + #13#10 + Memo[n];
        end;

      aSOUR := f_SOUR;
      while (aSOUR <> nil) do
        begin
          //On cherche la source qui porte l'ID
          if fPtrGedComStructure <> nil then
            begin
              aImportGedcom := TImportGedCom(fPtrGedComStructure.PtrObjetImportGedcom);
              obSour := aImportGedcom.GetSourceByID(ID);
              if obSour <> nil then
                begin
                  if obSour.GedcomBlock._SOUR <> nil then
                    begin
                      result := result + obSour.GedcomBlock._SOUR.AsLine;
                    end;
                end;
            end;

          aSOUR := TTag_SOURCE_RECORD(aSOUR.PtrNextSameTag);
        end;
    end
  else
    //soit une note qui pointe ailleurs, et des sources qui pointent ailleurs
    begin
      //On cherche la note qui porte l'ID
      if fPtrGedComStructure <> nil then
        begin
          aImportGedcom := TImportGedCom(fPtrGedComStructure.PtrObjetImportGedcom);
          obNote := aImportGedcom.GetNoteByID(ID);
          if obNote <> nil then
            begin
              if obNote.GedcomBlock._NOTE <> nil then
                begin
                  result := result + obNote.GedcomBlock._NOTE.AsLine;
                end;
            end;
        end;


      aSOUR := f_SOUR;
      while (aSOUR <> nil) do
        begin
          //On cherche la source qui porte l'ID
          if fPtrGedComStructure <> nil then
            begin
              aTagSourceRecord := fPtrGedComStructure.GetSourceRecordByID(aSOUR.ID);
              if aTagSourceRecord <> nil then
                begin
                  result := result + aTagSourceRecord.AsLine;
                end;
            end;

          aSOUR := TTag_SOURCE_RECORD(aSOUR.PtrNextSameTag);
        end;
    end;
end;

destructor TTag_NOTE_STRUCTURE.Destroy;
begin
  Memo.Free;
  inherited;
end;

function TGedComStructure.GetSourceRecordByID(ID: string): TTag_SOURCE_RECORD;
var
  n: integer;
begin
  result := nil;

  for n := 0 to SourceList.count - 1 do
    begin
      if TTag_SOURCE_RECORD(SourceList[n]).ID = ID then
        begin
          result := TTag_SOURCE_RECORD(SourceList[n]);
          break;
        end;
    end;
end;

function TGedComStructure.GetNoteRecordByID(ID: string): TTag_NOTE_RECORD;
var
  n: integer;
begin
  result := nil;

  for n := 0 to NoteList.count - 1 do
    begin
      if TTag_NOTE_RECORD(NoteList[n]).ID = ID then
        begin
          result := TTag_NOTE_RECORD(NoteList[n]);
          break;
        end;
    end;
end;

function TGedComStructure.GetRepoRecordByID(ID: string): TTag_REPO_RECORD;
var
  n: integer;
begin
  result := nil;

  for n := 0 to RepoList.count - 1 do
    begin
      if TTag_REPO_RECORD(RepoList[n]).ID = ID then
        begin
          result := TTag_REPO_RECORD(RepoList[n]);
          break;
        end;
    end;
end;

{ TTag_DATA1 }

procedure TTag_DATA1.AfterCreate(aGedComStructure: TGedComStructure);
begin
  inherited;
  Memo := TStringList.create;
end;

function TTag_DATA1.AsLine: string;
var
  n: integer;
  aTEXT: TTag_MEMO;
begin
  result := '';
  if f_DATE <> nil then
    begin
      result := result + #13#10;
      result := result + 'DATE: ' + f_DATE.Value;
    end;

  aTEXT := _TEXT;
  while (aTEXT <> nil) do
    begin
      result := result + aText.AsLine;

      aTEXT := TTag_MEMO(aTEXT.PtrNextSameTag);
    end;

  if Memo.Count > 0 then
    begin
      result := result + #13#10;
      for n := 0 to Memo.Count - 1 do result := result + #13#10 + Memo[n];
    end;

end;

destructor TTag_DATA1.Destroy;
begin
  Memo.Free;
  inherited;
end;

{ TTag_CHANGE_DATE }

function TTag_CHANGE_DATE.AsLine: string;
var
  aNOTE_STRUCTURE: TTag_NOTE_STRUCTURE;
begin
  result := '';
  if f_DATE <> nil then
    begin
      result := result + #13#10 + 'DATE: ' + f_DATE.Value;
      if f_DATE.f_TIME <> nil then
        begin
          result := result + #13#10 + 'TIME: ' + f_DATE.f_TIME.Value;
        end;
    end;

  aNOTE_STRUCTURE := _NOTE;
  while (aNOTE_STRUCTURE <> nil) do
    begin
      result := result + #13#10;
      result := result + #13#10;
      result := result + 'NOTE: ' + aNOTE_STRUCTURE.AsLine;

      aNOTE_STRUCTURE := TTag_NOTE_STRUCTURE(aNOTE_STRUCTURE.PtrNextSameTag);
    end;
end;

{ TTag_DATA_SOURCE_RECORD }

function TTag_DATA_SOURCE_RECORD.AsLine: string;
var
  aEVEN: TTag_EVENT_RECORD;
  aNOTE_STRUCTURE: TTag_NOTE_STRUCTURE;
begin
  result := '';

  aEVEN := f_EVEN;
  while (aEVEN <> nil) do
    begin
      result := result + #13#10;
      result := result + #13#10;
      result := result + 'EVENTS: ' + aEVEN.Value;

      if aEVEN.f_DATE <> nil then
        begin
          result := result + #13#10;
          result := result + '   DATE: ' + aEVEN.f_DATE.Value;
        end;

      if aEVEN.f_PLAC <> nil then
        begin
          result := result + #13#10;
          result := result + '   PLACE: ' + aEVEN.f_PLAC.Value;
        end;

      aEVEN := TTag_EVENT_RECORD(aEVEN.PtrNextSameTag);
    end;

  if f_AGNC <> nil then
    begin
      result := result + #13#10;
      result := result + #13#10;
      result := result + 'AGNC: ' + f_AGNC.Value;
    end;

  aNOTE_STRUCTURE := _NOTE;
  while (aNOTE_STRUCTURE <> nil) do
    begin
      result := result + #13#10;
      result := result + #13#10;
      result := result + 'NOTE: ' + aNOTE_STRUCTURE.AsLine;

      aNOTE_STRUCTURE := TTag_NOTE_STRUCTURE(aNOTE_STRUCTURE.PtrNextSameTag);
    end;
end;

function TTag_REPO_RECORD.AsLine: string;
var
  aNOTE_STRUCTURE: TTag_NOTE_STRUCTURE;
  aREFN: TTag;
begin
  result := '';

  if f_NAME <> nil then
    begin
      result := result + #13#10;
      result := result + #13#10;
      result := result + 'NAME: ' + f_NAME.Value;
    end;

  if f_ADDR <> nil then
    begin
      result := result + #13#10;
      result := result + #13#10;
      result := result + 'ADDR: ' + f_ADDR.AsLine;
    end;

  aNOTE_STRUCTURE := _NOTE;
  while (aNOTE_STRUCTURE <> nil) do
    begin
      result := result + #13#10;
      result := result + #13#10;
      result := result + 'NOTE: ' + aNOTE_STRUCTURE.AsLine;

      aNOTE_STRUCTURE := TTag_NOTE_STRUCTURE(aNOTE_STRUCTURE.PtrNextSameTag);
    end;

  aREFN := _REFN;
  while (aREFN <> nil) do
    begin
      result := result + #13#10;
      result := result + 'REFN: ' + aREFN.Value;

      aREFN := TTag(aREFN.PtrNextSameTag);
    end;

  if f_CHAN <> nil then
    begin
      result := result + #13#10;
      result := result + #13#10;
      result := result + f_CHAN.AsLine;
    end;
end;


{ TTag_ADDRESS_STRUCTURE }

function TTag_ADDRESS_STRUCTURE.AsLine: string;
var
  aCONT: TTag;
begin
  result := Value;

  aCONT := f_CONT;
  while (aCONT <> nil) do
    begin
      result := result + #13#10 + aCONT.Value;
      aCONT := TTag(aCONT.PtrNextSameTag);
    end;

  // if f_ADDR <> nil then result := result + #13#10 + f_ADDR.Value;
  if f_ADR1 <> nil then result := result + #13#10 + f_ADR1.Value;
  if f_ADR2 <> nil then result := result + #13#10 + f_ADR2.Value;
  if f_CITY <> nil then result := result + #13#10 + f_CITY.Value;
  if f_STAE <> nil then result := result + #13#10 + f_STAE.Value;
  if f_POST <> nil then result := result + #13#10 + f_POST.Value;
  if f_CTRY <> nil then result := result + #13#10 + f_CTRY.Value;
  // if f_XINSEE <> nil then result := result + #13#10 + f_XINSEE.Value;
end;

end.
