{-----------------------------------------------------------------------}
{                                                                       }
{           Subprogram Name:                                            }
{           Purpose:          Ancestrologie                             }
{           Source Language:  Francais                                  }
{           Auteur:                        }
{           Philippe Cazaux-Moutou                                       }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Description:                                                }
{           Ancestrologie est un Logiciel Libre                         }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Revision History                                            }
{           v#    ,Date       ,Author Name            ,Description      }
{                                                                       }
{-----------------------------------------------------------------------}

unit u_objet_TGedcomDate;



interface

uses
  Classes, SysUtils,
  u_objet_TContext;

type
  RRecordDate = record

    //date 1
    UseDate1: boolean;

    Key1: string;
    Day1: integer;
    Month1: integer;
    Year1: integer;
    ValidDateTime1: boolean;
    DateTime1: TDateTime;

    //date 2
    UseDate2: boolean;

    Key2: string;
    Day2: integer;
    Month2: integer;
    Year2: integer;
    ValidDateTime2: boolean;
    DateTime2: TDateTime;

    Phrase: string;
  end;


  TMyStringList = class(TStringList)
  public
    procedure Assign(Source: TPersistent); override;
  end;


  TGedcomDate = class

  private
    //mots clefs utiliss
    Temp_Token_Du: TMyStringList;
    Temp_Token_Au: TMyStringList;
    Temp_Token_Avant: TMyStringList;
    Temp_Token_Apres: TMyStringList;
    Temp_Token_Entre: TMyStringList;
    Temp_Token_Et: TMyStringList;
    Temp_Token_Calculee: TMyStringList;
    Temp_Token_Estimee: TMyStringList;
    Temp_Token_Vers: TMyStringList;

    Temp_Token_Human_Month_01: TMyStringList;
    Temp_Token_Human_Month_02: TMyStringList;
    Temp_Token_Human_Month_03: TMyStringList;
    Temp_Token_Human_Month_04: TMyStringList;
    Temp_Token_Human_Month_05: TMyStringList;
    Temp_Token_Human_Month_06: TMyStringList;
    Temp_Token_Human_Month_07: TMyStringList;
    Temp_Token_Human_Month_08: TMyStringList;
    Temp_Token_Human_Month_09: TMyStringList;
    Temp_Token_Human_Month_10: TMyStringList;
    Temp_Token_Human_Month_11: TMyStringList;
    Temp_Token_Human_Month_12: TMyStringList;

    Temp_Separator_Human_Date: TMyStringList;

    //recherche un token compos d'un seul mot-clef
    function GetNextToken(s: string; var token, NewS: string): boolean;

    //recherche un token compos de plusieurs mots
    function FindKeyToken(s: string; var token, NewS: string): boolean;

    //dcodage d'une date depuis le format gedcom
    function determineDate(line: string; var jour, mois, annee: integer; var KnowFormatDateTime: boolean; var theDate: TDateTime; var NewS: string): boolean;

    //dcodage d'une date saisie par l'utilisateur
    function determineHumanDate(line: string; var jour, mois, annee: integer; var KnowFormatDateTime: boolean; var theDate: TDateTime; var NewS: string): boolean;

    //vrification des parties d'une date
    function IsDay(s: string; var iDay: integer): boolean;
    function IsMonth(s: string; var iMonth: integer): boolean;
    function IsHumanMonth(s: string; var iMonth: integer): boolean;
    function IsYear(s: string; var iYear: integer): boolean;

    //vrification qu'une date est dans un format long, ou avec sparateur ("/", ".", ...)
    function ExistSeparatorHumanDateInToken(token: string; var Sep: string): boolean;

    //conversion d'un numro de mois en nom gedcom



    function MonthNum2MonthHuman(month: integer): string;
    function GedcomKey2HumanKey(key: string): string;



  public
    GDate: RRecordDate;

    constructor Create;
    destructor Destroy; override;

    function DecodeHumanDate(line: string): boolean;
    function DecodeGedcomDate(line: string): boolean;
    function EncodeGedcomDate: string;
    function EncodeHumanDate: string;
    function MonthNum2MonthGedcom(month: integer): string;

    function KnowDate: boolean;
    function KnowYear: boolean;

    procedure AssignTokens;
  end;


implementation

uses
  u_gedcom_var,
  u_common_var;

function AssembleString(s: array of string; Separator: string): string;
var
  i: Integer;
  sub: string;
begin
  result := '';
  for i := 0 to High(s) do
    begin
      sub := trim(s[i]);
      if sub <> '' then
        begin
          if (result <> '') and (i > 0) then result := result + Separator;
          result := result + sub;
        end;
    end;
end;


constructor TGedcomDate.Create;
begin
  Temp_Token_Du := TMyStringList.create;
  Temp_Token_Au := TMyStringList.create;
  Temp_Token_Avant := TMyStringList.create;
  Temp_Token_Apres := TMyStringList.create;
  Temp_Token_Entre := TMyStringList.create;
  Temp_Token_Et := TMyStringList.create;
  Temp_Token_Calculee := TMyStringList.create;
  Temp_Token_Estimee := TMyStringList.create;
  Temp_Token_Vers := TMyStringList.create;


  Temp_Token_Human_Month_01 := TMyStringList.create;
  Temp_Token_Human_Month_02 := TMyStringList.create;
  Temp_Token_Human_Month_03 := TMyStringList.create;
  Temp_Token_Human_Month_04 := TMyStringList.create;
  Temp_Token_Human_Month_05 := TMyStringList.create;
  Temp_Token_Human_Month_06 := TMyStringList.create;
  Temp_Token_Human_Month_07 := TMyStringList.create;
  Temp_Token_Human_Month_08 := TMyStringList.create;
  Temp_Token_Human_Month_09 := TMyStringList.create;
  Temp_Token_Human_Month_10 := TMyStringList.create;
  Temp_Token_Human_Month_11 := TMyStringList.create;
  Temp_Token_Human_Month_12 := TMyStringList.create;

  Temp_Separator_Human_Date := TMyStringList.create;
end;

destructor TGedcomDate.Destroy;
begin
  Temp_Token_Du.free;
  Temp_Token_Au.free;
  Temp_Token_Avant.free;
  Temp_Token_Apres.free;
  Temp_Token_Entre.free;
  Temp_Token_Et.free;
  Temp_Token_Calculee.free;
  Temp_Token_Estimee.free;
  Temp_Token_Vers.free;

  Temp_Token_Human_Month_01.free;
  Temp_Token_Human_Month_02.free;
  Temp_Token_Human_Month_03.free;
  Temp_Token_Human_Month_04.free;
  Temp_Token_Human_Month_05.free;
  Temp_Token_Human_Month_06.free;
  Temp_Token_Human_Month_07.free;
  Temp_Token_Human_Month_08.free;
  Temp_Token_Human_Month_09.free;
  Temp_Token_Human_Month_10.free;
  Temp_Token_Human_Month_11.free;
  Temp_Token_Human_Month_12.free;

  Temp_Separator_Human_Date.free;

  inherited;
end;



function TGedcomDate.DecodeGedcomDate(line: string): boolean;
var
  token, newline: string;
  k: integer;
  KEY1, KEY2: string;
  PHRASE: string;
  DATE_CALENDAR_ESCAPE: string;

  jour1, mois1, annee1: integer;
  KnowFormatDateTime1: boolean;
  theDate1: TDateTime;

  jour2, mois2, annee2: integer;
  KnowFormatDateTime2: boolean;
  theDate2: TDateTime;

  suite: boolean;
begin
  result := false;

  //on recherche le premier terme
  line := trim(line);
  Key1 := '';
  Key2 := '';
  Phrase := '';

  GDate.UseDate1 := false;
  GDate.Day1 := -1;
  GDate.Month1 := -1;
  GDate.Year1 := -1;
  GDate.ValidDateTime1 := false;
  GDate.Key1 := '';

  GDate.UseDate2 := false;
  GDate.Day2 := -1;
  GDate.Month2 := -1;
  GDate.Year2 := -1;
  GDate.ValidDateTime2 := false;
  GDate.Key2 := '';

  GDate.Phrase := '';

  if line <> '' then
    begin
      //soit la date est inconnue, car sous forme de phrase
      if line[1] = '(' then
        begin
          result := true;
          GDate.Phrase := copy(line, 2, length(line) - 2);
        end
      else
        begin
          //on cherche le premier terme
          if GetNextToken(line, token, NewLine) then
            begin
              if (token = 'FROM')
                or (token = 'TO')
                or (token = 'BEF')
                or (token = 'AFT')
                or (token = 'BET')
                or (token = 'ABT')
                or (token = 'CAL')
                or (token = 'EST')
                then
                begin
                  Key1 := token;
                  Line := NewLine;
                end
              else
                if (token = 'INT') then
                  begin
                    //on se dbarasse de la phrase qui doit se trouver  la fin
                    k := pos('(', NewLine);
                    if k <> 0 then
                      begin
                        Key1 := token;
                        Phrase := copy(NewLine, k + 1, length(NewLine) - k - 1);
                        Line := copy(NewLine, 1, k - 1);
                      end;
                  end;


              //soit une DATE_CALENDAR_ESCAPE
              if (copy(line, 1, 10) = '@#DHEBREW@') then Token := '@#DHEBREW@'
              else
                if (copy(line, 1, 9) = '@#DROMAN@') then token := '@#DROMAN@'
                else
                  if (copy(line, 1, 12) = '@#DFRENCH R@') then token := '@#DFRENCH R@'
                  else
                    if (copy(line, 1, 13) = '@#DGREGORIAN@') then token := '@#DGREGORIAN@'
                    else
                      if (copy(line, 1, 10) = '@#DJULIAN@') then token := '@#DJULIAN@'
                      else
                        if (copy(line, 1, 11) = '@#DUNKNOWN@') then token := '@#DUNKNOWN@'
                        else
                          token := '';

              //on enlve ce token de la ligne
              if Token <> '' then
                begin
                  delete(line, 1, length(token));
                  line := trim(line);
                  DATE_CALENDAR_ESCAPE := token;
                end
              else
                DATE_CALENDAR_ESCAPE := '@#DGREGORIAN@';


              //on ne peux convertir que si la date est en calendrier Gregorien
              if DATE_CALENDAR_ESCAPE = '@#DGREGORIAN@' then
                begin
                  if determineDate(line, jour1, mois1, annee1, KnowFormatDateTime1, theDate1, NewLine) then
                    begin
                      line := NewLine;

                      if Key1 = 'FROM' then
                        begin
                          suite := false;
                          //on regarde si on trouve le mot-clef 'TO'
                          if GetNextToken(line, token, NewLine) then
                            begin
                              if token = 'TO' then
                                begin
                                  line := NewLine;
                                  if determineDate(line, jour2, mois2, annee2, KnowFormatDateTime2, theDate2, NewLine) then
                                    begin
                                      GDate.UseDate2 := true;

                                      GDate.Day2 := jour2;
                                      GDate.Month2 := mois2;
                                      GDate.Year2 := annee2;
                                      GDate.ValidDateTime2 := KnowFormatDateTime2;
                                      GDate.DateTime2 := theDate2;
                                      GDate.Key2 := token;

                                      suite := true;
                                    end;
                                end;
                            end
                          else
                            begin
                              //pas de problme : le 'TO' est facultatif
                              suite := true;
                            end;
                        end
                      else
                        if Key1 = 'BET' then
                          begin
                            suite := false;
                            //on regarde si on trouve le mot-clef 'AND'
                            if GetNextToken(line, token, NewLine) then
                              begin
                                if token = 'AND' then
                                  begin
                                    line := NewLine;
                                    if determineDate(line, jour2, mois2, annee2, KnowFormatDateTime2, theDate2, NewLine) then
                                      begin
                                        GDate.UseDate2 := true;

                                        GDate.Day2 := jour2;
                                        GDate.Month2 := mois2;
                                        GDate.Year2 := annee2;
                                        GDate.ValidDateTime2 := KnowFormatDateTime2;
                                        GDate.DateTime2 := theDate2;
                                        GDate.Key2 := token;

                                        suite := true;
                                      end;
                                  end;
                              end;
                          end
                        else
                          suite := true;

                      if suite then
                        begin
                          GDate.UseDate1 := true;
                          GDate.Day1 := jour1;
                          GDate.Month1 := mois1;
                          GDate.Year1 := annee1;
                          GDate.ValidDateTime1 := KnowFormatDateTime1;
                          GDate.DateTime1 := theDate1;
                          GDate.Key1 := Key1;

                          GDate.Phrase := phrase;

                          result := true;
                        end;
                    end;
                end;
            end;
        end;
    end;
end;


{
function TGedcomDate.determineDate(line: string; var jour, mois, annee: integer; var KnowFormatDateTime: boolean; var theDate: TDateTime; var NewS: string): boolean;
var
     t1, t2, t3: string;
     s1, s2, s3: string;
begin
     result := false;
     KnowFormatDateTime := false;
     jour := -1;
     mois := -1;
     annee := -1;

     t1 := ''; s1 := '';
     t2 := ''; s2 := '';
     t3 := ''; s3 := '';

     //y-a-t'il au moins un terme ?
     if GetNextToken(line, t1, s1) then
     begin
          //un autre ?
          if GetNextToken(s1, t2, s2) then
          begin
               //un autre terme ?
               if GetNextToken(s2, t3, s3) then begin end;
          end
     end;


     //soit on connait 3 termes
     if t3 <> '' then
     begin
          //t3 est une anne ?
          if IsYear(t3, annee) then
          begin
               if IsMonth(t2, mois) then
               begin
                    if isDay(t1, jour) then
                    begin
                         //on vrifie la date
                         try
                              theDate := EncodeDate(annee, mois, jour);
                              result := true;
                              KnowFormatDateTime := true;
                              NewS := s3;
                         except
                         end;
                    end;
               end;
          end
          else
          begin
               if IsYear(t2, annee) then
               begin
                    if IsMonth(t1, mois) then
                    begin
                         result := true;
                         NewS := s2;
                    end;
               end
               else
               begin
                    if IsYear(t1, annee) then
                    begin
                         result := true;
                         NewS := s1;
                    end;
               end;
          end;
     end
     else
          //soit on connait que 2 termes
          if t2 <> '' then
     begin
          //t2 est une anne ?
          if IsYear(t2, annee) then
          begin
               if IsMonth(t1, mois) then
               begin
                    result := true;
                    NewS := s2;
               end;
          end
          else
          begin
               if IsYear(t1, annee) then
               begin
                    result := true;
                    NewS := s1;
               end;
          end;
     end
     else
          //soit on connait un seul terme
          if t1 <> '' then
     begin
          //c'est une anne ?
          if IsYear(t1, annee) then
          begin
               result := true;
               NewS := s1;
          end;
     end;
end;
}


function TGedcomDate.determineDate(line: string; var jour, mois, annee: integer; var KnowFormatDateTime: boolean; var theDate: TDateTime; var NewS: string): boolean;
var
  t1, t2, t3: string;
  s1, s2, s3: string;
begin
  result := false;
  KnowFormatDateTime := false;
  jour := -1;
  mois := -1;
  annee := -1;

  t1 := ''; s1 := '';
  t2 := ''; s2 := '';
  t3 := ''; s3 := '';

  //y-a-t'il au moins un terme ?
  if GetNextToken(line, t1, s1) then
    begin
      //un autre ?
      if GetNextToken(s1, t2, s2) then
        begin
          //un autre terme ?
          if GetNextToken(s2, t3, s3) then begin end;
        end
    end;


  //soit on connait 3 termes
  if t3 <> '' then
    begin
      //t3 est une anne ?
      if (IsYear(t3, annee)) and (IsMonth(t2, mois)) and (isDay(t1, jour)) then
        begin
          //on vrifie la date
          try
            theDate := EncodeDate(annee, mois, jour);
            result := true;
            KnowFormatDateTime := true;
            NewS := s3;
          except
          end;
        end
      else
        if (IsYear(t2, annee)) and (IsMonth(t1, mois)) then
          begin
            result := true;
            NewS := s2;
          end
        else
          if IsYear(t1, annee) then
            begin
              result := true;
              NewS := s1;
            end;
    end
  else
    //soit on connait que 2 termes
    if t2 <> '' then
      begin
        //t2 est une anne ?
        if (IsYear(t2, annee)) and (IsMonth(t1, mois)) then
          begin
            result := true;
            NewS := s2;
          end
        else
          if IsYear(t1, annee) then
            begin
              result := true;
              NewS := s1;
            end;
      end
    else
      //soit on connait un seul terme
      if t1 <> '' then
        begin
          //c'est une anne ?
          if IsYear(t1, annee) then
            begin
              result := true;
              NewS := s1;
            end;
        end;
end;



function TGedcomDate.GetNextToken(s: string; var token, NewS: string): boolean;
var
  k: integer;
begin
  result := false;
  s := trim(s);
  if s <> '' then
    begin
      k := pos(' ', s);

      //soit pas d'autres espace
      if k = 0 then
        begin
          Token := s;
          NewS := '';
          result := true;
        end
      else
        //soit plusieurs termes
        begin
          Token := copy(s, 1, k - 1);
          NewS := copy(s, k + 1, length(s) - k);
          result := true;
        end;
    end;
end;


function TGedcomDate.FindKeyToken(s: string; var token, NewS: string): boolean;

  function FindKey(list: TStringList): boolean;
  var
    n, k, p: integer;
  begin
    result := false;
    k := 0;
    for n := 0 to list.count - 1 do
      begin
        p := length(list[n]);
        if copy(s, 1, p) = list[n] then
          begin
            if (length(s) > p) and (s[p + 1] = ' ') then
              begin
                result := true;
                if p > k then
                  begin
                    k := p;
                    token := list[n];
                  end;
              end;
          end;
      end;

    if result then
      begin
        delete(s, 1, length(token));
        NewS := trim(s);
      end;
  end;

begin
  s := ansiuppercase(s);
  result := true;

  if FindKey(Temp_Token_Du) then exit
  else
    if FindKey(Temp_Token_Au) then exit
    else
      if FindKey(Temp_Token_Avant) then exit
      else
        if FindKey(Temp_Token_Apres) then exit
        else
          if FindKey(Temp_Token_Entre) then exit
          else
            if FindKey(Temp_Token_Et) then exit
            else
              if FindKey(Temp_Token_Calculee) then exit
              else
                if FindKey(Temp_Token_Estimee) then exit
                else
                  if FindKey(Temp_Token_Vers) then exit
                  else
                    result := false;
end;


function TGedcomDate.IsYear(s: string; var iYear: integer): boolean;
begin
  try
    iYear := StrToInt(s);
    result := true;
  except
    result := false;
  end;
end;

function TGedcomDate.IsHumanMonth(s: string; var iMonth: integer): boolean;
var
  m: integer;
  suite: boolean;
begin
  //mois d'un calendrier grgorien
  result := true;
  suite := true;
  //on test le mois sous forme de chiffre
  try
    m := StrToInt(s);
    if (m >= 1) and (m <= 12) then
      begin
        iMonth := m;
        suite := false;
      end;
  except
  end;



  if suite then
    begin
      if (Temp_Token_Human_Month_01.IndexOf(s) <> -1) then iMonth := 1
      else
        if (Temp_Token_Human_Month_02.IndexOf(s) <> -1) then iMonth := 2
        else
          if (Temp_Token_Human_Month_03.IndexOf(s) <> -1) then iMonth := 3
          else
            if (Temp_Token_Human_Month_04.IndexOf(s) <> -1) then iMonth := 4
            else
              if (Temp_Token_Human_Month_05.IndexOf(s) <> -1) then iMonth := 5
              else
                if (Temp_Token_Human_Month_06.IndexOf(s) <> -1) then iMonth := 6
                else
                  if (Temp_Token_Human_Month_07.IndexOf(s) <> -1) then iMonth := 7
                  else
                    if (Temp_Token_Human_Month_08.IndexOf(s) <> -1) then iMonth := 8
                    else
                      if (Temp_Token_Human_Month_09.IndexOf(s) <> -1) then iMonth := 9
                      else
                        if (Temp_Token_Human_Month_10.IndexOf(s) <> -1) then iMonth := 10
                        else
                          if (Temp_Token_Human_Month_11.IndexOf(s) <> -1) then iMonth := 11
                          else
                            if (Temp_Token_Human_Month_12.IndexOf(s) <> -1) then iMonth := 12
                            else
                              result := false;
    end;
end;

function TGedcomDate.IsDay(s: string; var iDay: integer): boolean;
begin
  try
    iDay := StrToInt(s);
    result := true;
  except
    result := false;
  end;
end;


function TGedcomDate.DecodeHumanDate(line: string): boolean;
var
  line0min, line0maj: string;
  token, newline: string;
  k: integer;
  KEY1, KEY2: string;
  PHRASE: string;

  jour1, mois1, annee1: integer;
  KnowFormatDateTime1: boolean;
  theDate1: TDateTime;

  jour2, mois2, annee2: integer;
  KnowFormatDateTime2: boolean;
  theDate2: TDateTime;

  suite: boolean;
begin
  result := false;


  //on recherche le premier terme
  repeat
    k := pos('  ', line);
    if k > 0 then delete(line, k, 1);
  until k = 0;

  line := trim(line);
  line0min := line;
  line := AnsiUpperCase(line);
  line0maj := line;

  Key1 := '';
  Key2 := '';
  Phrase := '';

  GDate.UseDate1 := false;
  GDate.Day1 := -1;
  GDate.Month1 := -1;
  GDate.Year1 := -1;
  GDate.ValidDateTime1 := false;
  GDate.Key1 := '';

  GDate.UseDate2 := false;
  GDate.Day2 := -1;
  GDate.Month2 := -1;
  GDate.Year2 := -1;
  GDate.ValidDateTime2 := false;
  GDate.Key2 := '';

  GDate.Phrase := '';

  if line <> '' then
    begin
      //on cherche le premier terme
      if FindKeyToken(line, token, NewLine) then
        begin
          if (Temp_Token_Du.indexOf(token) <> -1) then
            begin
              Key1 := 'FROM';
              Line := NewLine;
            end
          else
            if (Temp_Token_Au.IndexOf(token) <> -1) then
              begin
                Key1 := 'TO';
                Line := NewLine;
              end
            else
              if (Temp_Token_Avant.IndexOf(token) <> -1) then
                begin
                  Key1 := 'BEF';
                  Line := NewLine;
                end
              else
                if (Temp_Token_Apres.IndexOf(token) <> -1) then
                  begin
                    Key1 := 'AFT';
                    Line := NewLine;
                  end
                else
                  if (Temp_Token_Entre.IndexOf(token) <> -1) then
                    begin
                      Key1 := 'BET';
                      Line := NewLine;
                    end
                  else
                    if (Temp_Token_Vers.IndexOf(token) <> -1) then
                      begin
                        Key1 := 'ABT';
                        Line := NewLine;
                      end
                    else
                      if (Temp_Token_Calculee.IndexOf(token) <> -1) then
                        begin
                          Key1 := 'CAL';
                          Line := NewLine;
                        end
                      else
                        if (Temp_Token_Estimee.IndexOf(token) <> -1) then
                          begin
                            Key1 := 'EST';
                            Line := NewLine;
                          end;
        end;

      //on doit trouver une date
      if determineHumanDate(line, jour1, mois1, annee1, KnowFormatDateTime1, theDate1, NewLine) then
        begin
          line := NewLine;

          if (Key1 = 'FROM') then //FROM
            begin
              suite := false;
              //on regarde si on trouve le mot-clef 'TO'
              if FindKeyToken(line, token, NewLine) then
                begin
                  if (Temp_Token_Au.IndexOf(token) <> -1) then //TO
                    begin
                      line := NewLine;
                      if determineHumanDate(line, jour2, mois2, annee2, KnowFormatDateTime2, theDate2, NewLine) then
                        begin
                          if trim(NewLine) = '' then
                            begin
                              GDate.UseDate2 := true;

                              GDate.Day2 := jour2;
                              GDate.Month2 := mois2;
                              GDate.Year2 := annee2;
                              GDate.ValidDateTime2 := KnowFormatDateTime2;
                              GDate.DateTime2 := theDate2;
                              GDate.Key2 := 'TO';

                              suite := true;
                            end;
                        end;
                    end;
                end
              else
                begin
                  //pas de problme : le 'TO' est facultatif
                  suite := true;
                end;
            end
          else
            if (Key1 = 'BET') then //BET
              begin
                suite := false;
                //on regarde si on trouve le mot-clef 'AND'
                if FindKeyToken(line, token, NewLine) then
                  begin
                    if (Temp_Token_Et.IndexOf(token) <> -1) then
                      begin
                        line := NewLine;
                        if determineHumanDate(line, jour2, mois2, annee2, KnowFormatDateTime2, theDate2, NewLine) then
                          begin
                            //si il reste encore quelque chose
                            if trim(NewLine) = '' then
                              begin
                                GDate.UseDate2 := true;

                                GDate.Day2 := jour2;
                                GDate.Month2 := mois2;
                                GDate.Year2 := annee2;
                                GDate.ValidDateTime2 := KnowFormatDateTime2;
                                GDate.DateTime2 := theDate2;
                                GDate.Key2 := 'AND';

                                suite := true;
                              end;
                          end;
                      end;
                  end;
              end
            else
              begin
                line := trim(line);

                //soit il reste un commentaire
                if line <> '' then
                  begin
                    //un commentaire est-il possible ? oui, si Key1=''
                    if Key1 = '' then
                      begin
                        // Key1 := 'INT';
                        Phrase := line;
                        suite := true;
                      end
                    else
                      suite := false;
                  end
                else
                  begin
                    suite := true;
                    phrase := '';
                  end;
              end;

          if suite then
            begin
              GDate.UseDate1 := true;
              GDate.Day1 := jour1;
              GDate.Month1 := mois1;
              GDate.Year1 := annee1;
              GDate.ValidDateTime1 := KnowFormatDateTime1;
              GDate.DateTime1 := theDate1;
              GDate.Key1 := Key1;

              k := pos(phrase, line0maj);
              if k > 0 then GDate.Phrase := trim(copy(line0min, k, length(phrase)));

              result := true;
            end;

        end;
    end;

  if result = false then GDate.Phrase := line0min;
end;


function TGedcomDate.ExistSeparatorHumanDateInToken(token: string; var Sep: string): boolean;
var
  n: integer;
begin
  result := false;
  for n := 0 to Temp_Separator_Human_Date.count - 1 do
    begin
      if pos(Temp_Separator_Human_Date[n], token) <> 0 then
        begin
          Sep := Temp_Separator_Human_Date[n];
          result := true;
          break;
        end;
    end;
end;


function TGedcomDate.determineHumanDate(line: string; var jour, mois, annee: integer; var KnowFormatDateTime: boolean; var theDate: TDateTime; var NewS: string): boolean;
var

  t0, t1, t2, t3: string;
  s0, s1, s2, s3: string;
  sep: string;
  k: integer;
begin
  result := false;
  KnowFormatDateTime := false;
  jour := -1;
  mois := -1;
  annee := -1;

  t1 := ''; s1 := '';
  t2 := ''; s2 := '';
  t3 := ''; s3 := '';

  //y-a-t'il au moins un terme ?
  if GetNextToken(line, t1, s1) then
    begin
      //si dans ce terme, on trouve un sparateur de date, alors on ne cherche pas plus loin
      if ExistSeparatorHumanDateInToken(t1, sep) then
        begin
          //le premier terme est donc peut-tre une date, crite avec des sparateurs plutot qu'avec des espaces
          t0 := t1;
          s0 := s1;
          k := pos(sep, t0);
          if k <> 0 then
            begin
              t1 := copy(t0, 1, k - 1);
              s1 := copy(t0, k + 1, length(t0) - k);

              //deuxime terme ?
              k := pos(sep, s1);
              if k <> 0 then
                begin
                  t2 := copy(s1, 1, k - 1);
                  s2 := copy(s1, k + 1, length(s1) - k);

                  //troisime terme ?
                  if s2 <> '' then
                    begin
                      t3 := s2;
                      s3 := s0;
                    end
                  else
                    s2 := trim(s2 + ' ' + s0);
                end
              else
                begin
                  t2 := s1;
                  s2 := s0;
                  //                         s1 := trim(s1 + ' ' + s0);


                end;
            end;
        end
      else
        begin
          //un autre ?
          if GetNextToken(s1, t2, s2) then
            begin
              //un autre terme ?
              if GetNextToken(s2, t3, s3) then begin end;
            end;
        end;
    end;




  (*
  //soit on connait 3 termes
  if t3 <> '' then
  begin
       //t3 est une anne ?
       if IsYear(t3, annee) then
       begin
            if IsHumanMonth(t2, mois) then
            begin
                 if isDay(t1, jour) then
                 begin
                      //on vrifie la date
                      try
                           theDate := EncodeDate(annee, mois, jour);
                           result := true;
                           KnowFormatDateTime := true;
                           NewS := s3;
                      except
                      end;
                 end;
            end;
       end
       else
       begin
            if IsYear(t2, annee) then
            begin
                 if IsHumanMonth(t1, mois) then
                 begin
                      result := true;
                      NewS := s2;
                 end;
            end
            else
            begin
                 if IsYear(t1, annee) then
                 begin
                      result := true;
                      NewS := s1;
                 end;
            end;
       end;
  end
  else
       //soit on connait que 2 termes
       if t2 <> '' then
  begin
       //t2 est une anne ?
       if IsYear(t2, annee) then
       begin
            if IsHumanMonth(t1, mois) then
            begin
                 result := true;
                 NewS := s2;
            end;
       end
       else
       begin
            if IsYear(t1, annee) then
            begin
                 result := true;
                 NewS := s1;
            end;
       end;
  end
  else
       //soit on connait un seul terme
       if t1 <> '' then
  begin
       //c'est une anne ?
       if IsYear(t1, annee) then
       begin
            result := true;
            NewS := s1;
       end;
  end;
  *)

  //soit on connait 3 termes
  if t3 <> '' then
    begin
      if (IsYear(t3, annee)) and (IsHumanMonth(t2, mois)) and (isDay(t1, jour)) then
        begin
          //on vrifie la date
          try
            theDate := EncodeDate(annee, mois, jour);
            result := true;
            KnowFormatDateTime := true;
            NewS := s3;
          except
          end;
        end
      else
        if (IsYear(t2, annee)) and (IsHumanMonth(t1, mois)) then
          begin
            result := true;
            NewS := s2;
          end
        else
          if (IsYear(t1, annee)) then
            begin
              result := true;
              NewS := s1;
            end;
    end
  else
    //soit on connait que 2 termes
    if t2 <> '' then
      begin
        //t2 est une anne ?
        if (IsYear(t2, annee)) and (IsHumanMonth(t1, mois)) then
          begin
            result := true;
            NewS := s2;
          end
        else
          if IsYear(t1, annee) then
            begin
              result := true;
              NewS := s1;
            end;
      end
    else
      //soit on connait un seul terme
      if t1 <> '' then
        begin
          //c'est une anne ?
          if IsYear(t1, annee) then
            begin
              result := true;
              NewS := s1;
            end;
        end;
end;


function TGedcomDate.IsMonth(s: string; var iMonth: integer): boolean;
begin
  //mois d'un calendrier grgorien
  //mois d'un calendrier grgorien

  // Ici correction PCM -- du 01/02/2002 ------------------------------

  result := true;
  if UpperCase(s) = 'JAN' then iMonth := 1
  else
    if UpperCase(s) = 'FEB' then iMonth := 2
    else
      if UpperCase(s) = 'MAR' then iMonth := 3
      else
        if UpperCase(s) = 'APR' then iMonth := 4
        else
          if UpperCase(s) = 'MAY' then iMonth := 5
          else
            if UpperCase(s) = 'JUN' then iMonth := 6
            else
              if UpperCase(s) = 'JUL' then iMonth := 7
              else
                if UpperCase(s) = 'AUG' then iMonth := 8
                else
                  if UpperCase(s) = 'SEP' then iMonth := 9
                  else
                    if UpperCase(s) = 'OCT' then iMonth := 10
                    else
                      if UpperCase(s) = 'NOV' then iMonth := 11
                      else
                        if UpperCase(s) = 'DEC' then iMonth := 12
                        else
                          result := false;
end;


function TGedcomDate.MonthNum2MonthGedcom(month: integer): string;
begin
  case month of
    1: result := 'JAN';
    2: result := 'FEB';
    3: result := 'MAR';
    4: result := 'APR';
    5: result := 'MAY';
    6: result := 'JUN';
    7: result := 'JUL';
    8: result := 'AUG';
    9: result := 'SEP';
    10: result := 'OCT';
    11: result := 'NOV';
    12: result := 'DEC';
    else
      result := '';
  end;
end;


{ TMyTStringList }

procedure TMyStringList.Assign(Source: TPersistent);
var
  n: integer;
begin
  if Source is TStrings then
    begin
      BeginUpdate;
      try
        Clear;
        for n := 0 to TStrings(Source).Count - 1 do
          Add(AnsiUpperCase(TStrings(Source)[n]));
      finally
        EndUpdate;
      end;
    end;
end;


function TGedcomDate.EncodeGedcomDate: string;

  function AssembleGedcomDate(Day, Month, Year: integer): string;
  var
    d, m, y: string;
  begin
    if Year <> -1 then y := inttostr(Year) else y := '';
    m := MonthNum2MonthGedcom(Month);
    if Day <> -1 then d := inttostr(Day) else d := '';
    result := assembleString([d, m, y], ' ');
  end;

begin
  result := '';

  //soit que du commentaire
  if (GDate.UseDate1 = false) and (GDate.Phrase <> '') then
    begin
      result := '(' + GDate.Phrase + ')';
    end
  else
    begin
      if (GDate.UseDate1) and (GDate.UseDate2 = false) then
        begin
          result := AssembleString([GDate.Key1, AssembleGedcomDate(GDate.Day1, GDate.Month1, GDate.Year1)], ' ');
          if (GDate.Phrase <> '') and (GDate.Key1 = '') then
            begin
              result := assembleString(['INT', result, '(' + GDate.Phrase + ')'], ' ');
            end;
        end
      else
        if (GDate.UseDate1) and (GDate.UseDate2) then
          begin
            result := AssembleString([
              GDate.Key1,
                AssembleGedcomDate(GDate.Day1, GDate.Month1, GDate.Year1),
                GDate.Key2,
                AssembleGedcomDate(GDate.Day2, GDate.Month2, GDate.Year2)],
                ' ');
          end;
    end;
end;


procedure TGedcomDate.AssignTokens;
begin
  Temp_Token_Du.Assign(Token_Du);
  Temp_Token_Au.Assign(Token_Au);
  Temp_Token_Avant.Assign(Token_Avant);
  Temp_Token_Apres.Assign(Token_Apres);
  Temp_Token_Entre.Assign(Token_Entre);
  Temp_Token_Et.Assign(Token_Et);
  Temp_Token_Calculee.Assign(Token_Calculee);
  Temp_Token_Estimee.Assign(Token_Estimee);
  Temp_Token_Vers.Assign(Token_Vers);

  Temp_Token_Human_Month_01.assign(Token_Human_Month_01);
  Temp_Token_Human_Month_02.assign(Token_Human_Month_02);
  Temp_Token_Human_Month_03.assign(Token_Human_Month_03);
  Temp_Token_Human_Month_04.assign(Token_Human_Month_04);
  Temp_Token_Human_Month_05.assign(Token_Human_Month_05);
  Temp_Token_Human_Month_06.assign(Token_Human_Month_06);
  Temp_Token_Human_Month_07.assign(Token_Human_Month_07);
  Temp_Token_Human_Month_08.assign(Token_Human_Month_08);
  Temp_Token_Human_Month_09.assign(Token_Human_Month_09);
  Temp_Token_Human_Month_10.assign(Token_Human_Month_10);
  Temp_Token_Human_Month_11.assign(Token_Human_Month_11);
  Temp_Token_Human_Month_12.assign(Token_Human_Month_12);

  Temp_Separator_Human_Date.assign(Separator_Human_Date);
end;

function TGedcomDate.KnowDate: boolean;
begin
  result := (GDate.UseDate1) and (GDate.UseDate2 = false) and (GDate.ValidDateTime1);
end;

function TGedcomDate.KnowYear: boolean;
begin
  result := (GDate.UseDate1) and (GDate.UseDate2 = false) and (GDate.Year1 <> -1) and (GDate.Key1 = '');
end;

function TGedcomDate.EncodeHumanDate: string;
var
  s1, s2, d, m, y: string;
begin
  s1 := '';
  if GDate.UseDate1 then
    begin
      d := '';
      m := '';
      y := '';

      if GDate.Day1 <> -1 then d := inttostr(GDate.Day1);
      if GDate.Month1 <> -1 then m := MonthNum2MonthHuman(GDate.Month1);
      if GDate.Year1 <> -1 then y := inttostr(GDate.Year1);

      s1 := AssembleString([GedcomKey2HumanKey(GDate.Key1), d, m, y], ' ');
    end;

  s2 := '';
  if GDate.UseDate1 then
    begin
      d := '';
      m := '';
      y := '';

      if GDate.Day2 <> -1 then d := inttostr(GDate.Day2);
      if GDate.Month2 <> -1 then m := MonthNum2MonthHuman(GDate.Month2);
      if GDate.Year2 <> -1 then y := inttostr(GDate.Year2);

      s2 := AssembleString([GedcomKey2HumanKey(GDate.Key2), d, m, y], ' ');
    end;

  result := AssembleString([s1, s2, GDate.Phrase], ' ');
end;

function TGedcomDate.MonthNum2MonthHuman(month: integer): string;
var
  s: TStringList;
begin
  case month of
    1: s := Token_Human_Month_01;
    2: s := Token_Human_Month_02;
    3: s := Token_Human_Month_03;
    4: s := Token_Human_Month_04;
    5: s := Token_Human_Month_05;
    6: s := Token_Human_Month_06;
    7: s := Token_Human_Month_07;
    8: s := Token_Human_Month_08;
    9: s := Token_Human_Month_09;
    10: s := Token_Human_Month_10;
    11: s := Token_Human_Month_11;
    12: s := Token_Human_Month_12;
    else
      s := nil;
  end;

  if s = nil then result := ''
  else
    begin
      if s.Count = 0 then result := '' else result := s[0];
    end;
end;

function TGedcomDate.GedcomKey2HumanKey(key: string): string;
var
  s: TStringList;
begin
  if (key = 'FROM') then s := Token_Du
  else
    if (key = 'TO') then s := Token_Au
    else
      if (key = 'BEF') then s := Token_Avant
      else
        if (key = 'AFT') then s := Token_Apres
        else
          if (key = 'BET') then s := Token_Entre
          else
            if (key = 'AND') then s := Token_Et
            else
              if (key = 'ABT') then s := Token_Vers
              else
                if (key = 'CAL') then s := Token_Calculee
                else
                  if (key = 'EST') then s := Token_Estimee
                  else
                    s := nil;


  if s = nil then result := ''
  else
    begin
      if s.Count = 0 then result := '' else result := s[0];
    end;
end;

end.
