{-----------------------------------------------------------------------}
{                                                                       }
{           Subprogram Name:                                            }
{           Purpose:          Ancestrologie                             }
{           Source Language:  Francais                                  }
{           Auteur:                        }
{           Philippe Cazaux-Moutou                                       }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Description:                                                }
{           Ancestrologie est un Logiciel Libre                         }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Revision History                                            }
{           v#    ,Date       ,Author Name            ,Description      }
{                                                                       }
{-----------------------------------------------------------------------}

unit u_objet_TImportGedCom;

interface

uses
  Classes,
  contnrs,
  typinfo,
  dialogs,
  u_gedcom_func,
  u_gedcom_type;


type
  TNotifyProgressEvent = procedure(Sender: TObject; Pourcent: integer) of object;

  TIndi = class
  private
    fCLE_PERE: integer;
    fCle: integer;
    fCLE_MERE: integer;
    fCLE_PARENTS: integer;
    fID: string;
    fSEXE: string;

  public
    property ID: string read fID write fID;
    property Cle: integer read fCle write fCle;
    property CLE_PARENTS: integer read fCLE_PARENTS write fCLE_PARENTS;
    property CLE_PERE: integer read fCLE_PERE write fCLE_PERE;
    property CLE_MERE: integer read fCLE_MERE write fCLE_MERE;
    property SEXE: string read fSEXE write fSEXE;

    constructor Create;
  end;

  TFam = class
  private
    fCLE: integer;
    fCLEF_HUSB: integer;
    fCLEF_WIFE: integer;
    fID: string;
    fAlreadySaveThisUnion: boolean;

  public
    property ID: string read fID write fID;
    property CLE: integer read fCLE write fCLE; //cl du record dans la table
    property CLEF_HUSB: integer read fCLEF_HUSB write fCLEF_HUSB;
    property CLEF_WIFE: integer read fCLEF_WIFE write fCLEF_WIFE;
    property AlreadySaveThisUnion: boolean read fAlreadySaveThisUnion write fAlreadySaveThisUnion;
    constructor Create;

  end;

  TSour = class
  private
    fID: string;
    fFirstLine: string;
    fGedcomBlock: TGedcomStructure;

  public
    property ID: string read fID write fID;
    property FirstLine: string read fFirstLine write fFirstLine;
    property GedcomBlock: TGedcomStructure read fGedcomBlock write fGedcomBlock;
    constructor Create;
    destructor destroy; override;
  end;

  TNote = class
  private
    fID: string;
    fFirstLine: string;
    fGedcomBlock: TGedcomStructure;
  public
    property ID: string read fID write fID;
    property FirstLine: string read fFirstLine write fFirstLine;
    property GedcomBlock: TGedcomStructure read fGedcomBlock write fGedcomBlock;
    constructor Create;
    destructor destroy; override;

  end;

  TRepo = class
  private
    fID: string;
    fFirstLine: string;
    fGedcomBlock: TGedcomStructure;
  public
    property ID: string read fID write fID;
    property FirstLine: string read fFirstLine write fFirstLine;
    property GedcomBlock: TGedcomStructure read fGedcomBlock write fGedcomBlock;
    constructor Create;
    destructor destroy; override;
  end;

  TImportGedCom = class

  private
    //	fGedcomStructureOfAllFile: TGedComStructure;
    fGedcomStructureOfHeader: TGedComStructure;
    fEventsPossible: TStringList;
    fOnProgress: TNotifyProgressEvent;
    fPathFileName: string;

    //Liste des individus
    fListIndi: TObjectList;
    fListFam: TObjectList;
    fListNote: TObjectList;
    fListSour: TObjectList;
    fListRepo: TObjectList;

    procedure DestroyProperties;
    procedure PrepareProperties;

  public
    //	property GedcomStructureOfAllFile: TGedComStructure read fGedcomStructureOfAllFile write fGedcomStructureOfAllFile;
    property GedcomStructureOfHeader: TGedComStructure read fGedcomStructureOfHeader write fGedcomStructureOfHeader;
    property EventsPossible: TStringList read fEventsPossible write fEventsPossible;
    property PathFileName: string read fPathFileName write fPathFileName;
    property OnProgress: TNotifyProgressEvent read fOnProgress write fOnProgress;


    property ListIndi: TObjectList read fListIndi write fListIndi;
    property ListFam: TObjectList read fListFam write fListFam;
    property ListSour: TObjectList read fListSour write fListSour;
    property ListNote: TObjectList read fListNote write fListNote;
    property ListRepo: TObjectList read fListRepo write fListRepo;


    constructor Create;
    destructor Destroy; override;

    //lecture de tous les individus
    procedure PrepareIndividusAndFamilles;

    //lit seulement l'entte, et le place dans 'fContent'
    function LoadFromFileJustTheHeader(aPathFileName: string; aContent: TStringList): boolean;

    //lit quelques exemples de noms
    function LoadFromFileQuelquesExemplesDeNoms(aPathFileName: string; aContent: TStringList): boolean;

    //lit quelques adresses
    function LoadFromFileQuelquesAdresses(aPathFileName: string; aContent: TStringList): boolean;

    //Dcode le contenue de 'fContent', et le place dans la structure 'aGedComStructure'
    function DecodageGedcom(aGedComStructure: TGedComStructure; Content: TStringList; JeuxCaractere: TSetCodageCaracteres): boolean;

    //rinitialise cet objet
    procedure Clear;

    function GetIndividuByID(aID: string): TIndi;
    function GetIndividuByCle(aCLE: integer): TIndi;
    function GetFamilleByID(aID: string): TFam;
    function GetSourceByID(aID: string): TSour;
    function GetNoteByID(aID: string): TNote;
    function GetRepoByID(aID: string): TRepo;
  end;


implementation

uses
  u_dm,
  SysUtils;

function DecodeLigne(
  ligne: string;
  var niveau: integer;
  var tag: string;
  var ID: string;
  var infos: string
  ): boolean;
var
  p: integer;
begin
  result := false;
  tag := '';
  ID := '';
  infos := '';

  repeat
    p := pos(#9, ligne);
    if p > 0 then delete(ligne, p, 1);
  until p = 0;

  ligne := trim(ligne);

  //Level
  p := pos(' ', ligne);
  if p > 0 then
    begin
      niveau := StrToIntDef(copy(ligne, 1, p - 1), -1);

      delete(ligne, 1, p);
      if length(ligne) > 0 then
        begin

          //Soit le tag
          if ligne[1] <> '@' then
            begin
              p := pos(' ', ligne);
              if p = 0 then
                begin
                  tag := ligne;
                end
              else
                begin
                  tag := copy(ligne, 1, p - 1);
                  delete(ligne, 1, p);

                  if length(ligne) > 0 then
                    begin
                      if ligne[1] <> '@' then infos := ligne
                      else
                        begin
                          delete(ligne, 1, 1);
                          p := pos('@', ligne);
                          if p <> 0 then
                            begin
                              ID := copy(ligne, 1, p - 1);
                            end;
                        end;
                    end;
                end;
              result := true;
            end
          else
            //Soit la rfrence
            begin
              delete(ligne, 1, 1);
              p := pos('@', ligne);
              if p > 0 then
                begin
                  ID := copy(ligne, 1, p - 1);
                  delete(ligne, 1, p + 1);
                  tag := ligne;
                  result := true;
                end;
            end;
        end;
    end;
end;


constructor TImportGedCom.Create;
begin
  fOnProgress := nil;

  PrepareProperties;
end;


destructor TImportGedCom.Destroy;
begin
  DestroyProperties;

  inherited;
end;

{
function TImportGedCom.LoadFromFile(aPathFileName: string): boolean;
begin
     try
          fContent.LoadFromFile(aPathFileName);
          result := true;
     except
          result := false;
     end;
end;
 }


function TImportGedCom.DecodageGedcom(aGedComStructure: TGedComStructure; Content: TStringList; JeuxCaractere: TSetCodageCaracteres): boolean;
var
  p, tot: integer;
  numLigne, PaquetLigne: integer;
  fStack: TObjectStack;
  ExclureLevelUpperThan: integer;

  TagLevel: integer;
  TagCaption: string;
  TagID: string;
  TagValue: string;

  canAnalyseTag: boolean;
  CanCreateTag: boolean;
  TagIndex: integer;

  NewSubTag: TObject;
  TagPere: TTag;

  fLast: TTag;

  F: Text;
  line: string;
  Stop: Boolean;


  procedure DecodageLigne(aLine: string);
  var
    n: integer;
  begin
    aLine := ConvertionChaine(aLine, JeuxCaractere);
    //Dcodage de la ligne
    if DecodeLigne(aLine, TagLevel, TagCaption, TagID, TagValue) then
      begin
        if TagLevel < ExclureLevelUpperThan then
          begin
            canAnalyseTag := false;
            ExclureLevelUpperThan := maxInt;


            //soit on est dans un sous-niveau du pere actuel
            if TagLevel = TagPere.Level + 1 then canAnalyseTag := true

              //soit on saute plusieurs sous-niveaux du pere actuel (impossible)
            else if TagLevel > TagPere.Level + 1 then canAnalyseTag := false

              //soit on remonte le(s) pere(s) sur un ou plusieurs niveaux
            else if TagLevel <= TagPere.Level then
              begin
                //On dsempile le(s) pere(s)
                for n := TagLevel to TagPere.Level do TagPere := TTag(fStack.Pop);
                canAnalyseTag := true;
              end;

            if canAnalyseTag then
              begin
                //si c'est un mmo, traitement spcial
                if TagPere is TTag_MEMO then
                  begin
                    if TagCaption = 'CONT' then
                      begin
                        TTag_MEMO(TagPere).Memo.Add(TagValue);
                      end
                    else
                      if TagCaption = 'CONC' then
                        begin
                          n := TTag_MEMO(TagPere).Memo.Count;
                          if n > 0 then
                            begin
                              TTag_MEMO(TagPere).Memo[n - 1] := TTag_MEMO(TagPere).Memo[n - 1] + TagValue;
                            end
                          else
                            TTag_MEMO(TagPere).Memo.Add(TagValue);
                        end;
                  end
                else
                  if (TagPere is TTag_SOURCE_CITATION) and ((TagCaption = 'CONT') or (TagCaption = 'CONC')) then
                    begin
                      if TagCaption = 'CONT' then
                        begin
                          TTag_SOURCE_CITATION(TagPere).Memo.Add(TagValue);
                        end
                      else
                        if TagCaption = 'CONC' then
                          begin
                            n := TTag_SOURCE_CITATION(TagPere).Memo.Count;
                            if n > 0 then
                              begin
                                TTag_SOURCE_CITATION(TagPere).Memo[n - 1] := TTag_SOURCE_CITATION(TagPere).Memo[n - 1] + TagValue;
                              end
                            else
                              TTag_SOURCE_CITATION(TagPere).Memo.Add(TagValue);
                          end;
                    end
                  else
                    if (TagPere is TTag_NOTE_STRUCTURE) and ((TagCaption = 'CONT') or (TagCaption = 'CONC')) then
                      begin
                        if TagCaption = 'CONT' then
                          begin
                            TTag_NOTE_STRUCTURE(TagPere).Memo.Add(TagValue);
                          end
                        else
                          if TagCaption = 'CONC' then
                            begin
                              n := TTag_NOTE_STRUCTURE(TagPere).Memo.Count;
                              if n > 0 then
                                begin
                                  TTag_NOTE_STRUCTURE(TagPere).Memo[n - 1] := TTag_NOTE_STRUCTURE(TagPere).Memo[n - 1] + TagValue;
                                end
                              else
                                TTag_NOTE_STRUCTURE(TagPere).Memo.Add(TagValue);
                            end;
                      end
                    else
                      if (TagPere is TTag_NOTE_RECORD) and ((TagCaption = 'CONT') or (TagCaption = 'CONC')) then
                        begin
                          if TagCaption = 'CONT' then
                            begin
                              TTag_NOTE_RECORD(TagPere).Memo.Add(TagValue);
                            end
                          else
                            if TagCaption = 'CONC' then
                              begin
                                n := TTag_NOTE_RECORD(TagPere).Memo.Count;
                                if n > 0 then
                                  begin
                                    TTag_NOTE_RECORD(TagPere).Memo[n - 1] := TTag_NOTE_RECORD(TagPere).Memo[n - 1] + TagValue;
                                  end
                                else
                                  TTag_NOTE_RECORD(TagPere).Memo.Add(TagValue);
                              end;
                        end
                      else
                        if (TagPere is TTag_DATA1) and ((TagCaption = 'CONT') or (TagCaption = 'CONC')) then
                          begin
                            if TagCaption = 'CONT' then
                              begin
                                TTag_DATA1(TagPere).Memo.Add(TagValue);
                              end
                            else
                              if TagCaption = 'CONC' then
                                begin
                                  n := TTag_DATA1(TagPere).Memo.Count;
                                  if n > 0 then
                                    begin
                                      TTag_DATA1(TagPere).Memo[n - 1] := TTag_DATA1(TagPere).Memo[n - 1] + TagValue;
                                    end
                                  else
                                    TTag_DATA1(TagPere).Memo.Add(TagValue);
                                end;
                          end

                        else
                          //Soit ce tag contient d'autres tag
                          if TagPere.Simple = false then
                            begin
                              //on regarde si le tag est dans la liste des tags possible du Tag en cours.
                              if TagPere.ExistSubTag(TagCaption, TagIndex) then
                                begin
                                  //si ce Tag est un Tag_MEMO...


                                      //Cration d'une instance, pour ce record
                                  CanCreateTag := true;

                                  //si ce futur Tag est un event, alors
                                  if (TagPere.ClassSubTags[TagIndex] = TTag_EVENT_DETAIL) then
                                    // or (TagPere.ClassSubTags[TagIndex] = TTag_EVENT_DETAIL_2)
                                    // or (TagPere.ClassSubTags[TagIndex] = TTag_EVENT_DETAIL_3)
                                    begin
                                      //ce tag est-il autoris ?
                                      CanCreateTag := fEventsPossible.IndexOf(TagCaption) <> -1;
                                    end;

                                  if CanCreateTag then
                                    begin
                                      NewSubTag := TagPere.ClassSubTags[TagIndex].create;

                                      //Initialisation de l'objet
                                      TTag(NewSubTag).AfterCreate(aGedComStructure);


                                      //si ce Tag est en multi-instance
                                      if (tagPere.ObjectSubTags[TagIndex] = nil) then
                                        begin
                                          //Affectation de l'instance  la proprit du Record pre
                                          SetObjectProp(TagPere, TagPere.PropInfoSubTags[TagIndex], NewSubTag);
                                          TagPere.ObjectSubTags[TagIndex] := TTag(NewSubTag);
                                        end
                                      else
                                        begin
                                          //on fait un lien vers le noueau tag cr - on cherche le dernier
                                          fLast := TTag(tagPere.ObjectSubTags[TagIndex]);
                                          while fLast.PtrNextSameTag <> nil do fLast := TTag(fLast.PtrNextSameTag);

                                          //on connait le dernier de la chane
                                          fLast.PtrNextSameTag := NewSubTag;

                                          //Affectation de l'instance  la proprit du Record pre
                                   //   SetObjectProp(TagPere, TagPere.PropInfoSubTags[TagIndex], NewSubTag);
                                           //   TagPere.ObjectSubTags[TagIndex] := TTag(NewSubTag);
                                        end;
                                      {
                                                                                                         if (tagPere.ObjectSubTags[TagIndex] <> nil) then
                                                                               begin
                                                                                                          //on fait un lien vers le noueau tag cr
                  TTag(tagPere.ObjectSubTags[TagIndex]).PtrNextSameTag := NewSubTag;
                                 end;
                          //Affectation de l'instance  la proprit du Record pre
                      SetObjectProp(TagPere, TagPere.PropInfoSubTags[TagIndex], NewSubTag);
                        TagPere.ObjectSubTags[TagIndex] := TTag(NewSubTag);
                                       }

                          //remplissage du nouveau Tag
                                      TTag(NewSubTag).Caption := TagCaption;
                                      TTag(NewSubTag).ID := TagID;
                                      TTag(NewSubTag).Value := TagValue;
                                      TTag(NewSubTag).Level := TagLevel;

                                      //Si ce n'est pas un tag terminal, alors on entre dedant
                                      if TTag(NewSubTag).Simple = false then
                                        begin
                                          //On mmorise le pere de ce Tag
                                          fStack.Push(TagPere);

                                          //Le TagPere devient ce nouveau Tag, puisqu'il n'est pas Simple
                                          TagPere := TTag(NewSubTag);
                                        end;
                                    end;
                                end
                              else
                                begin
                                  //   fMemoLoadResult.Add('UNK  ->  ' + fContent[numLigne]);
                                end;

                            end
                          else
                            begin
                              //  fMemoLoadResult.Add('TGS  ->  ' + fContent[numLigne]);
                            end;
              end
            else
              begin
                //On ne connait pas ce tag, donc, pour les prochaines lignes, on ignore les tags de niveau supprieur
                ExclureLevelUpperThan := TagLevel;
                //                          fMemoLoadResult.Add('CNA  ->  ' + fContent[numLigne]);
              end;
          end
        else
          begin
            //				fMemoLoadResult.Add('EXC  ->  ' + fContent[numLigne]);
          end;
      end
    else
      begin
        //			fMemoLoadResult.Add('DEC  ->  ' + fContent[numLigne]);
      end;

    inc(numLigne);
    if PaquetLigne < 50 then inc(PaquetLigne)
    else
      begin
        PaquetLigne := 0;
        /////////////                              if assigned(fOnProgress) then fOnProgress(self, round(numLigne * 100 / fContent.count));
      end;
  end;

begin
  result := true;
  try

    //          if assigned(fOnProgress) then fOnProgress(self, 0);

    numLigne := 0;
    PaquetLigne := 0;
    ExclureLevelUpperThan := MaxInt;
    fStack := TObjectStack.create;

    try
      //Le pere de tous
      TagPere := aGedComStructure;

      if Content = nil then
        begin
          //On boucle sur les lignes du fichier GedCom
          AssignFile(F, fPathFileName);
          try
            try
              Reset(F);
              Stop := EOF(F);

              while (not Stop) do
                begin
                  //on lit une ligne
                  ReadLn(F, line); line := trim(line);

                  //Dcodage de la ligne
                  DecodageLigne(line);

                  if Stop = false then Stop := EOF(F);
                end;

            finally
              CloseFile(F);
            end;
          except
            result := false;
          end;
        end
      else
        begin
          p := 0; tot := Content.Count;
          while p < tot do
            begin
              DecodageLigne(Content[p]);
              p := p + 1;
            end;
        end;

    finally
      fStack.Free;
    end;
  except
    result := false;
  end;
end;


procedure TImportGedCom.Clear;
begin
  DestroyProperties;
  PrepareProperties;
end;


procedure TImportGedCom.DestroyProperties;
begin
  {
   fGedcomStructureOfAllFile.free;
   }
  fGedcomStructureOfHeader.free;
  fEventsPossible.free;
  fListIndi.Free;
  fListFam.Free;
  fListSour.Free;
  fListNote.Free;
  fListRepo.Free;
end;


procedure TImportGedCom.PrepareProperties;
begin
  {
   fGedcomStructureOfAllFile := TGedComStructure.create;
   fGedcomStructureOfAllFile.AfterCreate(nil);
   fGedcomStructureOfAllFile.Level := -1;
  }
  fGedcomStructureOfHeader := TGedComStructure.create;
  fGedcomStructureOfHeader.PtrObjetImportGedcom := self;
  fGedcomStructureOfHeader.AfterCreate(nil);
  fGedcomStructureOfHeader.Level := -1;

  fEventsPossible := TStringList.create;
  //     fMemoLoadResult := TStringList.create;

  fListIndi := TObjectList.create;
  fListFam := TObjectList.create;
  fListSour := TObjectList.create;
  fListNote := TObjectList.create;
  fListRepo := TObjectList.create;
end;

(*

procedure TImportGedCom.DecodHeader;
var
     fContentHeader: TStringList;

     numLigne: integer;
     fStack: TObjectStack;
     ExclureLevelUpperThan: integer;

     TagLevel: integer;
 TagCaption: string;
     TagID: string;
     TagValue: string;

     canAnalyseTag: boolean;
     n: integer;
     TagIndex: integer;

 NewSubTag: TObject;
     TagPere: TTag;

     fLast: TTag;
     F: TextFile;
     ligne: string;
begin
     try
          fContentHeader := TStringList.create;
          try
               AssignFile(F, fPathFileName);
               try
    Reset(F);

                    //lecture de la premire ligne
                    ReadLn(F, ligne);

                    if DecodeLigne(ligne, TagLevel, TagCaption, TagID, TagValue) then
                    begin
                         if (TagLevel = 0) and (TagCaption = 'HEAD') then
                         begin
                              ExclureLevelUpperThan := MaxInt;
                              fStack := TObjectStack.create;

                              try
                                   //Le pere de tous
                                   TagPere := fGedComStructure;

                                   //On boucle sur les lignes du fichier GedCom
                                   while (ligne <> '') do
       begin
                                        //Dcodage de la ligne
                                        if DecodeLigne(fContent[numLigne], TagLevel, TagCaption, TagID, TagValue) then
                                        begin
                                             if (TagLevel > 0) or ((TagLevel = 0) and (TagCaption = 'HEAD')) then
                                             begin
                                                  if TagLevel < ExclureLevelUpperThan then
                                                  begin
                                                       canAnalyseTag := false;
                                                       ExclureLevelUpperThan := maxInt;


                                                       //soit on est dans un sous-niveau du pere actuel
           if TagLevel = TagPere.Level + 1 then canAnalyseTag := true

                                                            //soit on saute plusieurs sous-niveaux du pere actuel (impossible)
                                                       else if TagLevel > TagPere.Level + 1 then canAnalyseTag := false

                                                            //soit on remonte le(s) pere(s) sur un ou plusieurs niveaux
                                                       else if TagLevel <= TagPere.Level then
                                                       begin
                                                            //On dsempile le(s) pere(s)
                                                            for n := TagLevel to TagPere.Level do TagPere := TTag(fStack.Pop);
                                                            canAnalyseTag := true;
           end;

                                                       if canAnalyseTag then
                                                       begin
                                                            //Soit ce tag contient d'autres tag
                                                            if TagPere.Simple = false then
            begin
                                                                 //on regarde si le tag est dans la liste des tags possible du Tag en cours.
                                                                 if TagPere.ExistSubTag(TagCaption, TagIndex) then
                                                                 begin
                                                                      //Cration d'une instance, pour ce record
                                                                      NewSubTag := TagPere.ClassSubTags[TagIndex].create;

                                                                      //Initialisation de l'objet
                                                                      TTag(NewSubTag).AfterCreate(fGedComStructure);


                                                                      //si ce Tag est en multi-instance
                                                                      if (tagPere.ObjectSubTags[TagIndex] = nil) then
                                                                      begin
               //Affectation de l'instance  la proprit du Record pre
                                                                           SetObjectProp(TagPere, TagPere.PropInfoSubTags[TagIndex], NewSubTag);
                                                                           TagPere.ObjectSubTags[TagIndex] := TTag(NewSubTag);
                                                                      end
                                                                      else
                                                                      begin
                                                                           //on fait un lien vers le noueau tag cr - on cherche le dernier
                                                                           fLast := TTag(tagPere.ObjectSubTags[TagIndex]);
                                                                           while fLast.PtrNextSameTag <> nil do fLast := TTag(fLast.PtrNextSameTag);

                                                                           //on connait le dernier de la chane
               fLast.PtrNextSameTag := NewSubTag;

                                                                           //Affectation de l'instance  la proprit du Record pre
                                                                            //   SetObjectProp(TagPere, TagPere.PropInfoSubTags[TagIndex], NewSubTag);
                                                                            //   TagPere.ObjectSubTags[TagIndex] := TTag(NewSubTag);
              end;






                                                                      {
                                                                      if (tagPere.ObjectSubTags[TagIndex] <> nil) then
              begin
                                                               //on fait un lien vers le noueau tag cr
                                                                       TTag(tagPere.ObjectSubTags[TagIndex]).PtrNextSameTag := NewSubTag;
                                                                      end;

                                                                      //Affectation de l'instance  la proprit du Record pre
              SetObjectProp(TagPere, TagPere.PropInfoSubTags[TagIndex], NewSubTag);
                                                                      TagPere.ObjectSubTags[TagIndex] := TTag(NewSubTag);
                                                                          }

                                                                          //remplissage du nouveau Tag
                                                                      TTag(NewSubTag).Caption := TagCaption;
                                                                      TTag(NewSubTag).ID := TagID;
                                                                      TTag(NewSubTag).Value := TagValue;
                                                                      TTag(NewSubTag).Level := TagLevel;

                                                                      //Si ce n'est pas un tag terminal, alors on entre dedant
              if TTag(NewSubTag).Simple = false then
                                                                      begin
                                                                           //On mmorise le pere de ce Tag
                                                                           fStack.Push(TagPere);

                                                                           //Le TagPere devient ce nouveau Tag, puisqu'il n'est pas Simple
                                                                           TagPere := TTag(NewSubTag);
                                                                      end;
                                                                 end
                                                                 else
                                                                 begin
                                                                      //											fMemoLoadResult.Add('UNK  ->  ' + fContent[numLigne]);
                                                                 end;
                                                            end
                                                            else
                                                            begin
                                                                 //                                                fMemoLoadResult.Add('TGS  ->  ' + fContent[numLigne]);
                                                            end;
                                                       end
                                                       else
           begin
                                                            //On ne connait pas ce tag, donc, pour les prochaines lignes, on ignore les tags de niveau supprieur
                                                            ExclureLevelUpperThan := TagLevel;
                                                            //                                         fMemoLoadResult.Add('CNA  ->  ' + fContent[numLigne]);
                                                       end;
                                                  end
                                                  else
                                                  begin
                                                       //							fMemoLoadResult.Add('EXC  ->  ' + fContent[numLigne]);
                                                  end;
                                             end
         else
                                                  Stop := true;
                                        end
        else
                                        begin
                                             //						fMemoLoadResult.Add('DEC  ->  ' + fContent[numLigne]);
                                        end;

                                        inc(numLigne);
                                   end;

                              finally
                                   fStack.Free;
                              end;
                         end;
                    end;


               finally
                    CloseFile(F);
   end;
  finally
   fContentHeader.free;
  end;
 except
 end;


  result := true;

end;
*)


function TImportGedCom.LoadFromFileJustTheHeader(aPathFileName: string; aContent: TStringList): boolean;
var
  s: string;
  F: Text;
  Stop: boolean;
begin
  aContent.Clear;

  AssignFile(F, aPathFileName);
  try
    try
      Reset(F);
      Stop := EOF(F);

      while (not Stop) do
        begin
          //on lit une ligne
          ReadLn(F, S);
          s := Trim(s);

          if copy(s, 1, 2) = '0 ' then
            begin
              if s <> '0 HEAD' then Stop := true;
            end;

          if Stop = false then
            begin
              aContent.Add(s);
              Stop := EOF(F);
            end;
        end;

      result := true;
    finally
      CloseFile(F);
    end;
  except
    result := false;
  end;
end;

function TImportGedCom.LoadFromFileQuelquesExemplesDeNoms(aPathFileName: string; aContent: TStringList): boolean;
var
  s: string;
  F: Text;
  Stop: boolean;
begin
  aContent.Clear;

  AssignFile(F, aPathFileName);
  try
    try
      Reset(F);
      Stop := EOF(F);

      while (not Stop) do
        begin
          //on lit une ligne
          ReadLn(F, S); s := trim(s);
          if copy(s, 1, 7) = '1 NAME ' then
            begin
              delete(s, 1, 7);
              aContent.Add(s);
              if aContent.count >= 20 then Stop := true;
            end;

          if Stop = false then Stop := EOF(F);
        end;
      result := true;
    finally
      CloseFile(F);
    end;
  except
    result := false;
  end;
end;


function TImportGedCom.LoadFromFileQuelquesAdresses(aPathFileName: string; aContent: TStringList): boolean;
var
  s: string;
  F: Text;
  Stop: boolean;
begin
  aContent.Clear;

  AssignFile(F, aPathFileName);
  try
    try
      Reset(F);
      Stop := EOF(F);

      while (not Stop) do
        begin
          //on lit une ligne
          ReadLn(F, S); s := trim(s);
          if copy(s, 1, 7) = '2 PLAC ' then
            begin
              delete(s, 1, 7);
              aContent.Add(s);
              if aContent.count >= 20 then Stop := true;
            end;

          if Stop = false then Stop := EOF(F);
        end;
      result := true;
    finally
      CloseFile(F);
    end;
  except
    result := false;
  end;
end;


procedure TImportGedCom.PrepareIndividusAndFamilles;
var
  s: string;
  F: Text;
  Stop: boolean;
  TagLevel: integer;
  TagCaption: string;
  TagID: string;
  TagValue: string;

  indi: TIndi;
  fam: TFam;
  sour: TSour;
  note: TNote;
  repo: TRepo;

begin
  indi := nil;
  try
    AssignFile(F, fPathFileName);
    try
      Reset(F);
      Stop := EOF(F);

      while (not Stop) do
        begin
          //on lit une ligne
          ReadLn(F, S);

          if copy(S, 1, 2) = '0 ' then
            begin
              if DecodeLigne(s, TagLevel, TagCaption, TagID, TagValue) then
                begin
                  if TagCaption = 'INDI' then
                    begin
                      indi := TIndi.create;
                      indi.Cle := dm.uf_GetClefUnique('INDIVIDU');
                      indi.ID := TagID;
                      fListIndi.Add(indi);

                      if assigned(fOnProgress) then fOnProgress(self, 0);
                    end
                  else
                    if TagCaption = 'FAM' then
                      begin
                        fam := TFam.create;
                        fam.Cle := dm.uf_GetClefUnique('T_UNION');
                        fam.ID := TagID;
                        fListFam.Add(fam);

                        if assigned(fOnProgress) then fOnProgress(self, 0);
                      end
                    else
                      if TagCaption = 'SOUR' then
                        begin
                          sour := TSour.create;
                          sour.FirstLine := s;
                          sour.ID := TagID;
                          fListSour.Add(sour);
                        end
                      else
                        if TagCaption = 'NOTE' then
                          begin
                            note := TNote.create;
                            note.FirstLine := s;
                            note.ID := TagID;
                            fListNote.Add(note);
                          end
                        else
                          if TagCaption = 'REPO' then
                            begin
                              repo := TRepo.create;
                              repo.FirstLine := s;
                              repo.ID := TagID;
                              fListRepo.Add(repo);
                            end
                end;
            end
          else
            if copy(S, 1, 6) = '1 SEX ' then
              begin
                if DecodeLigne(s, TagLevel, TagCaption, TagID, TagValue) then
                  begin
                    if (TagCaption = 'SEX') and (TagLevel = 1) and (indi <> nil) and (indi.SEXE = '') then
                      begin
                        indi.SEXE := TagValue;
                      end;
                  end;
              end;

          if Stop = false then Stop := EOF(F);
        end;

    finally
      CloseFile(F);
    end;


  except
  end;
end;

{ TIndi }

constructor TIndi.Create;
begin
  fID := '';
  fCle := -1;
  fCLE_PARENTS := -1;
  fCLE_PERE := -1;
  fCLE_MERE := -1;
  fSexe := '';
end;

{ TFam }

constructor TFam.Create;
begin
  fID := '';
  fCLE := -1;
  fCLEF_HUSB := -1;
  fCLEF_WIFE := -1;
  fAlreadySaveThisUnion := false;
end;

function TImportGedCom.GetIndividuByID(aID: string): TIndi;
var
  n: integer;
begin
  result := nil;
  for n := 0 to fListIndi.Count - 1 do
    begin
      if TIndi(fListIndi[n]).ID = aID then
        begin
          result := TIndi(fListIndi[n]);
          break;
        end;
    end;
end;

function TImportGedCom.GetFamilleByID(aID: string): TFam;
var
  n: integer;
begin
  result := nil;
  for n := 0 to fListFam.Count - 1 do
    begin
      if TFam(fListFam[n]).ID = aID then
        begin
          result := TFam(fListFam[n]);
          break;
        end;
    end;
end;


function TImportGedCom.GetIndividuByCle(aCLE: integer): TIndi;
var
  n: integer;
begin
  result := nil;
  for n := 0 to fListIndi.Count - 1 do
    begin
      if TIndi(fListIndi[n]).CLE = aCLE then
        begin
          result := TIndi(fListIndi[n]);
          break;
        end;
    end;
end;


{ TSour }

constructor TSour.Create;
begin
  fID := '';
  fGedcomBlock := TGedcomStructure.create;
end;

{ TComment }

constructor TNote.Create;
begin
  fID := '';
  fGedcomBlock := TGedcomStructure.create;
end;


destructor TSour.destroy;
begin
  fGedcomBlock.free;
  inherited;
end;

function TImportGedCom.GetSourceByID(aID: string): TSour;
var
  n: integer;
begin
  result := nil;
  for n := 0 to fListSour.Count - 1 do
    begin
      if TSour(fListSour[n]).ID = aID then
        begin
          result := TSour(fListSour[n]);
          break;
        end;
    end;
end;

destructor TNote.destroy;
begin
  fGedcomBlock.free;
  inherited;
end;

{ TRepo }

constructor TRepo.Create;
begin
  fID := '';
  fGedcomBlock := TGedcomStructure.create;
end;

destructor TRepo.destroy;
begin
  fGedcomBlock.free;
  inherited;
end;

function TImportGedCom.GetNoteByID(aID: string): TNote;
var
  n: integer;
begin
  result := nil;
  for n := 0 to fListNote.Count - 1 do
    begin
      if TNote(fListNote[n]).ID = aID then
        begin
          result := TNote(fListNote[n]);
          break;
        end;
    end;
end;

function TImportGedCom.GetRepoByID(aID: string): TRepo;
var
  n: integer;
begin
  result := nil;
  for n := 0 to fListRepo.Count - 1 do
    begin
      if TRepo(fListRepo[n]).ID = aID then
        begin
          result := TRepo(fListRepo[n]);
          break;
        end;
    end;
end;

end.
