{-----------------------------------------------------------------------}
{                                                                       }
{           Subprogram Name:                                            }
{           Purpose:          Ancestrologie                             }
{           Source Language:  Francais                                  }
{           Auteur:                        }
{           Philippe Cazaux-Moutou                                       }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Description:                                                }
{           Ancestrologie est un Logiciel Libre                         }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Revision History                                            }
{           v#    ,Date       ,Author Name            ,Description      }
{                                                                       }
{-----------------------------------------------------------------------}

unit u_common_graph_func;

interface

uses
  Classes,
  windows,
  SysUtils,
  Dialogs,
  graphics,
  u_common_graph_const,
  u_common_graph_var;

function AssureCorrectAngleDeg(e: single): single;
function CreateStructureChantierOnDisk(NameChantier: string; var CodeError: integer): boolean;
function StructureOnDiskIsOk(NameChantier: string): boolean;
function SFloatN(e: extended; digits: integer): string;
function TruncInf(e: single): integer;
function TruncSup(e: single): integer;
function DirExists(const Dir: string): boolean;
procedure GetDirList(InThisDir: string; aList: TStrings);
function extractFileNameWithoutExt(s: string): string;
function norme(P1x, P1y, P2x, P2y: single {en m}): extended; //en m
function FontToString(Font: TFont): string;
procedure StringToFont(Str: string; Font: TFont);

procedure TrouvePoint(x1, y1, x2, y2, d: extended; var x, y: extended);
procedure TrouveCoords(xc, yc, x1, y1, x0, y0, x2, y2, decal_de_P0: extended; var xa, ya, xb, yb: extended);
function TrouveIntersectionCercleDroite(x1, y1, x2, y2, xc, yc, rayon: extended; var xa, ya, xb, yb: extended): boolean;
function LargeurPossibleTexte(x1, y1, x2, y2, xc, yc, rayon: extended; var d: extended): boolean;
procedure PointLePlusProcheDeP(x1, y1, x2, y2, x, y: extended; var xp, yp: extended);

implementation



function AssureCorrectAngleDeg(e: single): single;
begin
  result := e;
end;



function CreateStructureChantierOnDisk(NameChantier: string; var CodeError: integer): boolean;
var
  suite: boolean;
begin
  CodeError := 0;
  result := false;

  //Cration du rpertoire pre de tous les chantiers
  if DirExists(_PathChantiers) = false then
    begin
      suite := CreateDir(_PathChantiers);
      if suite = false then CodeError := 1;
    end
  else
    suite := true;

  if suite then
    begin
      //Cration du rpertoire pre du chantier (le nom du chantier)
      if CreateDir(_PathChantiers + NameChantier) then suite := true
      else
        begin
          //La cration a chous, car le rpertoire existe peut-tre ?
          suite := DirExists(_PathChantiers + NameChantier);

          if suite = false then CodeError := 1; //'Impossible de crer ce chantier !

          //	MyMessageDlg('Impossible de crer ce chantier !', mtError, [mbOK], 0);
        end;
    end;


  if suite then
    begin
      //Prparation des rpertoires, dans la structure du chantier
      try
        CreateDir(_PathChantiers + NameChantier + '\' + _RELPATH_DONNEES_BRUTE);
        CreateDir(_PathChantiers + NameChantier + '\' + _RELPATH_FICHIERS_SB);
        CreateDir(_PathChantiers + NameChantier + '\' + _RELPATH_GESTION_PLANS);
      except
      end;

      //Si il y a eu un problme, c'est peut-tre parceque la structure est dj Ok
      result := StructureOnDiskIsOk(NameChantier);
      if result = false then CodeError := 1; //'Impossible de crer ce chantier !
    end;
end;


function StructureOnDiskIsOk(NameChantier: string): boolean;
begin
  result := false;
  try
    if DirExists(_PathChantiers + NameChantier) then
      if DirExists(_PathChantiers + NameChantier + '\' + _RELPATH_DONNEES_BRUTE) then
        if DirExists(_PathChantiers + NameChantier + '\' + _RELPATH_FICHIERS_SB) then
          if DirExists(_PathChantiers + NameChantier + '\' + _RELPATH_GESTION_PLANS) then result := true;
  except
  end;
end;


function SFloatN(e: extended; digits: integer): string;
begin
  result := FloattoStrF(e, ffFixed, 15, digits);
end;

function TruncInf(e: single): integer;
begin
  result := trunc(e);
end;

function TruncSup(e: single): integer;
begin
  result := trunc(e) + 1;
end;


function DirExists(const Dir: string): boolean;

  function StripTrailingBackslash(const Dir: string): string;
  begin
    Result := Dir;
    // Make sure we have a string, and if so, see if the last char is a \
    if (Result <> '') and (Result[Length(Result)] = '\') then
      SetLength(Result, Length(Result) - 1); // Shorten the length by one to remove
  end;

var
  Tmp: string;
  DriveBits: set of 0..25;
  SR: TSearchRec;
  Found: boolean;
  OldMode: Word;
begin
  OldMode := SetErrorMode(SEM_FAILCRITICALERRORS);
  try
    if (Length(Dir) = 3) and (Dir[2] = ':') and (Dir[3] = '\') then begin
        Integer(DriveBits) := GetLogicalDrives;
        Tmp := UpperCase(Dir[1]);
        Result := (ord(Tmp[1]) - ord('A')) in DriveBits;
      end else begin
        Found := FindFirst(StripTrailingBackslash(Dir), faDirectory, SR) = 0;
        Result := Found and (Dir <> '');
        if Result then
          Result := (SR.Attr and faDirectory) = faDirectory;
        if Found then
          // only call FinClose if FindFirst succeeds.  Can lock NT up if it didn't
          FindClose(SR);
      end;
  finally
    SetErrorMode(OldMode);
  end;
end; // DirExists



procedure GetDirList(InThisDir: string; aList: TStrings);
var
  SearchRec: TSearchRec;
  found: word;
begin
  InThisDir := InThisDir + '*.*';
  aList.Clear;
  try
    found := findfirst(InThisDir, faAnyFile, searchrec);
    while found = 0 do
      begin
        if (SearchRec.Attr = faDirectory) then
          begin
            if (SearchRec.Name <> '.') and (SearchRec.Name <> '..')
              then aList.Add(searchrec.Name);
          end;

        found := findnext(searchrec);
      end;
  finally
    findclose(searchrec);
  end;
end;

function extractFileNameWithoutExt(s: string): string;
var
  s1: string;
begin
  s := ExtractFileName(s);
  s1 := ExtractFileExt(s);
  if s1 <> '' then delete(s, length(s) - length(s1) + 1, length(s1));
  result := s;
end;

function norme(P1x, P1y, P2x, P2y: single {en m}): extended; //en m
begin
  //les coordonnes sont passes en m, et le rsultat en m
  result := sqrt(sqr(P2x - P1x) + sqr(P2y - P1y));
end;


function FontToString(Font: TFont): string;
begin
  Result := Format('%s,%d,%d%d%d%d,%s', [Font.Name, Font.Size,
    Integer(fsBold in Font.Style), Integer(fsItalic in Font.Style),
      Integer(fsUnderline in Font.Style), Integer(fsStrikeOut in Font.Style),
      ColorToString(Font.Color)]);
end;

procedure StringToFont(Str: string; Font: TFont);
const
  SEP = ',';
  EXCEPT_MSG = 'Invalid string to font conversion.';
var
  i: Integer;
  tmpFont: TFont;
begin
  try
    // name
    i := Pos(SEP, Str);
    if i = 0 then raise EConvertError.Create(EXCEPT_MSG);
    Font.Name := Copy(Str, 1, i - 1);
    Delete(Str, 1, i);

    // size
    i := Pos(SEP, Str);
    if i = 0 then raise EConvertError.Create(EXCEPT_MSG);
    Font.Size := StrToInt(Copy(Str, 1, i - 1));
    Delete(Str, 1, i);

    // bold, italic, underline, strikethrough
    if Pos(SEP, Str) <> 5 then raise EConvertError.Create(EXCEPT_MSG);
    Font.Style := [];
    if Boolean(StrToInt(Copy(Str, 1, 1))) then Font.Style := Font.Style + [fsBold];
    if Boolean(StrToInt(Copy(Str, 2, 1))) then Font.Style := Font.Style + [fsItalic];
    if Boolean(StrToInt(Copy(Str, 3, 1))) then Font.Style := Font.Style + [fsUnderline];
    if Boolean(StrToInt(Copy(Str, 4, 1))) then Font.Style := Font.Style + [fsStrikeOut];

    Delete(Str, 1, 5);

    // colour
    Font.Color := StringToColor(Str);
  except
    tmpFont := TFont.Create;
    try
      Font.Assign(tmpFont);
    finally
      tmpFont.Free;
    end;
  end;
end;


procedure TrouvePoint(x1, y1, x2, y2, d: extended; var x, y: extended);
var
  //coef de l'quation de la droite (y=_a*x+_b)
  dx, dy: extended;
begin
  //dx := (d * (x2 - x1)) / norme(x1, y1, x2, y2);
  //dy := (d * (y2 - y1)) / norme(x1, y1, x2, y2);
  if x1 <> x2 then
    begin
      dx := (d * (x2 - x1)) / norme(x1, y1, x2, y2);
    end
  else
    begin
      dx := 0;
    end;

  if y1 <> y2 then
    begin
      dy := (d * (y2 - y1)) / norme(x1, y1, x2, y2);
    end
  else
    begin
      dy := 0;
    end;


  x := x1 + dx;
  y := y1 + dy;



  (*

if (x1 <> x2) then
begin


{
  _a := (y1 - y2) / (x1 - x2);
  _b := y1 - (_a * x1);

  GA := (_a * _a) + 1;
  GB := -((2 * x1) + (2 * _a * (_b - y1)));
  GC := (x1 * x1) - (d * d) + sqr(_b - y1);

  delta := (GB * GB) - (4 * GA * GC);

  //x := (-GB) / (2 * GA);

  if delta = 0 then
  begin
       x := (-GB) / (2 * GA);
  end
  else
  begin
       x := (-GB + sqrt(delta)) / (2 * GA);
  end;

  y := (_a * x) + _b;
  }
end
else
begin
  x := x1;
  if y1 <= y2 then y := y1 + d else y := y1 - d;
end;
*)
end;

procedure TrouveCoords(xc, yc, x1, y1, x0, y0, x2, y2, decal_de_P0: extended; var xa, ya, xb, yb: extended);
var
  xi, yi: extended;
  //     dx, dy: extended;
  a0: extended;
  ai, bi: extended;
  Di_vert: boolean;
  a1, b1: extended;
  D1_vert: boolean;
  a2, b2: extended;
  D2_vert: boolean;
begin
  a1 := 0;
  a2 := 0;
  b1 := 0;
  b2 := 0;
  //on cherche le point I
  TrouvePoint(x0, y0, xc, yc, decal_de_P0, xi, yi);

  {
  if xc <> x0 then
  begin
       dx := (decal_de_P0 * (xc - x0)) / norme(xc, yc, x0, y0);
  end
  else
  begin
       dx := 0;
  end;

  if yc <> y0 then
  begin
       dy := (decal_de_P0 * (yc - y0)) / norme(xc, yc, x0, y0);
  end
  else
  begin
       dy := 0;
  end;

  xi := x0 + dx;
  yi := y0 + dy;
   }

  //equation de (C,xiyi)
  if (x0 <> xc) then
    begin
      if (y0 <> yc) then
        begin
          a0 := (y0 - yc) / (x0 - xc); //mme que pour (Cx0y0)
          ai := -1 / a0;
          bi := yi - (ai * xi);
        end
      else
        begin
          ai := 0;
          bi := yi;
        end;
    end
  else
    begin
      ai := 0;
      bi := yi;
    end;

  Di_vert := (yc = y0);

  //Droite passant par (C,x1y1)
  if x1 <> xc then
    begin
      D1_vert := false;
      a1 := (y1 - yc) / (x1 - xc);
      b1 := y1 - (a1 * x1);
    end
  else
    begin
      D1_vert := true;
      //               c1 := x1;
    end;

  //Droite passant par (C,x2y2)
  if x2 <> xc then
    begin
      D2_vert := false;
      a2 := (y2 - yc) / (x2 - xc);
      b2 := y2 - (a2 * x2);
    end
  else
    begin
      D2_vert := true;
      //  c2 := x2;
    end;

  //intersection de Di et (Cxy1)
  if (D1_vert = false) and (Di_vert = false) then
    begin
      if a1 <> ai then
        begin
          xa := (bi - b1) / (a1 - ai);
          ya := (a1 * xa) + b1;
        end
      else
        begin
          xa := x1;
          ya := yi;
        end;
    end
  else
    if (D1_vert = false) and (Di_vert = true) then
      begin
        xa := xi;
        ya := (a1 * xa) + b1;
      end
    else
      if (D1_vert = true) and (Di_vert = false) then
        begin
          xa := x1;
          ya := (ai * xa) + bi;
        end
      else
        if (D1_vert = true) and (Di_vert = true) then
          begin
            xa := xi;
            ya := y1;
            //xa := x1;
            //ya := yi;
          end;

  {
  if D1_vert then
  begin
       xa := x1;
       ya := (a1 * xa) + b1;
  end
  else
  begin
       xa := (bi - b1) / (a1 - ai);
       ya := (a1 * xa) + b1;
  end;
  }

  //intersection de Di et (Cxy2)
  {
  if D2_vert then
  begin
       xb := x2;
       yb := (a2 * xb) + b2;
  end
  else
  begin
       xb := (bi - b2) / (a2 - ai);
       yb := (a2 * xb) + b2;
  end;
   }
  if (D2_vert = false) and (Di_vert = false) then
    begin
      if (a2 <> ai) then
        begin
          xb := (bi - b2) / (a2 - ai);
          yb := (a2 * xb) + b2;
        end
      else
        begin
          xb := x2;
          yb := yi;
        end;
    end
  else
    if (D2_vert = false) and (Di_vert = true) then
      begin
        xb := xi;
        yb := (a2 * xb) + b2;
      end
    else
      if (D2_vert = true) and (Di_vert = false) then
        begin
          xb := x2;
          yb := (ai * xb) + bi;
        end
      else
        if (D2_vert = true) and (Di_vert = true) then
          begin
            //impossible
            xb := xi;
            yb := y2;
            //xb := x2;
            //yb := yi;
          end;



end;




function TrouveIntersectionCercleDroite(x1, y1, x2, y2, xc, yc, rayon: extended; var xa, ya, xb, yb: extended): boolean;
var
  a, b: extended;
  GA, GB, GC, delta: extended;

begin
  result := false;
  try
    if (x1 <> x2) then
      begin
        a := (y1 - y2) / (x1 - x2);
        b := y1 - (a * x1);

        GA := 1 + (a * a);
        GB := (-2 * xc) + (2 * a * b) - (2 * yc * a);
        GC := (xc * xc) + (yc * yc) - (rayon * rayon) - (2 * yc * b) + (b * b);
      end
    else
      begin
        a := x1;
        b := 0;

        GA := 1;
        GB := -2 * yc;
        GC := (a * a) + (xc * xc) + (yc * yc) - (rayon * rayon) - (2 * a * xc);
      end;

    delta := (GB * GB) - (4 * GA * GC);

    if delta >= 0 then
      begin
        if delta = 0 then
          begin
            if (x1 <> x2) then
              begin
                xa := -GB / (2 * GA);
                ya := (a * xa) + b;

                xb := xa;
                yb := ya;
              end
            else
              begin
                ya := -GB / (2 * GA);
                xa := x1;

                yb := xa;
                xb := x1;

              end;
          end
        else
          begin
            if (x1 <> x2) then
              begin
                xa := (-GB + sqrt(delta)) / (2 * GA);
                ya := (a * xa) + b;

                xb := (-GB - sqrt(delta)) / (2 * GA);
                yb := (a * xb) + b;
              end
            else
              begin
                ya := (-GB + sqrt(delta)) / (2 * GA);
                xa := x1;


                yb := (-GB - sqrt(delta)) / (2 * GA);
                xb := x1;
              end;
          end;

        result := true;
      end;
  except
  end;
end;


function LargeurPossibleTexte(x1, y1, x2, y2, xc, yc, rayon: extended; var d: extended): boolean;
var
  xa, ya, xb, yb: extended;
begin
  result := TrouveIntersectionCercleDroite(x1, y1, x2, y2, xc, yc, rayon, xa, ya, xb, yb);
  if result then d := norme(xa, ya, xb, yb);
end;

procedure PointLePlusProcheDeP(x1, y1, x2, y2, x, y: extended; var xp, yp: extended);
var
  n1, n2: extended;
begin
  n1 := norme(x1, y1, x, y);
  n2 := norme(x2, y2, x, y);
  if n1 < n2 then
    begin
      xp := x1;
      yp := y1;
    end
  else
    begin
      xp := x2;
      yp := y2;
    end;
end;

end.
