{-----------------------------------------------------------------------}
{                                                                       }
{           Subprogram Name:                                            }
{           Purpose:          Ancestrologie                             }
{           Source Language:  Francais                                  }
{           Auteur:                        }
{           Philippe Cazaux-Moutou                                       }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Description:                                                }
{           Ancestrologie est un Logiciel Libre                         }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Revision History                                            }
{           v#    ,Date       ,Author Name            ,Description      }
{                                                                       }
{-----------------------------------------------------------------------}

unit u_objet_TGraphArbre;

interface

uses
  Classes, windows, contnrs, graphics, extctrls, u_common_graph_type,
  Ibquery, SysUtils, Dialogs,
  u_objet_graph_TGraph,
  u_objet_graph_TPlot,
  u_objet_graph_TPlotList,
  RPBase,
  RPRPTF,
  RPrinter;



type
  TIndividu = class
  private
    fCleFiche: integer;
    fA: TPlot;
    fB: TPlot;
    fLevel: integer;
    fChilds: TObjectList;
    fC: TPlot;
    fNom: string;
    fPrenom: string;
    fCleMere: integer;
    fClePere: integer;
    fDeces: string;
    fNaissance: string;
    fProf: string;
    fSexe: integer;
    fPlotNom: TPlot;
    fPlotProf: TPlot;
    fPlotPrenom: TPlot;
    fPlotNaissance: TPlot;
    fPlotDeces: TPlot;
    procedure OrdreHommeFemme;

  protected
    //  fmax: integer;

    fOrderTete: integer;
    procedure EquilibreNoeuds(var NumOrderTete: integer);

  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear;

    property CleFiche: integer read fCleFiche write fCleFiche;
    property Nom: string read fNom write fNom;
    property Prenom: string read fPrenom write fPrenom;
    property Prof: string read fProf write fProf;
    property Naissance: string read fNaissance write fNaissance;
    property Deces: string read fDeces write fDeces;

    property Sexe: integer read fSexe write fSexe;

    property Level: integer read fLevel write fLevel;
    property Childs: TObjectList read fChilds write fChilds;

    property ClePere: integer read fClePere write fClePere;
    property CleMere: integer read fCleMere write fCleMere;

    property A: TPlot read fA write fA;
    property B: TPlot read fB write fB;
    property C: TPlot read fC write fC;

    property PlotNom: TPlot read fPlotNom write fPlotNom;
    property PlotPrenom: TPlot read fPlotPrenom write fPlotPrenom;
    property PlotProf: TPlot read fPlotProf write fPlotProf;
    property PlotNaissance: TPlot read fPlotNaissance write fPlotNaissance;
    property PlotDeces: TPlot read fPlotDeces write fPlotDeces;
  end;

  TGraphArbre = class(TGraph)

  private
    fNbGeneration: integer;
    fModeArbre: integer;

    fAll: TObjectList;
    fStartIndividu: TIndividu;
    fCleIndiDepart: integer;


    fHeightFont: integer;
    //          fSizeFont100: integer;
    //          fCanvas: TCanvas;

    fFontText: TFont;

    //fNumOrderTete: integer;
    procedure EquilibreNoeuds;

    function GetIndividuByClePere(ClePere: integer): TIndividu;
    function GetIndividuByCleMere(CleMere: integer): TIndividu;
    function GetPere(Clef: integer): TIndividu;

  public
    property NbGeneration: integer read fNbGeneration write fNbGeneration;
    property Descendance: integer read fModeArbre write fModeArbre;

    property CleIndiDepart: integer read fCleIndiDepart write fCleIndiDepart;
    //          property Canvas: TCanvas read fCanvas write fCanvas;
    //          property SizeFont100: integer read fSizeFont100 write fSizeFont100;


    constructor Create; override;
    destructor Destroy; override;

    //chargement
    procedure Load; override;
    procedure LoadDescendance;
    procedure Invert;

    function GetNbIndi: integer;

    //disposition
    procedure Prepare; override;

    procedure CoordChantier_2_CoordViewer(coef: single); override;
    procedure CoordChantier_2_CoordMiniature(coef: single); override;

    procedure Paint(PaintBox: TPaintBox; DecalX, DecalY: integer); override;
    procedure PaintMiniature(PaintBox: TPaintBox; DecalX, DecalY: integer); override;
    function GetRectEncadrement(var R: TFloatRect): boolean; override;


    procedure Print(rp: TReportPrinter); override;
    procedure PreparePrint; override;

    function GetCleIndividuAtXY(X, Y: integer): integer; override;
  end;

implementation

uses
  Math, u_dm, u_rm, u_common_var, u_common_func, u_common_graph_var;

{ TMonGraph }

constructor TGraphArbre.Create;
begin
  inherited Create;

  fNbGeneration := 5;

  fFontText := TFont.create;
  //     fSizeFont100 := _Context.ArbreFont;

  fStartIndividu := TIndividu.create;
  fAll := TObjectList.create(false);
end;


destructor TGraphArbre.Destroy;
begin
  fStartIndividu.free;
  fAll.free;
  fFontText.free;

  inherited Destroy;
end;

procedure TGraphArbre.CoordChantier_2_CoordMiniature(coef: single);
var
  n: integer;
begin
  for n := 0 to fAll.Count - 1 do
    begin
      TIndividu(fAll[n]).A.CoordChantier_2_CoordMiniature(coef);
      TIndividu(fAll[n]).B.CoordChantier_2_CoordMiniature(coef);
      TIndividu(fAll[n]).C.CoordChantier_2_CoordMiniature(coef);
    end;
end;

procedure TGraphArbre.CoordChantier_2_CoordViewer(coef: single);
var
  n: integer;
  indi: TIndividu;
begin
  for n := 0 to fAll.Count - 1 do
    begin
      TIndividu(fAll[n]).A.CoordChantier_2_CoordViewer(coef);
      TIndividu(fAll[n]).B.CoordChantier_2_CoordViewer(coef);
      TIndividu(fAll[n]).C.CoordChantier_2_CoordViewer(coef);
    end;

  //     fHeightFont := round(coef * 2.5);
//     fHeightFont := round((coef * 25.4 * fSizeFont100) / 72);
  fHeightFont := round((coef * 25.4 * fFontText.Size) / 72);


  //les individus
  for n := 0 to fAll.count - 1 do
    begin
      indi := TIndividu(fAll[n]);

      indi.PlotNom.YV := round(TIndividu(fAll[n]).PlotNom.YC * coef);
      indi.PlotPrenom.YV := round(TIndividu(fAll[n]).PlotPrenom.YC * coef);
      indi.PlotProf.YV := round(TIndividu(fAll[n]).PlotProf.YC * coef);
      indi.PlotDeces.YV := round(TIndividu(fAll[n]).PlotDeces.YC * coef);
      indi.PlotNaissance.YV := round(TIndividu(fAll[n]).PlotNaissance.YC * coef);
    end;
end;


function TGraphArbre.GetRectEncadrement(var R: TFloatRect): boolean;
var
  n: integer;
  indi: TIndividu;
begin
  //On encadre les coordonnes chantiers des pieux
//     result := false;

  //Rectangle de dlimitation de tout le chantier, en m
  R.left := Maxint;
  R.Top := Maxint;
  R.Right := -Maxint;
  R.Bottom := -Maxint;


  for n := 0 to fAll.Count - 1 do
    begin
      indi := TIndividu(fAll[n]);

      if R.Left > indi.A.XC then R.Left := indi.A.XC;
      if R.Right < indi.A.XC then R.Right := indi.A.XC;
      if R.Top > indi.A.YC then R.Top := indi.A.YC;
      if R.Bottom < indi.A.YC then R.Bottom := indi.A.YC;

      if R.Left > indi.B.XC then R.Left := indi.B.XC;
      if R.Right < indi.B.XC then R.Right := indi.B.XC;
      if R.Top > indi.B.YC then R.Top := indi.B.YC;
      if R.Bottom < indi.B.YC then R.Bottom := indi.B.YC;
    end;

  R.Right := R.Right + _Context.ArbreMargeLeft;
  R.Bottom := R.Bottom + _Context.ArbreMargeTop;

  REncadrementC := R;

  result := true;
end;

procedure TGraphArbre.Paint(PaintBox: TPaintBox; DecalX, DecalY: integer);
var
  n, k: integer;
  indi: TIndividu;
  R: TRect;
  //   d1: integer;
  s: string;
begin
  //on rcupre la font  utilisez, avec la taille en 100 %
  StringToFont(_Context.ArbreFont, PaintBox.Canvas.Font);

  //on adapte la taille en fonction du zoom
  PaintBox.Canvas.Font.Size := fHeightFont;

  PaintBox.Canvas.Pen.Style := psSolid;



  //     d1 := fHeightFont div 2;


  for n := 0 to fAll.Count - 1 do
    begin
      indi := TIndividu(fAll[n]);

      //le cadre
      if indi.Sexe = 1 then
        begin
          //homme
          if _Context.ArbrePaintFondRecHomme then
            begin
              PaintBox.Canvas.Pen.Color := _Context.ArbreCouleurRectHomme;
              PaintBox.Canvas.Brush.Style := bsSolid;
              PaintBox.Canvas.Brush.Color := _Context.ArbreCouleurFondRectHomme;
              PaintBox.Canvas.Rectangle(indi.A.XV + DecalX, indi.A.YV + DecalY, indi.B.XV + DecalX, indi.B.YV + DecalY);
            end
          else
            begin
              PaintBox.Canvas.Pen.Color := _Context.ArbreCouleurRectHomme;
              PaintBox.Canvas.Brush.Style := bsClear;

              PaintBox.Canvas.MoveTo(indi.A.XV + DecalX, indi.A.YV + DecalY);
              PaintBox.Canvas.LineTo(indi.B.XV + DecalX, indi.A.YV + DecalY);
              PaintBox.Canvas.LineTo(indi.B.XV + DecalX, indi.B.YV + DecalY);
              PaintBox.Canvas.LineTo(indi.A.XV + DecalX, indi.B.YV + DecalY);
              PaintBox.Canvas.LineTo(indi.A.XV + DecalX, indi.A.YV + DecalY);
            end;
        end
      else
        begin
          //femme
          if _Context.ArbrePaintFondRecFemme then
            begin
              PaintBox.Canvas.Pen.Color := _Context.ArbreCouleurRectFemme;
              PaintBox.Canvas.Brush.Style := bsSolid;
              PaintBox.Canvas.Brush.Color := _Context.ArbreCouleurFondRectFemme;
              PaintBox.Canvas.Rectangle(indi.A.XV + DecalX, indi.A.YV + DecalY, indi.B.XV + DecalX, indi.B.YV + DecalY);
            end
          else
            begin
              PaintBox.Canvas.Pen.Color := _Context.ArbreCouleurRectFemme;
              PaintBox.Canvas.Brush.Style := bsClear;

              PaintBox.Canvas.MoveTo(indi.A.XV + DecalX, indi.A.YV + DecalY);
              PaintBox.Canvas.LineTo(indi.B.XV + DecalX, indi.A.YV + DecalY);
              PaintBox.Canvas.LineTo(indi.B.XV + DecalX, indi.B.YV + DecalY);
              PaintBox.Canvas.LineTo(indi.A.XV + DecalX, indi.B.YV + DecalY);
              PaintBox.Canvas.LineTo(indi.A.XV + DecalX, indi.A.YV + DecalY);
            end;
        end;



      //les traits
      PaintBox.Canvas.Pen.Color := _Context.ArbreCouleurLiens;
      if indi.Childs.count > 0 then
        begin
          for k := 0 to indi.Childs.count - 1 do
            begin
              PaintBox.Canvas.MoveTo(indi.B.XV + DecalX, indi.C.YV + DecalY);
              PaintBox.Canvas.LineTo(TIndividu(indi.Childs[k]).A.XV + DecalX, TIndividu(indi.Childs[k]).C.YV + DecalY);
            end;
        end;

      if fHeightFont >= 1 then
        begin
          PaintBox.Canvas.Brush.Style := bsClear;

          //le texte
          {
               indi.PlotNom.YC := indi.A.YC + _Context.ArbreDistNom;
               indi.PlotPrenom.YC := indi.A.YC + _Context.ArbreDistPrenom;
               indi.PlotProf.YC := indi.A.YC + _Context.ArbreDistProfession;
               indi.PlotNaissance.YC := indi.A.YC + _Context.ArbreDistNaissance;
               indi.PlotDeces.YC := indi.A.YC + _Context.ArbreDistDeces;
          }

          if indi.Sexe = 1
            then PaintBox.Canvas.Font.Color := _Context.ArbreCouleurTexteHomme
          else PaintBox.Canvas.Font.Color := _Context.ArbreCouleurTexteFemme;


          if _Context.ArbreShowNom then
            begin
              R := Rect(indi.A.XV + DecalX + 1, indi.PlotNom.YV + DecalY + 1, indi.B.XV + DecalX - 1, indi.B.YV + DecalY - 1);
              //PaintBox.Canvas.TextRect(R,R.left,R.top,Indi.Nom);
              DrawText(PaintBox.Canvas.handle, PChar(Indi.Nom), length(Indi.Nom), R, DT_CENTER);
              //  DrawText(PaintBox.Canvas.handle, PChar(PaintBox.Canvas.Font.Name), length(PaintBox.Canvas.Font.Name), R, DT_CENTER);
            end;

          if _Context.ArbreShowPrenom then
            begin
              R := Rect(indi.A.XV + DecalX + 1, indi.PlotPrenom.YV + DecalY + 1, indi.B.XV + DecalX - 1, indi.B.YV + DecalY - 1);
              DrawText(PaintBox.Canvas.handle, PChar(Indi.Prenom), length(Indi.Prenom), R, DT_CENTER);
            end;

          if _Context.ArbreShowProfession then
            begin
              R := Rect(indi.A.XV + DecalX + 1, indi.PlotProf.YV + DecalY + 1, indi.B.XV + DecalX - 1, indi.B.YV + DecalY - 1);
              DrawText(PaintBox.Canvas.handle, PChar(Indi.Prof), length(Indi.Prof), R, DT_CENTER);
            end;

          if _Context.ArbreShowNaissance then
            begin
              if Indi.Naissance <> '' then
                begin
                  s := '' + Indi.Naissance;
                  R := Rect(indi.A.XV + DecalX + 1, indi.PlotNaissance.YV + DecalY + 1, indi.B.XV + DecalX - 1, indi.B.YV + DecalY - 1);
                  DrawText(PaintBox.Canvas.handle, PChar(s), length(s), R, DT_CENTER);
                end;
            end;

          if _Context.ArbreShowDeces then
            begin
              if Indi.Deces <> '' then
                begin
                  s := '+' + Indi.Deces;
                  R := Rect(indi.A.XV + DecalX + 1, indi.PlotDeces.YV + DecalY + 1, indi.B.XV + DecalX - 1, indi.B.YV + DecalY - 1);
                  DrawText(PaintBox.Canvas.handle, PChar(s), length(s), R, DT_CENTER);
                end;
            end;


          //          PaintBox.Canvas.TextRect(R, indi.A.XV + DecalX, indi.A.YV + DecalY, Indi.Nom);
        end;
    end;
end;





procedure TGraphArbre.PaintMiniature(PaintBox: TPaintBox; DecalX, DecalY: integer);
var
  n, k: integer;
  indi: TIndividu;
begin
  PaintBox.Canvas.Pen.Color := clRed;
  PaintBox.Canvas.Pen.Style := psSolid;

  for n := 0 to fAll.Count - 1 do
    begin
      indi := TIndividu(fAll[n]);

      //le cadre
      PaintBox.Canvas.MoveTo(indi.A.XM + DecalX, indi.A.YM + DecalY);
      PaintBox.Canvas.LineTo(indi.B.XM + DecalX, indi.A.YM + DecalY);
      PaintBox.Canvas.LineTo(indi.B.XM + DecalX, indi.B.YM + DecalY);
      PaintBox.Canvas.LineTo(indi.A.XM + DecalX, indi.B.YM + DecalY);
      PaintBox.Canvas.LineTo(indi.A.XM + DecalX, indi.A.YM + DecalY);

      //les traits
      if indi.Childs.count > 0 then
        begin
          for k := 0 to indi.Childs.count - 1 do
            begin
              PaintBox.Canvas.MoveTo(indi.B.XM + DecalX, indi.C.YM + DecalY);
              PaintBox.Canvas.LineTo(TIndividu(indi.Childs[k]).A.XM + DecalX, TIndividu(indi.Childs[k]).C.YM + DecalY);
            end;
        end;
    end;
end;

function TGraphArbre.GetPere(Clef: integer): TIndividu;
var
  n: integer;
begin
  result := nil;
  for n := 0 to fAll.count - 1 do
    begin
      if TIndividu(fAll[n]).CleFiche = Clef then
        begin
          result := TIndividu(fAll[n]);
          break;
        end;
    end;
end;

procedure TGraphArbre.Load;
var
  n, level, CleFiche: integer;
  indi, indiParent: TIndividu;
  Q: TIBQuery;
begin
  fAll.Clear;
  fStartIndividu.Clear;

  if fModeArbre = 1 then
    begin
      LoadDescendance;
      exit;
    end;

  //celui de dpart
  fAll.Add(fStartIndividu);


  //On charge les individus
  Q := TIBQuery.Create(nil);
  try
    Q.Database := dm.ibd_BASE;
    Q.SQL.Clear;
    Q.SQL.Add('select * from PROC_ARBRE_EXPORT(' + inttostr(fCleIndiDepart) + ',' + inttostr(fNbGeneration - 1) + ',' + inttostr(dm.NumDossier) + ',0)');
    Q.Open;
    Q.last;
    Q.first;

    for n := 1 to Q.RecordCount do
      begin
        Level := Q.FieldByName('NIVEAU').AsInteger;
        CleFiche := Q.FieldByName('CLE_FICHE').AsInteger;

        if level = 0 then
          begin
            indi := fStartIndividu;
            indiParent := nil;
          end
        else
          begin
            //le sexe ne peut-tre que 1 ou 2
            if Q.FieldByName('SEXE').AsInteger = 1
              then indiParent := GetIndividuByClePere(CleFiche)
            else indiParent := GetIndividuByCleMere(CleFiche);

            if indiParent <> nil then
              begin
                indi := TIndividu.create;
                fAll.Add(indi);
              end
            else indi := nil;

          end;


        if indi <> nil then
          begin
            //1=Homme, 2=Femme
            indi.Sexe := Q.FieldByName('SEXE').AsInteger;

            indi.CleFiche := CleFiche;
            indi.level := level;
            indi.ClePere := Q.FieldByName('CLE_PERE').AsInteger;
            indi.CleMere := Q.FieldByName('CLE_MERE').AsInteger;
            indi.Nom := Q.FieldByName('NOM').AsString;
            indi.Prof := Q.FieldByName('OCCUPATION').AsString;
            indi.Prenom := Q.FieldByName('PRENOM').AsString;
            indi.Naissance := Q.FieldByName('DATE_NAISSANCE').AsString;
            indi.Deces := Q.FieldByName('DATE_DECES').AsString;

            if indiParent <> nil then
              begin
                indiParent.Childs.Add(indi);
              end;
          end;

        Q.next;
      end;

  finally
    Q.Free;
  end;

  //on s'assure que les hommes sont  droite, et les femmes  gauche
  fStartIndividu.OrdreHommeFemme;

  //On quilibre les ttes
  EquilibreNoeuds;
end;

procedure TGraphArbre.LoadDescendance;
var
  n, level, CleFiche: integer;
  indi, indiParent: TIndividu;
  Q: TIBQuery;
begin
  fAll.Clear;
  fStartIndividu.Clear;

  //celui de dpart
  fAll.Add(fStartIndividu);


  //On charge les individus
  Q := TIBQuery.Create(nil);
  try
    Q.Database := dm.ibd_BASE;
    Q.SQL.Clear;
    Q.SQL.Add('select * from PROC_DESCENDANCE(:INDI, :GENE, :DOSSIER) ORDER BY NIVEAU, SOSA');

    Q.Params[0].AsInteger := fCleIndiDepart;
    Q.Params[1].AsInteger := fNbGeneration;
    Q.Params[2].AsInteger := dm.NumDossier;

    Q.Open;
    Q.last;
    Q.first;

    for n := 1 to Q.RecordCount do
      begin
        Level := Q.FieldByName('NIVEAU').AsInteger;
        CleFiche := Q.FieldByName('CLE_FICHE').AsInteger;

        if level = 1 then
          begin
            indi := fStartIndividu;
            indiParent := nil;
          end
        else
          begin
            //le sexe ne peut-tre que 1 ou 2

            indiParent := GetPere(Q.FieldByName('CLE_PERE').AsInteger);
            if indiparent = nil then indiParent := GetPere(Q.FieldByName('CLE_MERE').AsInteger);

            if Level <> 0 then
              begin
                indi := TIndividu.create;
                fAll.Add(indi);
              end
            else indi := nil;

          end;

        if indi <> nil then
          begin
            //1=Homme, 2=Femme
            indi.Sexe := Q.FieldByName('SEXE').AsInteger;

            indi.CleFiche := CleFiche;
            indi.level := level;
            indi.ClePere := Q.FieldByName('CLE_PERE').AsInteger;
            indi.CleMere := Q.FieldByName('CLE_MERE').AsInteger;
            indi.Nom := Q.FieldByName('NOM').AsString;
            indi.Prenom := Q.FieldByName('PRENOM').AsString;
            indi.Naissance := Q.FieldByName('DATE_NAISSANCE').AsString;
            indi.Deces := Q.FieldByName('DATE_DECES').AsString;

            if indiParent <> nil then
              begin
                indiParent.Childs.Add(indi);
              end;
          end;

        Q.next;
      end;


  finally
    Q.Free;
  end;

  //on s'assure que les hommes sont  droite, et les femmes  gauche
  fStartIndividu.OrdreHommeFemme;

  //On quilibre les ttes
  EquilibreNoeuds;
end;


procedure TGraphArbre.Prepare;
var
  w, n, k: integer;
  indi: TIndividu;
  //     totLastIndi: integer;
  levelMax: integer;
  Y, YA, YB: single;
begin
  //on assigne la police, qui correspond  celle utilise en 100%
 //     StringToFont(_Context.ArbreFont, fCanvas.Font);
  StringToFont(_Context.ArbreFont, fFontText);
  //     fCanvas.Font.Size := fSizeFont100;

       //on regarde dans les branches, celui qui est l'individu terminal
  levelMax := 0;
  for n := 0 to fAll.count - 1 do
    begin
      if TIndividu(fAll[n]).Level > levelMax then levelMax := TIndividu(fAll[n]).Level;
    end;

  //on positionne les ttes
  for n := 0 to fAll.count - 1 do
    begin
      if TIndividu(fAll[n]).Childs.count = 0 then
        begin
          indi := TIndividu(fAll[n]);
          //en X :
          indi.A.XC := _Context.ArbreMargeLeft + (indi.Level * (_Context.ArbreSpaceBetween2Gen + _Context.ArbreWidthBox));
          indi.B.XC := indi.A.XC + _Context.ArbreWidthBox;

          //en Y :
          indi.A.YC := _Context.ArbreMargeTop + (_Context.ArbreHeightBox + _Context.ArbreSpaceBetween2Indi) * (indi.fOrderTete - 1);
          indi.B.YC := indi.A.YC + _Context.ArbreHeightBox;

          indi.C.YC := (indi.A.YC) + ((indi.B.YC - indi.A.YC) / 2);
        end;
    end;

  for n := levelMax downto 0 do
    begin
      for w := 0 to fAll.count - 1 do
        begin
          indi := TIndividu(fAll[w]);

          if (indi.Level = n) and (indi.Childs.count <> 0) then
            begin
              //en X :
              indi.A.XC := _Context.ArbreMargeLeft + (indi.Level * (_Context.ArbreSpaceBetween2Gen + _Context.ArbreWidthBox));
              indi.B.XC := indi.A.XC + _Context.ArbreWidthBox;

              //nb d'enfants
              if indi.Childs.count = 1 then
                begin
                  //en Y :
                  indi.A.YC := TIndividu(indi.childs[0]).A.YC;
                  indi.B.YC := TIndividu(indi.childs[0]).B.YC;
                end
              else
                begin
                  YA := MaxInt;
                  YB := -MaxInt;
                  for k := 0 to indi.childs.count - 1 do
                    begin
                      if YA > TIndividu(indi.childs[k]).A.YC then YA := TIndividu(indi.childs[k]).A.YC;
                      if YB < TIndividu(indi.childs[k]).B.YC then YB := TIndividu(indi.childs[k]).B.YC;
                    end;

                  Y := YA + ((YB - YA) / 2) - (_Context.ArbreHeightBox / 2);
                  indi.A.YC := Y;
                  indi.B.YC := Y + _Context.ArbreHeightBox;
                end;

              indi.C.YC := (indi.A.YC) + ((indi.B.YC - indi.A.YC) / 2);

            end;
        end;
    end;


  //les textes
  for n := 0 to fAll.count - 1 do
    begin
      indi := TIndividu(fAll[n]);

      indi.PlotNom.YC := indi.A.YC + _Context.ArbreDistNom;
      indi.PlotPrenom.YC := indi.A.YC + _Context.ArbreDistPrenom;
      indi.PlotProf.YC := indi.A.YC + _Context.ArbreDistProfession;
      indi.PlotNaissance.YC := indi.A.YC + _Context.ArbreDistNaissance;
      indi.PlotDeces.YC := indi.A.YC + _Context.ArbreDistDeces;
    end;
end;

{ TIndividu }

procedure TIndividu.Clear;
begin
  fCleFiche := -1;
  fLevel := 0;
  fNom := '';
  fPrenom := '';
  fProf := '';
  fDeces := '';
  fNaissance := '';
  fSexe := -1;

  fCleMere := -1;
  fClePere := -1;

  fChilds.Clear;
end;

constructor TIndividu.Create;
begin
  fChilds := TObjectList.create(true);

  fA := TPlot.create;
  fB := TPlot.create;
  fC := TPlot.create;

  fPlotNom := TPlot.create;
  fPlotPrenom := TPlot.create;
  fPlotProf := TPlot.create;
  fPlotDeces := TPlot.create;
  fPlotNaissance := TPlot.create;

  Clear;
end;

destructor TIndividu.Destroy;
begin
  fChilds.Free;
  fA.Free;
  fB.Free;
  fC.Free;

  fPlotNom.Free;
  fPlotPrenom.Free;
  fPlotProf.Free;
  fPlotDeces.Free;
  fPlotNaissance.Free;

  inherited Destroy;
end;

function TGraphArbre.GetIndividuByClePere(ClePere: integer): TIndividu;
var
  n: integer;
begin
  result := nil;
  for n := 0 to fAll.count - 1 do
    begin
      if TIndividu(fAll[n]).ClePere = ClePere then
        begin
          result := TIndividu(fAll[n]);
          break;
        end;
    end;
end;

function TGraphArbre.GetIndividuByCleMere(CleMere: integer): TIndividu;
var
  n: integer;
begin
  result := nil;
  for n := 0 to fAll.count - 1 do
    begin
      if TIndividu(fAll[n]).CleMere = CleMere then
        begin
          result := TIndividu(fAll[n]);
          break;
        end;
    end;
end;



procedure TGraphArbre.EquilibreNoeuds;
var
  k: integer;
begin
  //     fNumOrderTete := 0;
  //     for n := 0 to fAll.count - 1 do TIndividu(fAll[n]).fmax := -1;
  k := 0;

  fStartIndividu.EquilibreNoeuds(k);
end;

procedure TIndividu.EquilibreNoeuds(var NumOrderTete: integer);
var
  n: integer;
begin
  if fChilds.count = 0 then
    begin
      inc(NumOrderTete);
      fOrderTete := NumOrderTete;
    end
  else
    begin
      for n := 0 to fChilds.count - 1 do TIndividu(fChilds[n]).EquilibreNoeuds(NumOrderTete);
    end;
end;


procedure TIndividu.OrdreHommeFemme;
var
  n: integer;
begin
  case fChilds.count of
    2: begin
        if TIndividu(fChilds[0]).Sexe = 2 then //homme
          begin
            //invertion
            fChilds.Exchange(0, 1);
          end;
      end;
  end;

  //on ordonne les suivants
  for n := 0 to fChilds.count - 1 do TIndividu(fChilds[n]).OrdreHommeFemme;
end;


procedure TGraphArbre.Invert;
begin

end;

function TGraphArbre.GetNbIndi: integer;
begin
  result := fAll.count;
end;

procedure TGraphArbre.Print(rp: TReportPrinter);
var
  n, k: integer;
  indi: TIndividu;
  //     R: TRect;
  s: string;
  DecalX, DecalY: extended;


  procedure PrintTheText(R: TRect; Text: string);
  var
    w, h, l: double;
    OkName: boolean;
    //          Ri: TRect;
  begin
    w := rp.TextWidth(text);
    h := RPTFTextHeight(rp, text);

    //est-ce que le nom du pieu rentre dans le rectangle de dlimitation ?
    okName := (w <= abs(R.right - R.Left + 1));

    //soit le texte entre dans le rectangle
    if okName then
      begin
        l := (abs(R.left - R.right) - w) / 2;
        rp.PrintXY(R.left + l, R.top + h, text);
      end
    else
      begin
        rp.TextRect(R, R.left, R.top + h, text);
      end;
  end;


  function MyCreateRect(X1, Y1, X2, Y2: double): TRect;
  begin { CreateRect }
    Result.Left := round(X1);
    Result.Top := round(Y1);
    Result.Right := round(X2);
    Result.Bottom := round(Y2);
  end;

begin
  DecalX := rp.MarginLeft + rm.DecalagePapierX;
  DecalY := rp.MarginTop + rm.DecalagePapierY;


  //on rcupre la font  utilisez, avec la taille en 100 %
  rp.SetFont(fFontText.Name, fFontText.Size);
  rp.Bold := (fsBold in fFontText.Style);
  rp.Underline := (fsUnderLine in fFontText.Style);
  rp.Italic := (fsItalic in fFontText.Style);
  rp.StrikeOut := (fsStrikeOut in fFontText.Style);
  rp.FontColor := fFontText.Color;



  for n := 0 to fAll.Count - 1 do
    begin
      indi := TIndividu(fAll[n]);

      //le cadre
      if indi.Sexe = 1 then
        begin
          //homme
          if _Context.ArbrePaintFondRecHomme then
            begin
              rp.SetPen(_Context.ArbreCouleurRectHomme, psSolid, 1, pmCopy);
              rp.SetBrush(_Context.ArbreCouleurFondRectHomme, bsSolid, nil);

              //                    PaintBox.Canvas.Pen.Color := _Context.ArbreCouleurRectHomme;
              //                    PaintBox.Canvas.Brush.Style := bsSolid;
              //                    PaintBox.Canvas.Brush.Color := _Context.ArbreCouleurFondRectHomme;
              rp.Rectangle(indi.A.XC + DecalX, indi.A.YC + DecalY, indi.B.XC + DecalX, indi.B.YC + DecalY);
            end
          else
            begin
              rp.SetPen(_Context.ArbreCouleurRectHomme, psSolid, 1, pmCopy);
              rp.SetBrush(clBlack, bsClear, nil);

              //                    PaintBox.Canvas.Pen.Color := _Context.ArbreCouleurRectHomme;
              //                    PaintBox.Canvas.Brush.Style := bsClear;

              rp.MoveTo(indi.A.XC + DecalX, indi.A.YC + DecalY);
              rp.LineTo(indi.B.XC + DecalX, indi.A.YC + DecalY);
              rp.LineTo(indi.B.XC + DecalX, indi.B.YC + DecalY);
              rp.LineTo(indi.A.XC + DecalX, indi.B.YC + DecalY);
              rp.LineTo(indi.A.XC + DecalX, indi.A.YC + DecalY);
            end;
        end
      else
        begin
          //femme
          if _Context.ArbrePaintFondRecFemme then
            begin
              rp.SetPen(_Context.ArbreCouleurRectFemme, psSolid, 1, pmCopy);
              rp.SetBrush(_Context.ArbreCouleurFondRectFemme, bsSolid, nil);

              // PaintBox.Canvas.Pen.Color := _Context.ArbreCouleurRectFemme;
              // PaintBox.Canvas.Brush.Style := bsSolid;
              // PaintBox.Canvas.Brush.Color := _Context.ArbreCouleurFondRectFemme;
              rp.Rectangle(indi.A.XC + DecalX, indi.A.YC + DecalY, indi.B.XC + DecalX, indi.B.YC + DecalY);
            end
          else
            begin
              rp.SetPen(_Context.ArbreCouleurRectFemme, psSolid, 1, pmCopy);
              rp.SetBrush(clBlack, bsClear, nil);
              // PaintBox.Canvas.Pen.Color := _Context.ArbreCouleurRectFemme;
              // PaintBox.Canvas.Brush.Style := bsClear;

              rp.MoveTo(indi.A.XC + DecalX, indi.A.YC + DecalY);
              rp.LineTo(indi.B.XC + DecalX, indi.A.YC + DecalY);
              rp.LineTo(indi.B.XC + DecalX, indi.B.YC + DecalY);
              rp.LineTo(indi.A.XC + DecalX, indi.B.YC + DecalY);
              rp.LineTo(indi.A.XC + DecalX, indi.A.YC + DecalY);
            end;
        end;



      //les traits
      rp.SetPen(_Context.ArbreCouleurLiens, psSolid, 2, pmCopy);
      rp.SetBrush(clBlack, bsClear, nil);
      //          PaintBox.Canvas.Pen.Color := _Context.ArbreCouleurLiens;
      if indi.Childs.count > 0 then
        begin
          for k := 0 to indi.Childs.count - 1 do
            begin
              rp.MoveTo(indi.B.XC + DecalX, indi.C.YC + DecalY);
              rp.LineTo(TIndividu(indi.Childs[k]).A.XC + DecalX, TIndividu(indi.Childs[k]).C.YC + DecalY);
            end;
        end;


      //          if fHeightFont >= 1 then
      begin
        //le texte
        if indi.Sexe = 1
          then rp.FontColor := _Context.ArbreCouleurTexteHomme
        else rp.FontColor := _Context.ArbreCouleurTexteFemme;

        if _Context.ArbreShowNom then
          begin
            PrintTheText(MyCreateRect(indi.A.XC + DecalX, indi.PlotNom.YC + DecalY, indi.B.XC + DecalX, indi.B.YC + DecalY), Indi.Nom);
          end;

        if _Context.ArbreShowPrenom then
          begin
            PrintTheText(MyCreateRect(indi.A.XC + DecalX, indi.PlotPrenom.YC + DecalY, indi.B.XC + DecalX, indi.B.YC + DecalY), Indi.Prenom);
          end;

        if _Context.ArbreShowProfession then
          begin
            PrintTheText(MyCreateRect(indi.A.XC + DecalX, indi.PlotProf.YC + DecalY, indi.B.XC + DecalX, indi.B.YC + DecalY), Indi.Prof);
          end;

        if _Context.ArbreShowNaissance then
          begin
            if Indi.Naissance <> '' then
              begin
                s := '' + Indi.Naissance;
                PrintTheText(MyCreateRect(indi.A.XC + DecalX, indi.PlotNaissance.YC + DecalY, indi.B.XC + DecalX, indi.B.YC + DecalY), s);
              end;
          end;

        if _Context.ArbreShowDeces then
          begin
            if Indi.Deces <> '' then
              begin
                s := '+' + Indi.Deces;
                PrintTheText(MyCreateRect(indi.A.XC + DecalX, indi.PlotDeces.YC + DecalY, indi.B.XC + DecalX, indi.B.YC + DecalY), s);
              end;
          end;

      end;
    end;
end;


procedure TGraphArbre.PreparePrint;
{
var
     n: integer;
     indi: TIndividu;
     coef: extended;
     fZoom: integer;
     }
begin
  {
       fZoom := 100;
       coef := (fZoom * 72) / (100 * 25.4);
       coef := 1;

       for n := 0 to fAll.Count - 1 do
       begin
            TIndividu(fAll[n]).A.CoordChantier_2_CoordPapier(coef);
            TIndividu(fAll[n]).B.CoordChantier_2_CoordPapier(coef);
            TIndividu(fAll[n]).C.CoordChantier_2_CoordPapier(coef);
       end;

       ////////////     fHeightFont := round((coef * 25.4 * fFontText.Size) / 72);


            //les individus
       for n := 0 to fAll.count - 1 do
       begin
            indi := TIndividu(fAll[n]);

            indi.PlotNom.YP := round(TIndividu(fAll[n]).PlotNom.YC * coef);
            indi.PlotPrenom.YP := round(TIndividu(fAll[n]).PlotPrenom.YC * coef);
            indi.PlotProf.YP := round(TIndividu(fAll[n]).PlotProf.YC * coef);
            indi.PlotDeces.YP := round(TIndividu(fAll[n]).PlotDeces.YC * coef);
            indi.PlotNaissance.YP := round(TIndividu(fAll[n]).PlotNaissance.YC * coef);
       end;
       }
end;

function TGraphArbre.GetCleIndividuAtXY(X, Y: integer): integer;
var
  n: integer;
  indi: TIndividu;
begin
  result := -1;
  for n := 0 to fAll.Count - 1 do
    begin
      indi := TIndividu(fAll[n]);
      if (X >= indi.A.XV + _Viewer.DecalX)
        and (X <= indi.B.XV + _Viewer.DecalX)
        and (Y >= indi.A.YV + _Viewer.DecalY)
        and (Y <= indi.B.YV + _Viewer.DecalY) then
        begin
          result := indi.CleFiche;
          break;
        end;
    end;
end;

end.
