{-----------------------------------------------------------------------}
{                                                                       }
{           Subprogram Name:                                            }
{           Purpose:          Ancestrologie                             }
{           Source Language:  Francais                                  }
{           Auteur:                        }
{           Philippe Cazaux-Moutou                                       }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Description:                                                }
{           Ancestrologie est un Logiciel Libre                         }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Revision History                                            }
{           v#    ,Date       ,Author Name            ,Description      }
{                                                                       }
{-----------------------------------------------------------------------}

unit u_objet_graph_TVue;

interface

uses
  Classes,
  Graphics,
  inifiles,
  u_common_graph_type;


const
  //Section rpertoriant les vues
  _LIST_VUES = 'LIST_VUES';



  //Sous-sections utiliss pour le section d'une vue (elles sont toutes dans le mme fichier)
  _COLORS = '_COLORS';
  _DESCRIPTION = '_DESCRIPTION';
  _PIEUX_VISIBLE = '_PIEUX_VISIBLE';

type

  TVue = class
  private
    fZoom: integer;
    fPieuxVisible: TStringList;
    fName: string;
    //	fModified: boolean;
    fColorBackground: TColor;
    fColorPieuxSelected: TColor;
    fColorPieuxTrace: TColor;
    fColorPieuxOutOfDepth: TColor;
    fColorPieux: TColor;
    fYChantierInCentre: single;
    fXChantierInCentre: single;
    //		fDepth: single;
    fFont: TFont;
    fEchelle: integer;
    fShowTrace: boolean;
    fCote: single;

  public
    //Nom de la vue, cad nom de la section dans le fichiers "vues.ini"
    property Name: string read fName write fName;

    //Description de la vue : Zoom utilis
    property Zoom: integer read fZoom write fZoom;

    //Profondeur courante
   // property Depth: single read fDepth write fDepth;
    property Cote: single read fCote write fCote;

    //Coordonnes du chantier plac au centre du Viewer
    property XChantierInCentre: single read fXChantierInCentre write fXChantierInCentre;
    property YChantierInCentre: single read fYChantierInCentre write fYChantierInCentre;

    //Couleurs utilises
    property ColorBackground: TColor read fColorBackground write fColorBackground;
    property ColorPieuxSelected: TColor read fColorPieuxSelected write fColorPieuxSelected;
    property ColorPieux: TColor read fColorPieux write fColorPieux;
    property ColorPieuxOutOfDepth: TColor read fColorPieuxOutOfDepth write fColorPieuxOutOfDepth;
    property ColorPieuxTrace: TColor read fColorPieuxTrace write fColorPieuxTrace;
    property Font: TFont read fFont write fFont;
    property Echelle: integer read fEchelle write fEchelle;
    property ShowTrace: boolean read fShowTrace write fShowTrace;

    //List de pieux visibles (le nom correspond au nom du fichier SB du pieu)
    property PieuxVisible: TStringList read fPieuxVisible write fPieuxVisible;

    //  property Modified: boolean read fModified write fModified;

    constructor Create;
    destructor Destroy; override;
    procedure Init;
    function SaveToFile(RelPathChantier: string): boolean;
    function LoadFromFile(NameVue, RelPathChantier: string) {; IsSavedInGlobalFileVues: boolean)}: boolean;
    function FlashViewer(PtrViewer: TObject): boolean;
  end;


implementation

uses
  sysutils,
  u_common_graph_var,
  u_common_graph_const,
  u_common_graph_func,
  u_objet_graph_TViewer;

{ TVue }

{
procedure TVue.Assign(source: TVue);
begin

end;
 }
{
procedure TVue.AssignToViewer(Vieewer: TViewer);
begin

end;
}

constructor TVue.Create;
begin
  fPieuxVisible := TStringList.create;
  fFont := TFont.create;
  Init;
end;

destructor TVue.Destroy;
begin
  fPieuxVisible.Free;
  fFont.Free;

  inherited;
end;

{
procedure TVue.FlashViewer(Viewer: TViewer);
begin

end;
 }



procedure TVue.Init;
begin
  //tout, sauf le nom
  fZoom := 100;
  //     fDepth := 0;
  fCote := 0;
  fPieuxVisible.Clear;
  //     fEchelle := _ECHELLE_PIXELS;
  fShowTrace := true;
  fColorBackground := clBlack;
  fColorPieuxSelected := clYellow;
  fColorPieuxTrace := clBlue;
  fColorPieuxOutOfDepth := clGreen;
  fColorPieux := clRed;
end;


function TVue.LoadFromFile(NameVue, RelPathChantier: string {; IsSavedInGlobalFileVues: boolean}): boolean;
var
  n, k: integer;
  s: string;
  fIni: TIniFile;
begin
  result := false;

  Init;
  fName := NameVue;
  //     fSavedInGlobalFileVues := IsSavedInGlobalFileVues;

   //Toutes les informations sont dans la section fName
  if fName <> '' then
    begin
      fIni := TIniFile.create(_PathChantiers + RelPathChantier + _RELPATH_GESTION_PLANS + _FILE_NAME_CONFIG_VUES);
      try
        try
          //On vrifie que les sections existent
          if fIni.SectionExists(fName + _DESCRIPTION) and fIni.SectionExists(fName + _COLORS) and fIni.SectionExists(fName + _PIEUX_VISIBLE) then
            begin
              //Description
              fZoom := fIni.ReadInteger(fName + _DESCRIPTION, 'Zoom', 0);
              // fDepth := fIni.ReadFloat(fName + _DESCRIPTION, 'Depth', 0);
              fCote := fIni.ReadFloat(fName + _DESCRIPTION, 'Cote', 0);
              fXChantierInCentre := fIni.ReadFloat(fName + _DESCRIPTION, 'XChantierInCentre', 0);
              fYChantierInCentre := fIni.ReadFloat(fName + _DESCRIPTION, 'YChantierInCentre', 0);
              StringToFont(fIni.ReadString(fName + _DESCRIPTION, 'Font', ''), fFont);
              //					fEchelle := fIni.ReadInteger(fName + _DESCRIPTION, 'Echelle', _ECHELLE_PIXELS);
              k := fIni.ReadInteger(fName + _DESCRIPTION, 'NbPieuxVisible', 0);
              fShowTrace := fIni.ReadBool(fName + _DESCRIPTION, 'ShowTrace', true);

              //Couleurs
              ColorBackground := fIni.ReadInteger(fName + _COLORS, 'ColorBackground', clBlack);
              fColorPieuxSelected := fIni.ReadInteger(fName + _COLORS, 'ColorPieuxSelected', clYellow);
              fColorPieux := fIni.ReadInteger(fName + _COLORS, 'ColorPieux', clRed);
              fColorPieuxOutOfDepth := fIni.ReadInteger(fName + _COLORS, 'ColorPieuxOutOfDepth', clGreen);
              fColorPieuxTrace := fIni.ReadInteger(fName + _COLORS, 'ColorPieuxTrace', clLime);

              //Uniquement les pieux visibles
              for n := 0 to k - 1 do
                begin
                  s := trim(fIni.ReadString(fName + _PIEUX_VISIBLE, 'Pieu' + inttostr(n), ''));
                  if s <> '' then fPieuxVisible.Add(s);
                end;

              result := true;
            end;
        except
        end;
      finally
        if fEchelle < 1 then fEchelle := 1;
        fIni.Free;
      end;
    end;
end;



function TVue.SaveToFile(RelPathChantier: string): boolean;
var
  fIni: TIniFile;
  n: integer;
begin
  result := false;

  //Toutes les informations sont dans la section fName
  if fName <> '' then
    begin
      //          if fSavedInGlobalFileVues
      fIni := TIniFile.create(_PathChantiers + RelPathChantier + _RELPATH_GESTION_PLANS + _FILE_NAME_CONFIG_VUES);
      //          else fIni := TIniFile.create(_PathChantiers + RelPathChantier + _RELPATH_GESTION_PLANS + _FILE_NAME_CONFIG_CHANTIER);
      try
        try
          //Description
          try
            fIni.EraseSection(fName + _DESCRIPTION);
          except
          end;
          fIni.WriteInteger(fName + _DESCRIPTION, 'Zoom', fZoom);
          //fIni.WriteFloat(fName + _DESCRIPTION, 'Depth', fDepth);
          fIni.WriteFloat(fName + _DESCRIPTION, 'Cote', fCote);
          fIni.WriteFloat(fName + _DESCRIPTION, 'XChantierInCentre', fXChantierInCentre);
          fIni.WriteFloat(fName + _DESCRIPTION, 'YChantierInCentre', fYChantierInCentre);
          fIni.WriteInteger(fName + _DESCRIPTION, 'NbPieuxVisible', fPieuxVisible.Count);
          fIni.WriteString(fName + _DESCRIPTION, 'Font', FontToString(fFont));
          fIni.WriteInteger(fName + _DESCRIPTION, 'Echelle', fEchelle);
          fIni.WriteBool(fName + _DESCRIPTION, 'ShowTrace', fShowTrace);

          //Couleurs
          try
            fIni.EraseSection(fName + _COLORS);
          except
          end;
          fIni.WriteInteger(fName + _COLORS, 'ColorBackground', ColorBackground);
          fIni.WriteInteger(fName + _COLORS, 'ColorPieuxSelected', fColorPieuxSelected);
          fIni.WriteInteger(fName + _COLORS, 'ColorPieux', fColorPieux);
          fIni.WriteInteger(fName + _COLORS, 'ColorPieuxOutOfDepth', fColorPieuxOutOfDepth);
          fIni.WriteInteger(fName + _COLORS, 'ColorPieuxTrace', fColorPieuxTrace);


          //Uniquement les pieux visibles
          try
            fIni.EraseSection(fName + _PIEUX_VISIBLE);
          except
          end;
          for n := 0 to fPieuxVisible.count - 1 do fIni.WriteString(fName + _PIEUX_VISIBLE, 'Pieu' + inttostr(n), fPieuxVisible[n]);


          fIni.UpdateFile;

          result := true;
        except
        end;
      finally
        fIni.Free;
      end;
    end;
end;


function TVue.FlashViewer(PtrViewer: TObject): boolean;
var
  //     n: integer;
  Viewer: TViewer;
begin
  result := false;
  Init;

  if (PtrViewer <> nil) and (PtrViewer is TViewer) then
    begin
      Viewer := TViewer(PtrViewer);
      if (Viewer.Chantier <> nil) then
        begin

          //Description de la vue : Zoom utilis
          fZoom := Viewer.Zoom;


          //Coordonnes du chantier plac au centre du Viewer
          Viewer.CoordViewer_To_CoordChantier(Viewer.PaintBox.Width div 2, Viewer.PaintBox.Height div 2, fXChantierInCentre, fYChantierInCentre);

          fFont.Assign(Viewer.Font);
          //               fEchelle := Viewer.Echelle;


                         //Couleurs utilises
          fColorBackground := Viewer.ColorBackground;
          fColorPieuxSelected := Viewer.ColorPieuxSelected;
          fColorPieux := Viewer.ColorPieux;
          fColorPieuxOutOfDepth := Viewer.ColorPieuxOutOfDepth;
          fColorPieuxTrace := Viewer.ColorPieuxTrace;


          result := True;
        end;
    end;
end;


end.
