{-----------------------------------------------------------------------}
{                                                                       }
{           Subprogram Name:                                            }
{           Purpose:          Ancestrologie                             }
{           Source Language:  Francais                                  }
{           Auteur:                        }
{           Philippe Cazaux-Moutou                                       }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Description:                                                }
{           Ancestrologie est un Logiciel Libre                         }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Revision History                                            }
{           v#    ,Date       ,Author Name            ,Description      }
{                                                                       }
{-----------------------------------------------------------------------}

unit u_objet_graph_TVueList;

interface


uses
  Classes,
  u_objet_graph_TVue;


const

  MaxListSize = Maxint div 16;
  SListIndexError = 'Index hors borne !';
  SListCapacityError = 'Erreur de capacite !';
  SListCountError = 'Erreur de Count';


type

  PVueArray = ^TVueArray;
  TVueArray = array[0..MaxListSize - 1] of TVue;


  PPointerList = ^TPointerList;
  TPointerList = array[0..MaxListSize - 1] of Pointer;

  TVueList = class(TObject)

  private
    FList: PVueArray;
    FCount: Integer;
    FCapacity: Integer;
  protected
    function Get(Index: Integer): TVue;
    procedure Grow; virtual;
    procedure Put(Index: Integer; Item: TVue);
    procedure SetCapacity(NewCapacity: Integer);
    procedure SetCount(NewCount: Integer);
  public
    destructor Destroy; override;
    function Add(Item: TVue): Integer;
    procedure Clear;
    procedure Delete(Index: Integer);
    class procedure Error(const Msg: string; Data: Integer); virtual;
    procedure Exchange(Index1, Index2: Integer);
    function Expand: TVueList;
    function First: TVue;
    function IndexOf(Item: TVue): Integer;
    procedure Insert(Index: Integer; Item: TVue);
    function Last: TVue;
    procedure Move(CurIndex, NewIndex: Integer);
    function Remove(Item: TVue): Integer;
    property Capacity: Integer read FCapacity write SetCapacity;
    property Count: Integer read FCount write SetCount;
    property Items[Index: Integer]: TVue read Get write Put; default;
    property List: PVueArray read FList;

    function IndexOfName(Name: string): integer;
    procedure VerifyFileIntegrity;
    procedure LoadAllVues(RelPathChantier: string);
    procedure DeleteByName(Name, RelPathChantier: string);
    function SaveVueToFile(Vue: TVue; RelPathChantier: string): boolean;
  end;


implementation

uses
  u_common_graph_const,
  u_common_graph_var,
  inifiles,
  SysUtils;


destructor TVueList.Destroy;
begin
  Clear;

  inherited Destroy;
end;

function TVueList.Add(Item: TVue): Integer;
begin
  Result := FCount;
  if Result = FCapacity then Grow;
  FList^[Result] := Item;
  Inc(FCount);
end;

procedure TVueList.Clear;
var
  n: integer;
begin
  for n := 1 to FCount do FList^[n - 1].Free;
  SetCount(0);
  SetCapacity(0);
end;

procedure TVueList.Delete(Index: Integer);
{
begin
 if (Index < 0) or (Index >= FCount) then Error(SListIndexError, Index);
 Dec(FCount);

 if Index < FCount then
 begin
  FList^[Index].Free;
  System.Move(FList^[Index + 1], FList^[Index], (FCount - Index) * SizeOf(Pointer));
 end;
end;
}
begin
  if (Index >= 0) or (Index < FCount) then
    begin
      Dec(FCount);
      FList^[Index].Free;

      if Index < FCount then
        begin
          System.Move(FList^[Index + 1], FList^[Index], (FCount - Index) * SizeOf(Pointer));
        end;
    end;
end;

class procedure TVueList.Error(const Msg: string; Data: Integer);

  function ReturnAddr: Pointer;
  asm
          MOV     EAX,[EBP+4]
  end;

begin
  raise EListError.CreateFmt(Msg, [Data])at ReturnAddr;
end;

procedure TVueList.Exchange(Index1, Index2: Integer);
var
  Item: TVue;
begin
  if (Index1 < 0) or (Index1 >= FCount) then Error(SListIndexError, Index1);
  if (Index2 < 0) or (Index2 >= FCount) then Error(SListIndexError, Index2);
  Item := FList^[Index1];
  FList^[Index1] := FList^[Index2];
  FList^[Index2] := Item;
end;

function TVueList.Expand: TVueList;
begin
  if FCount = FCapacity then Grow;
  Result := Self;
end;

function TVueList.First: TVue;
begin
  if FCount = 0 then result := nil else Result := Get(0);
end;

function TVueList.Get(Index: Integer): TVue;
begin
  if (Index < 0) or (Index >= FCount) then Error(SListIndexError, Index);
  Result := FList^[Index];
end;

procedure TVueList.Grow;
var
  Delta: Integer;
begin
  if FCapacity > 64 then Delta := FCapacity div 4 else
    if FCapacity > 8 then Delta := 16 else
      Delta := 4;
  SetCapacity(FCapacity + Delta);
end;

function TVueList.IndexOf(Item: TVue): Integer;
begin
  Result := 0;
  while (Result < FCount) and (FList^[Result] <> Item) do Inc(Result);
  if Result = FCount then Result := -1;
end;

procedure TVueList.Insert(Index: Integer; Item: TVue);
begin
  if (Index < 0) or (Index > FCount) then Error(SListIndexError, Index);
  if FCount = FCapacity then Grow;
  if Index < FCount then
    System.Move(FList^[Index], FList^[Index + 1],
      (FCount - Index) * SizeOf(Pointer));
  FList^[Index] := Item;
  Inc(FCount);
end;

function TVueList.Last: TVue;
begin
  if FCount = 0 then result := nil else Result := Get(FCount - 1);
end;

procedure TVueList.Move(CurIndex, NewIndex: Integer);
var
  Item: TVue;
begin
  if CurIndex <> NewIndex then
    begin
      if (NewIndex < 0) or (NewIndex >= FCount) then Error(SListIndexError, NewIndex);
      Item := Get(CurIndex);
      Delete(CurIndex);
      Insert(NewIndex, Item);
    end;
end;

procedure TVueList.Put(Index: Integer; Item: TVue);
begin
  if (Index < 0) or (Index >= FCount) then Error(SListIndexError, Index);
  FList^[Index] := Item;
end;

function TVueList.Remove(Item: TVue): Integer;
begin
  Result := IndexOf(Item);
  if Result <> -1 then Delete(Result);
end;


procedure TVueList.SetCapacity(NewCapacity: Integer);
begin
  if (NewCapacity < FCount) or (NewCapacity > MaxListSize) then
    Error(SListCapacityError, NewCapacity);
  if NewCapacity <> FCapacity then
    begin
      ReallocMem(FList, NewCapacity * SizeOf(Pointer));
      FCapacity := NewCapacity;
    end;
end;

procedure TVueList.SetCount(NewCount: Integer);
begin
  if (NewCount < 0) or (NewCount > MaxListSize) then
    Error(SListCountError, NewCount);
  if NewCount > FCapacity then SetCapacity(NewCount);
  if NewCount > FCount then
    FillChar(FList^[FCount], (NewCount - FCount) * SizeOf(Pointer), 0);
  FCount := NewCount;
end;


function TVueList.IndexOfName(Name: string): integer;
var
  n: integer;

begin
  result := -1;
  Name := AnsiUpperCase(Name);
  for n := 0 to FCount - 1 do
    begin
      if AnsiUpperCase(FList^[n].Name) = Name then
        begin
          result := n;
          break;
        end;
    end;
end;


procedure TVueList.VerifyFileIntegrity;
begin

end;


procedure TVueList.LoadAllVues(RelPathChantier: string);
var
  n, k: integer;
  s: string;
  fIni: TIniFile;
  fContent: TStringList;
  aVue: TVue;
begin
  Clear;

  fContent := TStringList.Create;
  try
    fContent.Sorted := true;

    //On rcupre les vues stockes dans le fichier des vues
    try
      fIni := TIniFile.create(_PathChantiers + RelPathChantier + _RELPATH_GESTION_PLANS + _FILE_NAME_CONFIG_VUES);
      try
        k := fIni.ReadInteger('CONTENT', 'NbVues', 0);

        for n := 0 to k - 1 do
          begin
            s := trim(fIni.ReadString('CONTENT', 'Vue' + inttostr(n), ''));
            if s <> '' then fContent.Add(s);
          end;

      finally
        fIni.Free;
      end;
    except
    end;


    //On charge les vues
    for n := 0 to fContent.Count - 1 do
      begin
        aVue := TVue.create;
        if aVue.LoadFromFile(fContent[n], RelPathChantier)
          then Add(aVue)
        else aVue.Free;
      end;

  finally
    fContent.Free;
  end;
end;

procedure TVueList.DeleteByName(Name, RelPathChantier: string);
var
  k, n: integer;
  fIni: TIniFile;
begin
  k := IndexOfName(Name);
  if k <> -1 then
    begin
      //Destruction de la vue
      Delete(k);

      //Mise  jour dans le fichier
      try
        fIni := TIniFile.create(_PathChantiers + RelPathChantier + _RELPATH_GESTION_PLANS + _FILE_NAME_CONFIG_VUES);
        try
          //Description
          try
            fIni.EraseSection(Name + _DESCRIPTION);
          except
          end;

          //Couleurs
          try
            fIni.EraseSection(Name + _COLORS);
          except
          end;

          //Uniquement les pieux visibles
          try
            fIni.EraseSection(Name + _PIEUX_VISIBLE);
          except
          end;


          //La liste des vue disponibles
          try
            fIni.EraseSection('CONTENT');
          except
          end;
          fIni.WriteInteger('CONTENT', 'NbVues', FCount);
          for n := 0 to FCount - 1 do fIni.WriteString('CONTENT', 'Vue' + inttostr(n), FList^[n].Name);


          fIni.UpdateFile;

        finally
          fIni.Free;
        end;
      except
      end;
    end;
end;

function TVueList.SaveVueToFile(Vue: TVue; RelPathChantier: string): boolean;
var
  n: integer;
  fIni: TIniFile;
begin
  result := false;
  if IndexOf(Vue) <> -1 then
    begin
      //On sauvegarde la vue
      if Vue.SaveToFile(RelPathChantier) then
        begin
          //On met  jour la liste des vues
          try
            fIni := TIniFile.create(_PathChantiers + RelPathChantier + _RELPATH_GESTION_PLANS + _FILE_NAME_CONFIG_VUES);
            try
              try
                fIni.EraseSection('CONTENT');
              except
              end;
              fIni.WriteInteger('CONTENT', 'NbVues', FCount);
              for n := 0 to FCount - 1 do fIni.WriteString('CONTENT', 'Vue' + inttostr(n), FList^[n].Name);

              fIni.UpdateFile;

              result := true;
            finally
              fIni.Free;
            end;
          except
          end;
        end;
    end;
end;

end.
