{-----------------------------------------------------------------------}
{                                                                       }
{           Subprogram Name:                                            }
{           Purpose:          Ancestrologie                             }
{           Source Language:  Francais                                  }
{           Auteur:                        }
{           Philippe Cazaux-Moutou                                       }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Description:                                                }
{           Ancestrologie est un Logiciel Libre                         }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Revision History                                            }
{           v#    ,Date       ,Author Name            ,Description      }
{                                                                       }
{-----------------------------------------------------------------------}

unit u_rm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  RPDefine, RPBase, RPCanvas, RPrinter, WinProcs,
  //u_objet_TPieu,
  u_objet_graph_TGraph,
  u_common_graph_type;

type


  Trm = class(TDataModule)
    rp: TReportPrinter;
    procedure rpPrint(Sender: TObject);
    procedure DataModuleCreate(Sender: TObject);
    procedure DataModuleDestroy(Sender: TObject);
  private
    fGraph: TGraph;


    fMarginBetweenPlanAndFooter: integer;
    fFooterHeight: integer;
    fHeaderHeight: integer;
    fMarginBetweenHeaderAndPlan: integer;
    fZoom: integer;
    fDecalX: single;
    fDecalY: single;
    fEchelle: integer;

    cadreHeader: TFloatRect;
    cadrePlan: TFloatRect;
    cadreFooter: TFloatRect;

    xchantier, ychantier: single;

    //les pages
//          fTPXP, fTPYP: TSingleList;
    fDecalagePapierY: extended;
    fDecalagePapierX: extended;

    procedure ApplyZoomAtPoint2(aZoom: integer; xv, yv, xc, yc: single);
    procedure PrepareAllPieu;
    procedure CoordChantier_To_CoordPapier(xc, yc: single; var xv, yv: single);
    //          procedure CoordPapier_To_CoordChantier(xv, yv: single; var xc, yc: single);
//          procedure doPreparePieu(aPieu: TPieu; coef: single);

    procedure doPaintChantier;
    procedure doPaintHeader;
    procedure doPaintFooter;
    function Prepare: boolean;
    //          procedure DrawLine(Ax, Ay, Bx, By: single);



  public
    //Hauteur de l'entte (en mm)
    property HeaderHeight: integer read fHeaderHeight write fHeaderHeight;

    //hauteur de l'espace entre l'entte et le plan
    property MarginBetweenHeaderAndPlan: integer read fMarginBetweenHeaderAndPlan write fMarginBetweenHeaderAndPlan;


    //hauteur de l'espace entre le plan et le bas de page
    property MarginBetweenPlanAndFooter: integer read fMarginBetweenPlanAndFooter write fMarginBetweenPlanAndFooter;

    property DecalagePapierX: extended read fDecalagePapierX write fDecalagePapierX;
    property DecalagePapierY: extended read fDecalagePapierY write fDecalagePapierY;

    //hauteur du bas de page (en mm)
    property FooterHeight: integer read fFooterHeight write fFooterHeight;



    procedure Print(aGraph: TGraph);
    procedure PrintPage(aGraph: TGraph; px, py: integer);
    procedure PrinterSetup;
  end;

var
  rm: Trm;

implementation

uses u_common_graph_const, u_common_graph_var, u_common_graph_func,
  u_common_var, u_objet_TSingleList;

{$R *.DFM}


procedure Trm.rpPrint(Sender: TObject);
begin
  rp.MarginLeft := _Context.MargeLeftPrinter;
  rp.MarginTop := _Context.MargeTopPrinter;
  rp.MarginRight := _Context.MargeRightPrinter;
  rp.MarginBottom := _Context.MargeBottomPrinter;


  if Prepare then
    begin
      fGraph.PreparePrint;
      fGraph.Print(rp);

      doPaintHeader;
      doPaintChantier;
      doPaintFooter;
    end;
end;


function Trm.Prepare: boolean;
begin
  result := true;

  cadreHeader.left := rp.MarginLeft;
  cadreHeader.top := rp.MarginTop;
  cadreHeader.right := (rp.PageWidth) - rp.MarginRight;
  cadreHeader.Bottom := rp.MarginTop + fHeaderHeight;

  //On calcul la taille du rectangle disponible, pour dessiner le plan
  cadrePlan.left := rp.MarginLeft;
  cadrePlan.top := rp.MarginTop + fHeaderHeight + fMarginBetweenHeaderAndPlan;
  cadrePlan.right := (rp.PageWidth) - rp.MarginRight;
  cadrePlan.Bottom := (rp.PageHeight) - rp.MarginBottom - fMarginBetweenPlanAndFooter - fFooterHeight;

  cadreFooter.Bottom := (rp.PageHeight) - rp.MarginBottom;
  cadreFooter.top := cadreFooter.Bottom - fFooterHeight;
  cadreFooter.left := rp.MarginLeft;
  cadreFooter.right := (rp.PageWidth) - rp.MarginRight;


  if (cadrePlan.right > cadrePlan.left) and (cadrePlan.bottom > cadrePlan.top) then
    begin
      //1m sur le chantier, correspond  1000 mm sur le papier, en zoom 1:1 (100%)
      fEchelle := 1000;

      _Viewer.CoordViewer_To_CoordChantier(_Viewer.PaintBox.width div 2, _Viewer.PaintBox.height div 2, xchantier, ychantier);

      ApplyZoomAtPoint2(
        fZoom,
        cadrePlan.left + ((cadrePlan.right - cadrePlan.left) / 2),
        cadrePlan.top + ((cadrePlan.bottom - cadrePlan.top) / 2),
        xchantier,

        ychantier);
      result := true;
    end;
end;


//but : on applique le zoom,
//puis on fait correspondre (xc,yc) coord chantier, sur le point (xv,yv) du papier

procedure Trm.ApplyZoomAtPoint2(aZoom: integer; xv, yv, xc, yc: single);
var
  NewX, NewY: single;
begin
  //on vrifie que le Zoom est correct
  if aZoom < _ZOOM_MIN then aZoom := _ZOOM_MIN;
  if aZoom > _ZOOM_MAX then aZoom := _ZOOM_MAX;

  fZoom := aZoom;

  //On recalcul tout, avec le nouveau Zoom
  PrepareAllPieu;

  CoordChantier_To_CoordPapier(xc, yc, NewX, NewY);

  fDecalX := (xv - NewX);
  fDecalY := (yv - NewY);
end;


procedure Trm.PrepareAllPieu;
//var
//     n: integer;
//     e: single;
begin
  {
       //Calcul des coord de tous les pieux, dans le Viewer
       e := fEchelle / fZoom;

       for n := 0 to _Chantier.PieuList.Count - 1 do
       begin
            doPreparePieu(_Chantier.PieuList[n], e);
       end;
       }
end;

{
procedure Trm.doPreparePieu(aPieu: TPieu; coef: single);
begin

     if aPieu.enabled then
     begin
          if (aPieu.KnowPositionInChantier) and (aPieu.Visible) then
          begin

               //Coordonnes des 4 points du pieu
               aPieu.RectP.P1.x := (coef * aPieu.RectC.P1.x);
               aPieu.RectP.P2.x := (coef * aPieu.RectC.P2.x);
               aPieu.RectP.P3.x := (coef * aPieu.RectC.P3.x);
               aPieu.RectP.P4.x := (coef * aPieu.RectC.P4.x);

               aPieu.RectP.P1.y := -(coef * aPieu.RectC.P1.y);
               aPieu.RectP.P2.y := -(coef * aPieu.RectC.P2.y);
               aPieu.RectP.P3.y := -(coef * aPieu.RectC.P3.y);
               aPieu.RectP.P4.y := -(coef * aPieu.RectC.P4.y);


               //Coordonnes du rectangle d'encadrement
               aPieu.REncadreP.Left := (coef * aPieu.REncadreC.Left);
               aPieu.REncadreP.Right := (coef * aPieu.REncadreC.Right);
               aPieu.REncadreP.Top := -(coef * aPieu.REncadreC.Top);
               aPieu.REncadreP.Bottom := -(coef * aPieu.REncadreC.Bottom);


               //Coordonnes des 4 points du pieu,  CteNGF
               aPieu.RectPAtCoteNGF.P1.x := (coef * aPieu.RectCAtCoteNGF.P1.x);
               aPieu.RectPAtCoteNGF.P2.x := (coef * aPieu.RectCAtCoteNGF.P2.x);
               aPieu.RectPAtCoteNGF.P3.x := (coef * aPieu.RectCAtCoteNGF.P3.x);
               aPieu.RectPAtCoteNGF.P4.x := (coef * aPieu.RectCAtCoteNGF.P4.x);

               aPieu.RectPAtCoteNGF.P1.y := -(coef * aPieu.RectCAtCoteNGF.P1.y);
               aPieu.RectPAtCoteNGF.P2.y := -(coef * aPieu.RectCAtCoteNGF.P2.y);
               aPieu.RectPAtCoteNGF.P3.y := -(coef * aPieu.RectCAtCoteNGF.P3.y);
               aPieu.RectPAtCoteNGF.P4.y := -(coef * aPieu.RectCAtCoteNGF.P4.y);
          end;
     end;

end;
}


procedure Trm.CoordChantier_To_CoordPapier(xc, yc: single; var xv, yv: single);
var
  e: single;
begin
  e := fEchelle / fZoom;
  //     xv := trunc(xc * e) + fDecalX;
      //     yv := -trunc(yc * e) + fDecalY;
  xv := (xc * e);
  yv := -(yc * e);
end;

{
procedure Trm.CoordPapier_To_CoordChantier(xv, yv: single; var xc, yc: single);
var
e: single;
begin
e := fEchelle * fZoom / 100;
xc := (xv - fDecalX) / e;
yc := -(yv - fDecalY) / e;
end;
}

procedure Trm.Print(aGraph: TGraph);
var
  nx, ny: integer;
begin
  if _Viewer <> nil then
    begin
      fGraph := aGraph;
      //     fZoom := _Viewer.Zoom;
      //prparation

      rp.MarginLeft := _Context.MargeLeftPrinter;
      rp.MarginRight := _Context.MargeRightPrinter;
      rp.MarginTop := _Context.MargeTopPrinter;
      rp.MarginBottom := _Context.MargeBottomPrinter;

      //les pages
  //          fTPXP.Assign(_Viewer.TPXC); //on est dj en mm
  //          fTPYP.Assign(_Viewer.TPYC);
  //          px := fTPXC.Count - 1; if px < 0 then px := 0;
  //          py := fTPYC.Count - 1; if py < 0 then py := 0;


      for ny := 0 to _Viewer.TPYC.count - 2 do
        begin
          for nx := 0 to _Viewer.TPXC.count - 2 do
            begin
              //le dcalage
              fDecalagePapierX := -_Viewer.TPXC[nx];
              fDecalagePapierY := -_Viewer.TPYC[ny];

              rp.Execute;

            end;
        end;

      //    if (px > 0) and (py > 0) then
        //  begin
  //             fGraph.PreparePrint;


    //           rp.execute;
       //   end;
    end;
end;

procedure Trm.PrinterSetup;
begin
  if rp.ShowPrinterSetupDialog then
    begin
      //changement de papier
    end;
end;



procedure Trm.doPaintChantier;
{
var
     n: integer;
     w, h, t, l: single;
     aPieu: TPieu;
     ok, okName: boolean;
     R: TFloatRect;
     xc, yc: single;
     xp, yp: single;
     cont: boolean;
     k: integer;
     e: single;
     }
begin
  (*
       //Quadrillage
       if _Context.PrintCadrillage then
       begin
            if _Context.CadrillageEveryMM > 0 then
            begin
                 //type de trait
                 rp.SetPen(clBlack, psDot, 1, pmCopy);

                 //On calcul le rectangle d'encadrement du chantier

                         //Rectangle de dlimitation de tout le chantier, en m
                 ok := false;
                 R.left := Maxint;
                 R.Top := -Maxint;
                 R.Right := -Maxint;
                 R.Bottom := Maxint;

                 for n := 0 to _Chantier.PieuList.Count - 1 do
                 begin
                      aPieu := _Chantier.PieuList[n];
                      if (aPieu.enabled) and (aPieu.Visible) and (aPieu.KnowPositionInChantier) then
                      begin
                           if R.Left > aPieu.REncadreC.Left then R.Left := aPieu.REncadreC.Left;
                           if R.Right < aPieu.REncadreC.Right then R.Right := aPieu.REncadreC.Right;
                           if R.Top < aPieu.REncadreC.Top then R.Top := aPieu.REncadreC.Top;
                           if R.Bottom > aPieu.REncadreC.Bottom then R.Bottom := aPieu.REncadreC.Bottom;
                           ok := true;
                      end;
                 end;

                 if Ok then
                 begin
                      e := fEchelle / fZoom;

                      //vertical, en positif
                      cont := true;
                      k := 0;
                      repeat
                           xc := k * (_Context.CadrillageEveryMM / 1000);
                           xp := (e * xc);
                           xp := xp + fDecalX;

                           //	CoordChantier_To_CoordPapier(xc, 0, xp, yp);

                            //est-ce dans le cadre ?
                           if (xp >= cadrePlan.left) and (xp <= cadrePlan.right) then
                           begin
                                rp.MoveTo(xp, cadrePlan.top);
                                rp.LineTo(xp, cadrePlan.bottom);
                           end;

                           k := k + 1;
                           if (xp > cadrePlan.right) then cont := false;

                      until cont = false;




                      //vertical, en ngatif
                      cont := true;
                      k := 1;
                      repeat
                           xc := -(k * (_Context.CadrillageEveryMM / 1000));
                           xp := (e * xc);
                           xp := xp + fDecalX;

                           //est-ce dans le cadre ?
                           if (xp >= cadrePlan.left) and (xp <= cadrePlan.right) then
                           begin
                                rp.MoveTo(xp, cadrePlan.top);
                                rp.LineTo(xp, cadrePlan.bottom);
                           end;

                           k := k + 1;
                           if (xp < cadrePlan.left) then cont := false;

                      until cont = false;








                      //horizontal, en positif
                      cont := true;
                      k := 0;
                      repeat
                           yc := k * (_Context.CadrillageEveryMM / 1000);
                           yp := -(e * yc);
                           yp := yp + fDecalY;

                           //est-ce dans le cadre ?
                           if (yp >= cadrePlan.top) and (yp <= cadrePlan.bottom) then
                           begin
                                rp.MoveTo(cadrePlan.left, yp);
                                rp.LineTo(cadrePlan.right, yp);
                           end;

                           k := k + 1;
                           if (yp < cadrePlan.top) then cont := false;

                      until cont = false;



                      //horizontal, en ngatif
                      cont := true;
                      k := 1;
                      repeat
                           yc := -(k * (_Context.CadrillageEveryMM / 1000));
                           yp := -(e * yc);
                           yp := yp + fDecalY;


                           //est-ce dans le cadre ?
                           if (yp >= cadrePlan.top) and (yp <= cadrePlan.bottom) then
                           begin
                                rp.MoveTo(cadrePlan.left, yp + fDecalY);
                                rp.LineTo(cadrePlan.right, yp + fDecalY);
                           end;

                           k := k + 1;
                           if (yp > cadrePlan.bottom) then cont := false;

                      until cont = false;



                      {
                      CoordPapier_To_CoordChantier(cadrePlan.left - 50, cadrePlan.top - 50, P1x, P1y);
                                                  CoordPapier_To_CoordChantier(cadrePlan.right + 50, cadrePlan.bottom + 50, P2x, P2y);

                                                  //horizontal
                      w := P1x * 1000 / _Context.CadrillageEveryMM;
              n := trunc(w) - 1000;

                                                  ok := true;
                                                  repeat

                                                   CoordChantier_To_CoordPapier(n / 1000, 0, t, l);
                                       //if (t >= cadrePlan.left) and (t <= cadrePlan.right) then
                                       if (t <= cadrePlan.right) then
                                       begin
                                        rp.MoveTo(t, cadrePlan.top);
                                        rp.LineTo(t, cadrePlan.bottom);

                                       end
                                       else
                                        ok := false;

                                       n := n + _Context.CadrillageEveryMM;

                                      until ok = false

                                         }
                                      //vertical
                 end;
            end;
       end;




       //les pieux
       for n := 0 to _Chantier.PieuList.Count - 1 do
       begin
            aPieu := _Chantier.PieuList[n];
            if (aPieu.Enabled) and (aPieu.Visible) and (aPieu.KnowPositionInChantier) then
            begin
                 //trace du pieux,  CteDef
                 if _Viewer.ShowTrace then
                 begin
                      rp.SetPen(_Context.ColorPieuxTracePrinter, psSolid, _Context.ColorPieuxTracePrinterHeight, pmCopy);

                      DrawLine(aPieu.RectPAtCoteNGF.P1.x + fDecalX, aPieu.RectPAtCoteNGF.P1.y + fDecalY,
                           aPieu.RectPAtCoteNGF.P2.x + fDecalX, aPieu.RectPAtCoteNGF.P2.y + fDecalY);

                      DrawLine(aPieu.RectPAtCoteNGF.P2.x + fDecalX, aPieu.RectPAtCoteNGF.P2.y + fDecalY,
                           aPieu.RectPAtCoteNGF.P3.x + fDecalX, aPieu.RectPAtCoteNGF.P3.y + fDecalY);

                      DrawLine(aPieu.RectPAtCoteNGF.P3.x + fDecalX, aPieu.RectPAtCoteNGF.P3.y + fDecalY,
                           aPieu.RectPAtCoteNGF.P4.x + fDecalX, aPieu.RectPAtCoteNGF.P4.y + fDecalY);

                      DrawLine(aPieu.RectPAtCoteNGF.P4.x + fDecalX, aPieu.RectPAtCoteNGF.P4.y + fDecalY,
                           aPieu.RectPAtCoteNGF.P1.x + fDecalX, aPieu.RectPAtCoteNGF.P1.y + fDecalY);
                 end;


                 //Le pieu
                 case aPieu.WhereIsDepth of
                      icAbovePieu, icUnderPieu:
                           begin
                                rp.SetPen(_Context.ColorPieuxOutOfDepth, psSolid, _Context.ColorPieuxOutOfDepthPrinterHeight, pmCopy);
                                ok := true;
                           end;
                      icInPieu:
                           begin
                                rp.SetPen(_Context.ColorPieux, psSolid, _Context.ColorPieuxPrinterHeight, pmCopy);
                                ok := true;
                           end;
                 else
                      ok := false;
                 end;

                 if Ok then
                 begin
                      //Dessin du pieu
                      DrawLine(aPieu.RectP.P1.x + fDecalX, aPieu.RectP.P1.y + fDecalY,
                           aPieu.RectP.P2.x + fDecalX, aPieu.RectP.P2.y + fDecalY);

                      DrawLine(aPieu.RectP.P2.x + fDecalX, aPieu.RectP.P2.y + fDecalY,
                           aPieu.RectP.P3.x + fDecalX, aPieu.RectP.P3.y + fDecalY);

                      DrawLine(aPieu.RectP.P3.x + fDecalX, aPieu.RectP.P3.y + fDecalY,
                           aPieu.RectP.P4.x + fDecalX, aPieu.RectP.P4.y + fDecalY);

                      DrawLine(aPieu.RectP.P4.x + fDecalX, aPieu.RectP.P4.y + fDecalY,
                           aPieu.RectP.P1.x + fDecalX, aPieu.RectP.P1.y + fDecalY);

                      {
                     rp.MoveTo(aPieu.RectP.P1.x + fDecalX, aPieu.RectP.P1.y + fDecalY);
                     rp.LineTo(aPieu.RectP.P2.x + fDecalX, aPieu.RectP.P2.y + fDecalY);
                     rp.LineTo(aPieu.RectP.P3.x + fDecalX, aPieu.RectP.P3.y + fDecalY);
                     rp.LineTo(aPieu.RectP.P4.x + fDecalX, aPieu.RectP.P4.y + fDecalY);
                     rp.LineTo(aPieu.RectP.P1.x + fDecalX, aPieu.RectP.P1.y + fDecalY);
                                           }

                     //Nom du pieu
                      if _Viewer.ShowNamePieuMode <> snNewerShow then
                      begin
                           if aPieu.Name <> '' then
                           begin
                                rp.SetFont(_Context.FontPrinter.Name, _Context.FontPrinter.Size);
                                rp.Bold := (fsBold in _Context.FontPrinter.Style);
                                rp.Underline := (fsUnderLine in _Context.FontPrinter.Style);
                                rp.Italic := (fsItalic in _Context.FontPrinter.Style);
                                rp.StrikeOut := (fsStrikeOut in _Context.FontPrinter.Style);
                                rp.FontColor := _Context.FontPrinter.Color;


                                //	fPaintBox.Canvas.Brush.Style := bsClear;
                            //	fPaintBox.Canvas.Font.Assign(fFont);

                            //On enlve le dcalage, pour obtenir le rectangle d'encadrement du pieu
                                R.left := aPieu.REncadreP.left + fDecalX;
                                R.right := aPieu.REncadreP.right + fDecalX;
                                R.top := aPieu.REncadreP.top + fDecalY;
                                R.bottom := aPieu.REncadreP.bottom + fDecalY;

                                w := rp.TextWidth(aPieu.Name);
                                //  h := rp.YD2U(abs(_Viewer.Font.Height));
        //h := (abs(_Viewer.Font.Height));
        //	h:=rp.LPI/25.4;
                                h := 2;

                                if _Viewer.ShowNamePieuMode = snAlwaysShow then okName := true
                                else
                                begin
                                     //est-ce que le nom du pieu rentre dans le rectangle de dlimitation ?
                                     okName := (w <= abs(R.right - R.Left + 1)) and (h <= abs(R.bottom - R.top + 1));
                                end;

                                if okName then
                                begin
                                     //							t := (abs(R.top - R.bottom) - h) div 2;
            // t := (abs(R.top - R.bottom) - h) / 2;
                                     t := (abs(R.top - R.bottom)) / 2;
                                     l := (abs(R.left - R.right) - w) / 2;
                                     R.top := r.top + t + 1;
                                     R.left := R.left + l;

                                     //il faut que le texte soit dans le cadre

                                     if (R.left >= cadrePlan.Left) and (R.left + w <= cadrePlan.Right) and (R.top <= cadrePlan.bottom) and (R.top - h >= cadrePlan.top)
                                          then rp.PrintXY(R.left, R.top, aPieu.Name);
                                     //							DrawText(fPaintBox.Canvas.Handle, PChar(aPieu.Name), length(aPieu.Name), R, DT_CENTER or DT_VCENTER);
                                end;
                           end;
                      end;
                 end;
            end;
       end;



       //encadrement
       rp.SetPen(clBlack, psSolid, 1, pmCopy);

       rp.MoveTo(cadrePlan.left, cadrePlan.Top);
       rp.LineTo(cadrePlan.right, cadrePlan.Top);
       rp.LineTo(cadrePlan.right, cadrePlan.bottom);
       rp.LineTo(cadrePlan.left, cadrePlan.bottom);
       rp.LineTo(cadrePlan.left, cadrePlan.Top);
       *)
end;






procedure Trm.doPaintHeader;
begin
  {
       rp.SetPen(clBlack, psSolid, 1, pmCopy);

       rp.SetFont('arial', 12);
       rp.Bold := true;
       rp.Underline := false;
       rp.Italic := false;
       rp.StrikeOut := false;
       rp.FontColor := clBlack;


       rp.PrintXY(cadreHeader.left + 5, cadreHeader.Top + 10, 'Site : ' + _Chantier.Site);
       rp.PrintXY(cadreHeader.left + 5, cadreHeader.Top + 20, 'Cte : ' + SFloatN(_Chantier.Cote, 2) + ' m');
       rp.PrintXY(cadreHeader.left + 5, cadreHeader.Top + 30, 'Echelle : 1/' + inttostr(_Viewer.Zoom));




       rp.MoveTo(cadreHeader.left, cadreHeader.Top);
       rp.LineTo(cadreHeader.right, cadreHeader.Top);
       rp.LineTo(cadreHeader.right, cadreHeader.bottom);
       rp.LineTo(cadreHeader.left, cadreHeader.bottom);
       rp.LineTo(cadreHeader.left, cadreHeader.Top);
       }
end;


procedure Trm.doPaintFooter;
begin
  {
       rp.SetPen(clBlack, psSolid, 1, pmCopy);

       rp.MoveTo(cadreFooter.left, cadreFooter.Top);
       rp.LineTo(cadreFooter.right, cadreFooter.Top);
       rp.LineTo(cadreFooter.right, cadreFooter.bottom);
       rp.LineTo(cadreFooter.left, cadreFooter.bottom);
       rp.LineTo(cadreFooter.left, cadreFooter.Top);
       }
end;


procedure Trm.DataModuleCreate(Sender: TObject);
begin
  fGraph := nil;


  fHeaderHeight := 50;
  fMarginBetweenHeaderAndPlan := 5;
  fMarginBetweenPlanAndFooter := 5;
  fFooterHeight := 20;

  //     fTPXP := TSingleList.create;
  //     fTPYP := TSingleList.create;
end;

{
procedure Trm.DrawLine(Ax, Ay, Bx, By: single);
var
     e, a, b, x, y: single;
     i1x, i1y, i2x, i2y: single;
     Findi1, Findi2: boolean;

     procedure PrintTheLine(fAx, fAy, fBx, fBy: single);
     begin
          rp.MoveTo(fAx, fAy);
          rp.LineTo(fBx, fBy);
     end;

begin
     try
          //vertical ?
          if Ax = Bx then
               begin
                    if Ay > By then
                         begin
                              e := Ay; Ay := By; By := e;
                         end;

                    if (Ax >= CadrePlan.Left) and (Ax <= CadrePlan.Right) then
                         begin
                              if (By < CadrePlan.Top) then begin end
                              else
                                   if (Ay > CadrePlan.Bottom) then begin end
                                   else
                                        if (Ay >= CadrePlan.Top) and (By <= CadrePlan.Bottom) then PrintTheLine(Ax, Ay, Bx, By)
                                        else
                                             if (Ay <= CadrePlan.Top) and (By >= CadrePlan.Bottom) then PrintTheLine(Ax, CadrePlan.Top, Bx, CadrePlan.Bottom)
                                             else
                                                  if (Ay <= CadrePlan.Top) and (By >= CadrePlan.Top) and (By <= CadrePlan.Bottom) then PrintTheLine(Ax, CadrePlan.Top, Bx, By)
                                                  else
                                                       if (By >= CadrePlan.Bottom) and (Ay >= CadrePlan.Top) and (Ay <= CadrePlan.Bottom) then PrintTheLine(Ax, Ay, Bx, CadrePlan.Bottom);
                         end;
               end
          else
               //Horizontal ?
               if Ay = By then
                    begin
                         if Ax > Bx then
                              begin
                                   e := Ax; Ax := Bx; Bx := e;
                              end;

                         if (Ay >= CadrePlan.Top) and (Ay <= CadrePlan.Bottom) then
                              begin
                                   if Bx < CadrePlan.Left then begin end //rien
                                   else
                                        if Ax > CadrePlan.Right then begin end //rien
                                        else
                                             if (Ax >= CadrePlan.Left) and (Bx <= CadrePlan.Right) then PrintTheLine(Ax, Ay, Bx, By)
                                             else
                                                  if (Ax <= CadrePlan.Left) and (Bx >= CadrePlan.Right) then PrintTheLine(CadrePlan.left, Ay, CadrePlan.right, By)
                                                  else
                                                       if (Ax < CadrePlan.Left) and (Bx <= CadrePlan.Right) and (Bx >= CadrePlan.Left) then PrintTheLine(CadrePlan.left, Ay, Bx, By)
                                                       else
                                                            if (Ax >= CadrePlan.Left) and (Ax <= CadrePlan.Right) and (Bx >= CadrePlan.Right) then PrintTheLine(Ax, Ay, CadrePlan.Right, By);
                              end;
                    end
               else
                    //Diagonal
                    begin
                         if Ax > Bx then
                              begin
                                   e := Ax; Ax := Bx; Bx := e;
                                   e := Ay; Ay := By; By := e;
                              end;

                         if (Ax >= CadrePlan.Left) and (Ax <= CadrePlan.Right) and (Ay >= CadrePlan.Top) and (Ay <= CadrePlan.Bottom)
                              and (Bx >= CadrePlan.Left) and (Bx <= CadrePlan.Right) and (By >= CadrePlan.Top) and (By <= CadrePlan.Bottom) then
                              begin
                                   PrintTheLine(Ax, Ay, Bx, By);
                              end
                         else
                              begin
                                   Findi1 := false; i1x := 0; i1y := 0;
                                   Findi2 := false; i2x := 0; i2y := 0;
                                   a := (By - Ay) / (Bx - Ax);
                                   b := Ay - (a * Ax);

                                   //intersection avec P1P2
                                   y := CadrePlan.Top;
                                   x := (y - b) / a;

                                   if (x >= CadrePlan.Left) and (x <= CadrePlan.Right) then
                                        begin
                                             if Findi1 = false then begin Findi1 := true; i1x := x; i1y := y; end
                                             else
                                                  if Findi2 = false then begin Findi2 := true; i2x := x; i2y := y; end;
                                        end;

                                   //intersection avec P3P4
                                   y := CadrePlan.Bottom;
                                   x := (y - b) / a;

                                   if (x >= CadrePlan.Left) and (x <= CadrePlan.Right) then
                                        begin
                                             if Findi1 = false then begin Findi1 := true; i1x := x; i1y := y; end
                                             else
                                                  if Findi2 = false then begin Findi2 := true; i2x := x; i2y := y; end;
                                        end;


                                   //intersection avec P2P3
                                   x := CadrePlan.Right;
                                   y := a * x + b;

                                   if (y >= CadrePlan.Top) and (y <= CadrePlan.Bottom) then
                                        begin
                                             if Findi1 = false then begin Findi1 := true; i1x := x; i1y := y; end
                                             else
                                                  if Findi2 = false then begin Findi2 := true; i2x := x; i2y := y; end;
                                        end;


                                   //intersection avec P4P1
                                   x := CadrePlan.Left;
                                   y := a * x + b;

                                   if (y >= CadrePlan.Top) and (y <= CadrePlan.Bottom) then
                                        begin
                                             if Findi1 = false then begin Findi1 := true; i1x := x; i1y := y; end
                                             else
                                                  if Findi2 = false then begin Findi2 := true; i2x := x; i2y := y; end;
                                        end;

                                   //A-t'on trouv les 2 intersections ?
                                   if (Findi1) and (Findi2) then
                                        begin
                                             //Si A est dans le cadre :
                                             if (Ax >= CadrePlan.Left) and (Ax <= CadrePlan.Right) and (Ay >= CadrePlan.Top) and (Ay <= CadrePlan.Bottom) then
                                                  begin
                                                       if i1x >= Ax
                                                            then PrintTheLine(Ax, Ay, i1x, i1y)
                                                       else PrintTheLine(Ax, Ay, i2x, i2y);
                                                  end
                                             else
                                                  if (Bx >= CadrePlan.Left) and (Bx <= CadrePlan.Right) and (By >= CadrePlan.Top) and (By <= CadrePlan.Bottom) then
                                                       begin
                                                            if i1x <= Bx
                                                                 then PrintTheLine(Bx, By, i1x, i1y)
                                                            else PrintTheLine(Bx, By, i2x, i2y);
                                                       end
                                                  else
                                                       begin
                                                            //on classe i1 et i2
                                                            if i1x > i2x then
                                                                 begin
                                                                      e := i1x; i1x := i2x; i2x := e;
                                                                      e := i1y; i1y := i2y; i2y := e;
                                                                 end;

                                                            if (Ax <= i1x) and (i1x <= i2x) and (i2x <= Bx) then
                                                                 begin
                                                                      PrintTheLine(i1x, i1y, i2x, i2y);
                                                                 end;
                                                       end;
                                        end;
                              end;
                    end;
     except
     end;
end;

}

procedure Trm.DataModuleDestroy(Sender: TObject);
begin
  //     fTPXP.free;
  //     fTPYP.free;
end;

(*
function Trm.TextWidth(Text: string): double;

var
TextExtent: TSize;

begin { TextWidth }
//     CheckPrinting;
if Pos(RPTFPrefix, Text) = 0 then
begin
if Text = '' then
begin
    TextExtent.cX := 1;
end
else
begin
    SelectRPFont;
    WinProcs.GetTextExtentPoint(Canvas.Handle, @Text[1], Length(Text), TextExtent);
    SelectCanvasFont;
    if TextExtent.cX = 0 then
    begin
         TextExtent.cX := 1;
    end; { if }
end; { else }
end
else
begin
TextExtent.cX := RPTFTextWidth(Self, Text);
end; { else }
Result := XI2U(TextExtent.cX * FScaleX / (FXDPI * 100.0));
end; { TextWidth }

*)


procedure Trm.PrintPage(aGraph: TGraph; px, py: integer);
begin
  if _Viewer <> nil then
    begin
      fGraph := aGraph;
      //prparation

      rp.MarginLeft := _Context.MargeLeftPrinter;
      rp.MarginRight := _Context.MargeRightPrinter;
      rp.MarginTop := _Context.MargeTopPrinter;
      rp.MarginBottom := _Context.MargeBottomPrinter;

      //le dcalage
      fDecalagePapierX := -_Viewer.TPXC[px];
      fDecalagePapierY := -_Viewer.TPYC[py];

      rp.Execute;
    end;
end;




end.
