{-----------------------------------------------------------------------}
{                                                                       }
{           Subprogram Name:                                            }
{           Purpose:          Ancestrologie                             }
{           Source Language:  Francais                                  }
{           Auteur:                        }
{           Philippe Cazaux-Moutou                                       }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Description:                                                }
{           Ancestrologie est un Logiciel Libre                         }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Revision History                                            }
{           v#    ,Date       ,Author Name            ,Description      }
{                                                                       }
{-----------------------------------------------------------------------}

unit u_objet_TCoherenceDate;

interface

uses
  Classes, contnrs, Forms;

type
  TInfoDate = class
  private
    fiDate: TDateTime;
    fKnowDate: boolean;
    fInfos: string;
    fIDOwner: integer;
    fID: integer;
    fLongEventName: string;

  public
    property KnowDate: boolean read fKnowDate write fKnowDate;
    property iDate: TDateTime read fiDate;
    property Infos: string read fInfos write fInfos;
    property ID: integer read fID write fID;
    property IDOwner: integer read fIDOwner write fIDOwner;
    property LongEventName: string read fLongEventName write fLongEventName;

    constructor Create;
    procedure Reset;
    procedure SetDateWriten(s: string);
    procedure SetDate(d: TDateTime);
  end;


const
  _MAX_PB = 7;

type
  TCoherenceDate = class

  private
    fFormIndividu: TForm;
    fFormCoherence: TForm;
    function MinDate(List: TObjectList; var aDate: TDateTime): boolean;
    function MaxDate(List: TObjectList; var aDate: TDateTime): boolean;
    function FindInfoDate(List: TObjectList; aID: integer): TInfoDate;
  public

    {
         naissance
         dcs
         vnements ind autres 1..n
         vnements union 1..n
         naissance enfants 1..n

         //Tests
         1-n avant dcs
         2-Esprence de vie
         3-Age minimum pour se marier
         4-Age maximum pour se marier
         5-Age minimum pour avoir un enfant
         6-Age maximum pour avoir un enfant
         7-Ecart d'age maximum entre 2 poux

         -> pour tous les vnements ind et fam, sauf naissance/mort/mariage

            si on connait la date de naissance, on regarde si d>=date_naissance
            si on connait la date du dcs, on regarde si d<=date_dces

         -> pour tous les enfants

            si on connait la date du dcs, on regarde si date_naissance<=(d+9 mois)
        }

    Sexe: integer;

    Naissance: TInfoDate;
    Deces: TInfoDate;
    AutresEventInd: TObjectList;
    Mariages: TObjectList;
    AutresEventUnion: TObjectList;
    NaissancesEnfants: TObjectList;

    ListEventNotTestedWithDeces: TStringList;


    Pb: array[1.._MAX_PB] of boolean;
    MorePbEventIndi: TStringList;

    property FormCoherence: TForm read fFormCoherence write fFormCoherence;
    property FormIndividu: TForm read fFormIndividu write fFormIndividu;

    constructor Create;
    destructor Destroy; override;
    procedure Reset;
    procedure Test;

    procedure AddInfoDateWriten(dWriten, Infos: string; ID, IDOwner: integer; List: TObjectList; LongEventName: string);
    procedure AddInfoDate(d: TDateTime; Infos: string; List: TObjectList);
  end;



implementation

uses
  SysUtils,
  u_common_var,
  u_form_individu,
  u_form_coherence,
  u_objet_TGedcomDate;


var
  GedDateCoherence: TGedcomDate;


  { TInfoDate }

constructor TInfoDate.Create;
begin
  Reset;
end;

procedure TInfoDate.Reset;
begin
  fKnowDate := false;
  fInfos := '';
  fIDOwner := -1;
  fID := -1;
  fLongEventName := '';
end;

procedure TInfoDate.SetDate(d: TDateTime);
begin
  fiDate := d;
  fKnowDate := true;
end;

procedure TInfoDate.SetDateWriten(s: string);
begin
  fKnowDate := false;
  if GedDateCoherence.DecodeHumanDate(s) then
    begin
      //Soit on connait la date exacte
      if GedDateCoherence.KnowDate then
        begin
          fiDate := GedDateCoherence.GDate.DateTime1;
          fKnowDate := true;
        end
      else
        //Soit on ne connait que le mois et l'anne
        if (GedDateCoherence.GDate.UseDate1) and
          (GedDateCoherence.GDate.UseDate2 = false) and
          (GedDateCoherence.GDate.Year1 <> -1) and
          (GedDateCoherence.GDate.Month1 <> -1) and
          (GedDateCoherence.GDate.Key1 = '') then
          begin
            fiDate := EncodeDate(GedDateCoherence.GDate.Year1, GedDateCoherence.GDate.Month1, 1);
            fKnowDate := true;
          end
        else
          //Soit on ne connait que l'anne
          if GedDateCoherence.KnowYear then
            begin
              fiDate := EncodeDate(GedDateCoherence.GDate.Year1, 1, 1);
              fKnowDate := true;
            end;
    end;
end;




{ TCoherenceDate }

procedure TCoherenceDate.AddInfoDate(d: TDateTime; Infos: string; List: TObjectList);
var
  aInfoDate: TInfoDate;
begin
  aInfoDate := TInfoDate.create;
  aInfoDate.Infos := Infos;
  aInfoDate.SetDate(d);
  List.Add(aInfoDate);
end;

procedure TCoherenceDate.AddInfoDateWriten(dWriten, Infos: string; ID, IDOwner: integer; List: TObjectList; LongEventName: string);
var
  Find: boolean;
  aInfoDate: TInfoDate;
begin
  aInfoDate := FindInfoDate(List, ID);
  Find := aInfoDate <> nil;
  if aInfoDate = nil then aInfoDate := TInfoDate.create;
  aInfoDate.ID := ID;
  aInfoDate.IDOwner := IDOwner;
  aInfoDate.Infos := Infos;
  aInfoDate.LongEventName := LongEventName;
  aInfoDate.SetDateWriten(dWriten);
  if not Find then List.Add(aInfoDate);
end;


constructor TCoherenceDate.Create;
begin
  GedDateCoherence.AssignTokens;

  fFormCoherence := nil;
  fFormIndividu := nil;

  MorePbEventIndi := TStringList.create;
  Naissance := TInfoDate.create;
  Deces := TInfoDate.create;
  AutresEventInd := TObjectList.create(true);
  Mariages := TObjectList.create(true);
  AutresEventUnion := TObjectList.create(true);
  NaissancesEnfants := TObjectList.create(true);

  ListEventNotTestedWithDeces := TStringList.create;
  ListEventNotTestedWithDeces.Add('INHU');
  ListEventNotTestedWithDeces.Add('TITL');
  ListEventNotTestedWithDeces.Add('ANUL');
  ListEventNotTestedWithDeces.Add('BENE');
  ListEventNotTestedWithDeces.Add('BLES');
  ListEventNotTestedWithDeces.Add('BURI');
  ListEventNotTestedWithDeces.Add('CREM');
  ListEventNotTestedWithDeces.Add('DECO');
  ListEventNotTestedWithDeces.Add('DIV');

  {
//BIRT
CHR
//DEAT
BURI
CREM
ADOP
BAPM
BARM
BASM
BLES
CHRA
CONF
FCOM
ORDN
NATU
EMIG
IMMI
CENS
PROB
WILL
GRAD
RETI
EVEN
BRIT


//INDIVIDUALATTRIBUTESTRUCTURE
CAST
DSCR
EDUC
IDNO
NATI
NCHI
NMR
OCCU
PROP
RELI
RESI
SSN
-----TITL

//LDSINDIVIDUALORDINANCE
BAPL
CONL
ENDL
SLGC

//Autres Tag (de Heredis...)
PURC
ALIV
MILI
SALE
TRIP
-------INHU
}



  Reset;
end;

destructor TCoherenceDate.Destroy;
begin
  MorePbEventIndi.Free;
  Naissance.free;
  Deces.free;
  AutresEventInd.free;
  Mariages.free;
  AutresEventUnion.free;
  NaissancesEnfants.free;
  ListEventNotTestedWithDeces.free;

  inherited;
end;

function TCoherenceDate.FindInfoDate(List: TObjectList; aID: integer): TInfoDate;
var
  n: integer;
begin
  result := nil;
  for n := 0 to List.Count - 1 do
    begin
      if TInfoDate(List[n]).ID = aID then
        begin
          result := TInfoDate(List[n]);
          break;
        end;
    end;
end;

function TCoherenceDate.MaxDate(List: TObjectList; var aDate: TDateTime): boolean;
var
  n: integer;
begin
  result := false;
  for n := 0 to List.Count - 1 do
    begin
      if TInfoDate(List[n]).KnowDate then
        begin
          if result = false then
            begin
              aDate := TInfoDate(List[n]).iDate;
              result := true;
            end
          else
            begin
              if aDate < TInfoDate(List[n]).iDate then aDate := TInfoDate(List[n]).iDate;
            end;
        end;
    end;
end;

function TCoherenceDate.MinDate(List: TObjectList; var aDate: TDateTime): boolean;
var
  n: integer;
begin
  result := false;
  for n := 0 to List.Count - 1 do
    begin
      if TInfoDate(List[n]).KnowDate then
        begin
          if result = false then
            begin
              aDate := TInfoDate(List[n]).iDate;
              result := true;
            end
          else
            begin
              if aDate > TInfoDate(List[n]).iDate then aDate := TInfoDate(List[n]).iDate;
            end;
        end;
    end;
end;

procedure TCoherenceDate.Reset;
begin
  Naissance.Reset;
  Deces.Reset;
  AutresEventInd.Clear;
  Mariages.Clear;
  AutresEventUnion.Clear;
  NaissancesEnfants.Clear;
  MorePbEventIndi.Clear;
end;

procedure TCoherenceDate.Test;
var
  n: integer;
  i: integer;
  d: TDateTime;
  fShouldShowInfos: boolean;
begin
  for n := 1 to _MAX_PB do Pb[n] := false;
  MorePbEventIndi.Clear;


  if _Context.CoherenceActive then
    begin
      // 1-n avant dcs
      if Naissance.KnowDate and Deces.KnowDate then
        begin
          Pb[1] := Naissance.iDate > Deces.iDate;
        end;


      // 2-Esprence de vie
      if _Context.CheckEsperanceVie then
        begin
          if Naissance.KnowDate and Deces.KnowDate then
            begin
              i := trunc((Deces.iDate - Naissance.iDate) / 365.25);
              if Sexe = 1 then Pb[2] := i > _Context.AgeMaxiAuDecesHommes
              else
                if Sexe = 2 then Pb[2] := i > _Context.AgeMaxiAuDecesFemmes;
            end;
        end;


      // 3-Age minimum pour se marier
      if _Context.CheckAgeMiniMariage then
        begin
          if Naissance.KnowDate then
            begin
              if MinDate(Mariages, d) then
                begin
                  i := trunc((d - Naissance.iDate) / 365.25);
                  if Sexe = 1 then Pb[3] := i < _Context.AgeMiniMariageHommes
                  else
                    if Sexe = 2 then Pb[3] := i < _Context.AgeMiniMariageFemmes;
                end;
            end;
        end;


      // 4-Age maximum pour se marier
      if _Context.CheckAgeMaxiMariage then
        begin
          if Naissance.KnowDate then
            begin
              if MaxDate(Mariages, d) then
                begin
                  i := trunc((d - Naissance.iDate) / 365.25);
                  if Sexe = 1 then Pb[4] := i > _Context.AgeMaxiMariageHommes
                  else
                    if Sexe = 2 then Pb[4] := i > _Context.AgeMaxiMariageFemmes;
                end;
            end;
        end;


      // 5-Age minimum pour avoir un enfant
      if _Context.CheckAgeMiniNaissanceEnfant then
        begin
          if Naissance.KnowDate then
            begin
              if MinDate(NaissancesEnfants, d) then
                begin
                  i := trunc((d - Naissance.iDate) / 365.25);
                  if Sexe = 1 then Pb[5] := i < _Context.AgeMiniNaissanceEnfantHommes
                  else
                    if Sexe = 2 then Pb[5] := i < _Context.AgeMiniNaissanceEnfantFemmes;
                end;
            end;
        end;


      // 6-Age maximum pour avoir un enfant
      if _Context.CheckAgeMaxiNaissanceEnfant then
        begin
          if Naissance.KnowDate then
            begin
              if MaxDate(NaissancesEnfants, d) then
                begin
                  i := trunc((d - Naissance.iDate) / 365.25);
                  if Sexe = 1 then Pb[6] := i > _Context.AgeMaxiNaissanceEnfantHommes
                  else
                    if Sexe = 2 then Pb[6] := i > _Context.AgeMaxiNaissanceEnfantFemmes;
                end;
            end;
        end;


      // 7-Ecart d'age maximum entre 2 poux
      if _Context.CheckEcartAgeEpoux then
        begin

        end;

      {
      //Erreurs possibles sur les autres event indi
      for n:=0 to AutresEventInd.Count-1 do
      begin
       if TInfoDate(AutresEventInd).KnowDate then
           begin
            if Naissance.KnowDate then
                begin
                     if (TInfoDate(AutresEventInd).iDate<Naissance.iDate)
                     then MorePbEventIndi.Add(TInfoDate(AutresEventInd).Infos+' - avant la naissance');
                end;
           end;


           MorePbEventIndi

      end;
      }
      {
      // -> pour tous les vnements ind et fam, sauf naissance/mort/mariage

      //    si on connait la date de naissance, on regarde si d>=date_naissance
          si on connait la date du dcs, on regarde si d<=date_dces

       -> pour tous les enfants

          si on connait la date du dcs, on regarde si date_naissance<=(d+9 mois)
      }

      //on test tous les autres vnements individu
      for n := 0 to AutresEventInd.Count - 1 do
        begin
          if TInfoDate(AutresEventInd[n]).KnowDate then
            begin
              //On test par rapport  la naissance
              if Naissance.KnowDate then
                begin
                  if TInfoDate(AutresEventInd[n]).iDate < Naissance.iDate then
                    begin
                      MorePbEventIndi.Add(TInfoDate(AutresEventInd[n]).LongEventName + '  <->  ' + Naissance.LongEventName);
                    end;
                end;


              //On test par rapport au dcs
              if ListEventNotTestedWithDeces.IndexOf(TInfoDate(AutresEventInd[n]).Infos) = -1 then
                begin
                  if Deces.KnowDate then
                    begin
                      if TInfoDate(AutresEventInd[n]).iDate > Deces.iDate then
                        begin
                          MorePbEventIndi.Add(TInfoDate(AutresEventInd[n]).LongEventName + '  <->  ' + Deces.LongEventName);
                        end;
                    end;
                end;
            end;
        end;




      //on test tous les autres vnements union
      for n := 0 to AutresEventUnion.Count - 1 do
        begin
          if TInfoDate(AutresEventUnion[n]).KnowDate then
            begin
              //On test par rapport  la naissance
              if Naissance.KnowDate then
                begin
                  if TInfoDate(AutresEventUnion[n]).iDate < Naissance.iDate then
                    begin
                      MorePbEventIndi.Add(TInfoDate(AutresEventUnion[n]).LongEventName + '  <->  ' + Naissance.LongEventName);
                    end;
                end;

              //On test par rapport au dcs
              if ListEventNotTestedWithDeces.IndexOf(TInfoDate(AutresEventUnion[n]).Infos) = -1 then
                begin
                  if Deces.KnowDate then
                    begin
                      if TInfoDate(AutresEventUnion[n]).iDate > Deces.iDate then
                        begin
                          MorePbEventIndi.Add(TInfoDate(AutresEventUnion[n]).LongEventName + '  <->  ' + Deces.LongEventName);
                        end;
                    end;
                end;
            end;
        end;


    end;



  fShouldShowInfos := false;
  for n := 1 to _MAX_PB do
    if Pb[n] then fShouldShowInfos := true;

  if MorePbEventIndi.Count > 0 then fShouldShowInfos := true;


  if fShouldShowInfos then
    begin
      if _Context.CanShowWindowPbCoherence then
        begin
          if fFormCoherence <> nil then
            begin
              TFCoherence(fFormCoherence).Refresh;
              TFCoherence(fFormCoherence).Show;
              TFCoherence(fFormCoherence).BringToFront;
            end;
        end;
      if fFormIndividu <> nil then TFIndividu(fFormIndividu).btnPbCoherence.visible := true;
    end
  else
    begin
      if fFormCoherence <> nil then
        begin
          TFCoherence(fFormCoherence).Hide;
        end;
      if fFormIndividu <> nil then TFIndividu(fFormIndividu).btnPbCoherence.visible := false;
    end;
end;


initialization
  GedDateCoherence := TGedcomDate.Create;
  GedDateCoherence.AssignTokens;

finalization
  GedDateCoherence.Free;

end.
